package de.desy.mst.common.util.autoclient;

import de.desy.mst.libs.util.helper.LogProvider;
import de.desy.tine.client.TLink;
import de.desy.tine.dataUtils.TDataType;
import de.desy.tine.definitions.TAccess;
import de.desy.tine.definitions.TErrorList;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.InetAddress;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.logging.Logger;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;

/**
 * ClientPanel.java
 * 
 * Created on July 17, 2008, 5:10 PM
 * 
 * @author herb
 * @author <a href="marcus(dot)walla(at)desy(dot)de">walla</a>
 */
public class ClientPanel extends javax.swing.JPanel {
	private static final long serialVersionUID = -446709653436245712L;

	private static final Logger log = LogProvider.getLogger(ClientPanel.class);

	BufferedReader rxStream = null;

	PrintWriter txStream = null;

	private Socket apSocket = null;

	static final int PORT = 3510;

	String rxBuffer;

	String context;

	String apServer = null; // autoproc: DNS or IP#

	String tineServer = null; // tine server name
	
	EServerType serverType = null;

	String tineBinary = null; // for now not servername

	ServerData serverData = null;

	ServerList tineServerList = null; // list per context

	String connString;

	StringBuffer sbuf;

	// ------ for Magnet Server -------------------
	static final String MAGNETS = "MAGNETS";

	static final String GROUPS = "GROUPS";

	String MagFile = null;

	String MagCmd = null;

	// --------------------------------------------
	boolean initOk = false;

	boolean apConnected = false;

	/** Creates new form ClientPanel */
	public ClientPanel() {
		initComponents();
		initialize();
		apServer = "none"; // "131.169.154.150"; // ?????
	}

	// WALLA - added, more info see here:

	private boolean ping(final String ipAddress) {
		try {
			return InetAddress.getByName(ipAddress).isReachable(5000);
		}
		catch (final UnknownHostException exception) {
			log.severe("unknow host: " + exception.getMessage());
			return false;
		}
		catch (final SecurityException exception) {
			log.severe("security enhanced of pinging the host " + ipAddress + ": " + exception.getMessage());
			return false;
		}
		catch (final IOException exception) {
			log.severe("IO exception on pinging the host " + ipAddress + ": " + exception.getMessage());
			return false;
		}
	}

	void initialize() {
		initContext();
		initOk = true;
		enableApButtons(false);
		enableTineButtons(false);
		jComboBoxContext.setSelectedIndex(0);
	}

	void initContext() {
		Object[] servers = { "", "def" }; // ?? what's this for?
		jListServer.setListData(servers);
		jComboBoxContext.removeAllItems();

		// -----
		ServerList linac2_cdi = new ServerList("LINAC2");
		jComboBoxContext.addItem(linac2_cdi);
		// linac2_cdi.addServer("L2TRIM.CDI","acclxl2lkr1", EServerType.CDI,
		// "");// wuerde alle prozesse stoppen
		linac2_cdi.addServer("L2TRIM.CDI", "acclxl2lkr1", EServerType.CDI_1, "l2trim"); // bisher cdiserv
		
		// Autoproc Connected to acclxl2lkr1
		// l2steer 09.09.13 15:06 ON ON 42.0 1 0 "L2 Steerer Magnets CAN Server"
		// l2trim 09.09.13 15:06 ON ON 2.5 1 0 "L2 MSK Scope Trim CDI"

		linac2_cdi.addServer("L2KICKER.CDI", "acclxl2mdlator1", EServerType.CDI_1, "l2kicker"); // bisher cdiserv
		
		// Autoproc Connected to acclxl2mdlator1
		// l2cor-gtst 02.09.13 13:36 ON ON 37.0 1 0 "L2 Magnets CAN Server"
		// l2kicker 02.09.13 13:36 ON ON 8.0 1 0 "L2 Kicker/Sept CAN Server"

		linac2_cdi.addServer("L2VAC.CDI", "acclxl2vac", EServerType.CDI_1, "l2vac"); // bisher cdiserv
		
		// Autoproc Connected to acclxl2vac
		// l2vac 21.10.13 08:51 ON ON 42.2 1 0 "PC104 CDI Vacuum Server LINAC2"

		// -----
		ServerList regae_cdi = new ServerList("REGAE");
		jComboBoxContext.addItem(regae_cdi);
		regae_cdi.addServer("RGVAC.CDI", "acclxrgmdlator1", EServerType.CDI_1, "rgvac");  // bisher cdiserv
		
		// Autoproc Connected to acclxrgmdlator1
		// rgcormag 10.10.13 09:58 ON ON 21.0 1 0 "Magnet PS CAN Server"
		// rgvac 10.10.13 09:58 ON ON 38.0 1 0 "REGAE VACUUM CDI Server"

		// -----
		ServerList amtf_cdi = new ServerList("AMTF");
		jComboBoxContext.addItem(amtf_cdi);
		amtf_cdi.addServer("AMVAC.CDI", "acclxamvac", EServerType.CDI_1, "amvac");  // bisher cdiserv
		
		// -----
		ServerList desy2_cdi = new ServerList("DESY2");
		jComboBoxContext.addItem(desy2_cdi);
		desy2_cdi.addServer("D2KICKER.CDI", "acclxd2ser1", EServerType.CDI_1, "d2kicker");  // bisher cdiserv
		// Autoproc Connected to acclxd2ser1
		// d2cormag 21.08.13 09:56 ON ON 22.5 1 0 "Magnet PS CAN Server"
		// d2kicker 21.08.13 09:56 ON ON 31.0 1 0 "D2 KICKER CDI Server"

		desy2_cdi.addServer("D2VAC.CDI", "acclxd2vac", EServerType.CDI_1, "d2vac");  // bisher cdiserv
		
		// Autoproc Connected to acclxd2vac
		// d2vac 14.08.13 12:46 ON ON 18.1 1 0 "PC104 CDI Vacuum Server DESY2"

		// -----
		ServerList petra_cdi = new ServerList("PETRA");
		jComboBoxContext.addItem(petra_cdi);
		petra_cdi.addServer("PEKICK-SO.CDI", "acclxpekick-so1", EServerType.CDI_1, "pekick-so"); // bisher cdiserv
		petra_cdi.addServer("PEKICK-SW.CDI", "acclxpekick-sw1", EServerType.CDI_1, "pekick-sw"); // bisher cdiserv
		petra_cdi.addServer("PEKICK-W.CDI", "acclxpekick-w1", EServerType.CDI_1, "pekick-w");  // bisher cdiserv
		petra_cdi.addServer("PENEG-SO.CDI", "acclxpevac-so1", EServerType.CDI_1, "peneg-so");  // bisher cdiserv
		petra_cdi.addServer("PEVAC-SO.CDI", "acclxpevac-so1", EServerType.CDI_2, "pevac-so");  // bisher cdiserv
		petra_cdi.addServer("PENEG-SR.CDI", "acclxpevac-sr1", EServerType.CDI_1, "peneg-sr");  // bisher cdiserv
		petra_cdi.addServer("PEVAC-SR.CDI", "acclxpevac-sr1", EServerType.CDI_2, "pevac-sr");  // bisher cdiserv
		petra_cdi.addServer("PENEG-SW.CDI", "acclxpevac-sw1", EServerType.CDI_1, "peneg-sw");  // bisher cdiserv
		petra_cdi.addServer("PEVAC-SW.CDI", "acclxpevac-sw1", EServerType.CDI_2, "pevac-sw");  // bisher cdiserv
		petra_cdi.addServer("PENEG-W.CDI", "acclxpevac-w1", EServerType.CDI_1, "peneg-w");  // bisher cdiserv
		petra_cdi.addServer("PEVAC-W.CDI", "acclxpevac-w1", EServerType.CDI_2, "pevac-w");  // bisher cdiserv
		petra_cdi.addServer("PENEG-NW.CDI", "acclxpevac-nw1", EServerType.CDI_1, "peneg-nw");  // bisher cdiserv
		petra_cdi.addServer("PEVAC-NW.CDI", "acclxpevac-nw1", EServerType.CDI_2, "pevac-nw");  // bisher cdiserv
		petra_cdi.addServer("PENEG-NR.CDI", "acclxpevac-nr1", EServerType.CDI_1, "peneg-nr");  // bisher cdiserv
		petra_cdi.addServer("PEVAC-NR.CDI", "acclxpevac-nr1", EServerType.CDI_2, "pevac-nr");  // bisher cdiserv
		petra_cdi.addServer("PENEG-NO.CDI", "acclxpevac-no1", EServerType.CDI_1, "peneg-no");  // bisher cdiserv
		petra_cdi.addServer("PEVAC-NO.CDI", "acclxpevac-no1", EServerType.CDI_2, "pevac-no");  // bisher cdiserv
		petra_cdi.addServer("PENEG-O.CDI", "acclxpevac-o1", EServerType.CDI_1, "peneg-o");  // bisher cdiserv
		petra_cdi.addServer("PEVAC-O.CDI", "acclxpevac-o1", EServerType.CDI_2, "pevac-o");  // bisher cdiserv
		
		// -----
		ServerList petra_test = new ServerList("PETRA_TEST");
		jComboBoxContext.addItem(petra_test);
		petra_test.addServer("PEKICK-TST.CDI", "acclxcstdot", EServerType.CDI_1, "pekick-tst");  // bisher cdiserv
		ServerList test = new ServerList("TEST");

		// jComboBoxContext.addItem(test);
		// test.addServer("L2KICKTST.CDI","acclxl2mdlator1","cdiserv");
		// -----

		ServerList xfel_cdi = new ServerList("XFEL");
		jComboBoxContext.addItem(xfel_cdi);
		xfel_cdi.addServer("XFINVAC.CDI", "acclxxfinvac", EServerType.CDI_1, "xfinvac"); // xfinvac

		// - create context TEST
		ServerList test_cdi = new ServerList("TEST");
		jComboBoxContext.addItem(test_cdi);
		test_cdi.addServer("TEST-MSK.CDI", "131.169.128.129", EServerType.CDI_1, "test");

		// -----
		jComboBoxGetM.removeAllItems();
		jComboBoxGetM.addItem(MAGNETS);
		jComboBoxGetM.addItem(GROUPS);
	}

	void setServers() {
		tineServerList = (ServerList) jComboBoxContext.getSelectedItem();
		jListServer.setListData(tineServerList.getServerList()); // Object[]
	}

	String dnsName(String tineServer) {
		return tineServerList.getHostName(tineServer);
	}

	// ------- METHODS FOR AUTOPROC -------------------------------------
	void connectApServer() {
		if (apConnected) {
			jTextAreaAp.setText("Already Connected to: " + apServer);
			return;
		}
		else if (apServer == null) {
			jTextAreaAp.setText("No Autoproc Server chosen");
			return;
		}

		SwingUtilities.invokeLater(new Runnable() {
			@Override
			public void run() {
				jTextAreaAp.setText("Trying to connect to host " + apServer + " ...\n");
			}
		});

		new Thread(new Runnable() {
			@Override
			public void run() {
				if (!ping(apServer))
					ClientPanel.this.setConnectionString("Host unreachable " + apServer + "\n");
				else {
					try {
						apSocket = new Socket(apServer, PORT);
						rxStream = new BufferedReader(new InputStreamReader(apSocket.getInputStream()));
						txStream = new PrintWriter(apSocket.getOutputStream(), true);
						connString = "Autoproc Connected to " + apServer + "\n";
						apConnected = true;
						enableApButtons(true);
						jComboBoxContext.setEnabled(false);
						jListServer.setEnabled(false);
					}
					catch (Exception e) {
						apSocket = null;
						connString = "No Autoproc Connection to " + apServer + "\n";
					}
				}
				SwingUtilities.invokeLater(new Runnable() {
					@Override
					public void run() {
						jTextAreaAp.setText(connString);
					}
				});
			}
		}, "Connection").start();
	}

	private void setConnectionString(final String value) {
		if (value != null && !value.isEmpty())
			this.connString = value;
	}

	void closeApConnection() {
		if (apSocket != null) {
			try {
				rxStream.close();
				txStream.close();
				apSocket.close();
				apSocket = null;
			}
			catch (Exception e) {
				apSocket = null;
			}
		}
		jTextAreaAp.setText("");
		connString = "";
		apConnected = false;
		enableApButtons(false);
		jComboBoxContext.setEnabled(true);
		jListServer.setEnabled(true);
	}

	void sendApCommand(String cmd) {
		try {
			txStream.println(cmd); // send to port
			jTextAreaAp.setText(connString + "cmd: " + cmd + "\n"); // listen
			for (;;) {
				rxBuffer = rxStream.readLine();
				if (rxBuffer.equals("."))
					break;
				else {
					jTextAreaAp.append(rxBuffer + "\n");
				}
			}
		}
		catch (Exception e) {
			jTextAreaAp.append("\nCmd Fails: " + e.getMessage() + ", cmd: " + cmd + "\n");
		}
	}

	// ------- TINE FILE TRANSFERS FROM SERVER -----------------------
	void getTineFile(String filename) {
		sbuf = new StringBuffer(32000);
		TDataType din = new TDataType(filename);
		TDataType dout = new TDataType(sbuf);
		TLink lnk = new TLink("/" + context + "/" + tineServer, "SRVLOGFILE", dout, din, TAccess.CA_READ);
		int rc = lnk.execute();
		if (rc != 0) {
			jTextAreaFile.setText("TINE Error: " + TErrorList.getErrorString(rc) + " (" + rc + ")" + '\n');
		}
		else
			jTextAreaFile.setText(sbuf.toString());
	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	@SuppressWarnings("unchecked")
	// <editor-fold defaultstate="collapsed"
	// desc="Generated Code">//GEN-BEGIN:initComponents
	private void initComponents() {

		jButton1 = new javax.swing.JButton();
		jPanel1 = new javax.swing.JPanel();
		jButton3 = new javax.swing.JButton();
		jButton2 = new javax.swing.JButton();
		jButton7 = new javax.swing.JButton();
		jButton12 = new javax.swing.JButton();
		jMenuBar1 = new javax.swing.JMenuBar();
		jMenu1 = new javax.swing.JMenu();
		jMenu2 = new javax.swing.JMenu();
		jScrollPane1 = new javax.swing.JScrollPane();
		jTextAreaFile = new JTextArea() {
			@Override
			public String toString() {
				return this.getText();
			}
		};
		jButtonFecLog = new javax.swing.JButton();
		jScrollPane2 = new javax.swing.JScrollPane();
		jTextAreaAp = new JTextArea() {
			@Override
			public String toString() {
				return this.getText();
			}
		};
		jScrollPane3 = new javax.swing.JScrollPane();
		jListServer = new javax.swing.JList();
		jComboBoxContext = new javax.swing.JComboBox();
		jLabelDns = new javax.swing.JLabel();
		jTabbedPane1 = new javax.swing.JTabbedPane();
		jPanel2 = new javax.swing.JPanel();
		jPanel4 = new javax.swing.JPanel();
		jButtonLink = new javax.swing.JButton();
		jButtonClose = new javax.swing.JButton();
		jButtonPs = new javax.swing.JButton();
		jButtonStat = new javax.swing.JButton();
		jButtonStart = new javax.swing.JButton();
		jButtonStop = new javax.swing.JButton();
		jButtonGetCdi = new javax.swing.JButton();
		jPanel5 = new javax.swing.JPanel();
		jButtonFec = new javax.swing.JButton();
		jButtonCdi = new javax.swing.JButton();
		jButtonGet = new javax.swing.JButton();
		jButtonUrl = new javax.swing.JButton();
		jPanel3 = new javax.swing.JPanel();
		jPanel6 = new javax.swing.JPanel();
		jButtonLinkM = new javax.swing.JButton();
		jButtonCloseM = new javax.swing.JButton();
		jButtonPsM = new javax.swing.JButton();
		jButtonStatM = new javax.swing.JButton();
		jButtonStartM = new javax.swing.JButton();
		jButtonStopM = new javax.swing.JButton();
		jComboBoxGetM = new javax.swing.JComboBox();
		jButtonLdM = new javax.swing.JButton();
		jPanel7 = new javax.swing.JPanel();
		jButtonFecM = new javax.swing.JButton();
		jButtonGetM = new javax.swing.JButton();
		jButtonUrlM = new javax.swing.JButton();

		jButton1.setText("jButton1");

		javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
		jPanel1.setLayout(jPanel1Layout);
		jPanel1Layout.setHorizontalGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING).addGap(0, 100, Short.MAX_VALUE));
		jPanel1Layout.setVerticalGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING).addGap(0, 100, Short.MAX_VALUE));

		jButton3.setText("jButton3");

		jButton2.setText("jButton2");

		jButton7.setText("jButton7");

		jButton12.setText("jButton12");

		jMenu1.setText("File");
		jMenuBar1.add(jMenu1);

		jMenu2.setText("Edit");
		jMenuBar1.add(jMenu2);

		setFont(new java.awt.Font("Tahoma", 1, 11)); // NOI18N

		jTextAreaFile.setColumns(20);
		jTextAreaFile.setRows(5);
		jTextAreaFile.setBorder(javax.swing.BorderFactory.createTitledBorder("View Files"));
		jScrollPane1.setViewportView(jTextAreaFile);

		jButtonFecLog.setText("Fec Log");
		jButtonFecLog.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jButtonFecLogActionPerformed(evt);
			}
		});

		jTextAreaAp.setColumns(20);
		jTextAreaAp.setRows(5);
		jTextAreaAp.setBorder(javax.swing.BorderFactory.createTitledBorder("Autoproc Info"));
		jScrollPane2.setViewportView(jTextAreaAp);

		jListServer.setBorder(javax.swing.BorderFactory.createTitledBorder("Tine Server Name"));
		jListServer.setModel(new javax.swing.AbstractListModel() {
			String[] strings = { "Item 1", "Item 2", "Item 3", "Item 4", "Item 5" };

			public int getSize() {
				return strings.length;
			}

			public Object getElementAt(int i) {
				return strings[i];
			}
		});
		jListServer.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
			public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
				jListServerValueChanged(evt);
			}
		});
		jScrollPane3.setViewportView(jListServer);

		jComboBoxContext.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Item 1", "Item 2", "Item 3", "Item 4" }));
		jComboBoxContext.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jComboBoxContextActionPerformed(evt);
			}
		});

		jLabelDns.setBackground(new java.awt.Color(255, 255, 255));
		jLabelDns.setFont(new java.awt.Font("Tahoma", 1, 12)); // NOI18N
		jLabelDns.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
		jLabelDns.setText("DNS Name");
		jLabelDns.setBorder(javax.swing.BorderFactory.createTitledBorder("DNS Name"));
		jLabelDns.setOpaque(true);

		jPanel2.setLayout(new javax.swing.BoxLayout(jPanel2, javax.swing.BoxLayout.X_AXIS));

		jPanel4.setBorder(javax.swing.BorderFactory.createTitledBorder("Autoproc Commands"));

		jButtonLink.setText("Link");
		jButtonLink.setToolTipText("Open Link to Autoproc server");
		jButtonLink.setMargin(new java.awt.Insets(2, 10, 2, 10));
		jButtonLink.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jButtonLinkActionPerformed(evt);
			}
		});

		jButtonClose.setText("Close");
		jButtonClose.setToolTipText("Close Link to Autoproc server");
		jButtonClose.setMargin(new java.awt.Insets(2, 6, 2, 6));
		jButtonClose.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jButtonCloseActionPerformed(evt);
			}
		});

		jButtonPs.setText("PS");
		jButtonPs.setToolTipText("Process Status from Autoproc");
		jButtonPs.setMargin(new java.awt.Insets(2, 13, 2, 13));
		jButtonPs.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jButtonPsActionPerformed(evt);
			}
		});

		jButtonStat.setText("Stat");
		jButtonStat.setToolTipText("Status from Autoproc");
		jButtonStat.setMargin(new java.awt.Insets(2, 9, 2, 9));
		jButtonStat.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jButtonStatActionPerformed(evt);
			}
		});

		jButtonStart.setText("Start");
		jButtonStart.setToolTipText("Start cdiserv via Autoproc");
		jButtonStart.setMargin(new java.awt.Insets(2, 6, 2, 6));
		jButtonStart.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jButtonStartActionPerformed(evt);
			}
		});

		jButtonStop.setText("Stop");
		jButtonStop.setToolTipText("Stop cdiserv via Autoproc");
		jButtonStop.setMargin(new java.awt.Insets(2, 8, 2, 8));
		jButtonStop.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jButtonStopActionPerformed(evt);
			}
		});

		jButtonGetCdi.setText("Load cdiaddr");
		jButtonGetCdi.setToolTipText("Initiate download of cdiaddr.csv");
		jButtonGetCdi.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jButtonGetCdiActionPerformed(evt);
			}
		});

		javax.swing.GroupLayout jPanel4Layout = new javax.swing.GroupLayout(jPanel4);
		jPanel4.setLayout(jPanel4Layout);
		jPanel4Layout.setHorizontalGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING).addGroup(jPanel4Layout.createSequentialGroup().addContainerGap().addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false).addComponent(jButtonGetCdi, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE).addGroup(javax.swing.GroupLayout.Alignment.LEADING, jPanel4Layout.createSequentialGroup().addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING).addComponent(jButtonLink).addComponent(jButtonPs).addComponent(jButtonStart)).addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED).addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING).addComponent(jButtonStat).addComponent(jButtonClose).addComponent(jButtonStop)))).addContainerGap()));

		jPanel4Layout.linkSize(javax.swing.SwingConstants.HORIZONTAL, new java.awt.Component[] { jButtonLink, jButtonPs, jButtonStart });

		jPanel4Layout.setVerticalGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING).addGroup(jPanel4Layout.createSequentialGroup().addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE).addComponent(jButtonLink).addComponent(jButtonClose)).addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED).addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE).addComponent(jButtonPs).addComponent(jButtonStat)).addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED).addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE).addComponent(jButtonStart).addComponent(jButtonStop)).addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 16, Short.MAX_VALUE).addComponent(jButtonGetCdi).addContainerGap()));

		jPanel2.add(jPanel4);

		jPanel5.setBorder(javax.swing.BorderFactory.createTitledBorder("Tine (View Files)"));

		jButtonFec.setText("fec.log");
		jButtonFec.setToolTipText("fec.log file from server");
		jButtonFec.setMargin(new java.awt.Insets(2, 10, 2, 10));
		jButtonFec.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jButtonFecActionPerformed(evt);
			}
		});

		jButtonCdi.setText("cdi.log");
		jButtonCdi.setToolTipText("cdi.log file from server");
		jButtonCdi.setMargin(new java.awt.Insets(2, 10, 2, 10));
		jButtonCdi.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jButtonCdiActionPerformed(evt);
			}
		});

		jButtonGet.setText("get.log");
		jButtonGet.setToolTipText("log from http download script");
		jButtonGet.setMargin(new java.awt.Insets(2, 10, 2, 10));
		jButtonGet.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jButtonGetActionPerformed(evt);
			}
		});

		jButtonUrl.setText("myurl");
		jButtonUrl.setToolTipText("Http FileServer Address for files");
		jButtonUrl.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jButtonUrlActionPerformed(evt);
			}
		});

		javax.swing.GroupLayout jPanel5Layout = new javax.swing.GroupLayout(jPanel5);
		jPanel5.setLayout(jPanel5Layout);
		jPanel5Layout.setHorizontalGroup(jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING).addGroup(jPanel5Layout.createSequentialGroup().addContainerGap().addGroup(jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING).addComponent(jButtonFec).addComponent(jButtonCdi).addComponent(jButtonGet).addComponent(jButtonUrl)).addContainerGap(43, Short.MAX_VALUE)));

		jPanel5Layout.linkSize(javax.swing.SwingConstants.HORIZONTAL, new java.awt.Component[] { jButtonCdi, jButtonFec, jButtonGet, jButtonUrl });

		jPanel5Layout.setVerticalGroup(jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING).addGroup(jPanel5Layout.createSequentialGroup().addComponent(jButtonFec).addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED).addComponent(jButtonCdi).addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED).addComponent(jButtonGet).addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 16, Short.MAX_VALUE).addComponent(jButtonUrl).addContainerGap()));

		jPanel2.add(jPanel5);

		jTabbedPane1.addTab("CDI", jPanel2);

		jPanel3.setLayout(new javax.swing.BoxLayout(jPanel3, javax.swing.BoxLayout.X_AXIS));

		jPanel6.setBorder(javax.swing.BorderFactory.createTitledBorder("Autoproc Commands"));

		jButtonLinkM.setText("Link");
		jButtonLinkM.setToolTipText("Open Link to Autoproc Server");
		jButtonLinkM.setMargin(new java.awt.Insets(2, 10, 2, 10));
		jButtonLinkM.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jButtonLinkMActionPerformed(evt);
			}
		});

		jButtonCloseM.setText("Close");
		jButtonCloseM.setToolTipText("Close Link to Autoproc Server");
		jButtonCloseM.setMargin(new java.awt.Insets(2, 6, 2, 6));
		jButtonCloseM.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jButtonCloseMActionPerformed(evt);
			}
		});

		jButtonPsM.setText("PS");
		jButtonPsM.setToolTipText("Process Status from Autoproc");
		jButtonPsM.setMargin(new java.awt.Insets(2, 13, 2, 13));
		jButtonPsM.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jButtonPsMActionPerformed(evt);
			}
		});

		jButtonStatM.setText("Stat");
		jButtonStatM.setToolTipText("Status from Autoproc");
		jButtonStatM.setMargin(new java.awt.Insets(2, 9, 2, 9));
		jButtonStatM.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jButtonStatMActionPerformed(evt);
			}
		});

		jButtonStartM.setText("Start");
		jButtonStartM.setToolTipText("Start magserv via Autoproc");
		jButtonStartM.setMargin(new java.awt.Insets(2, 6, 2, 6));
		jButtonStartM.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jButtonStartMActionPerformed(evt);
			}
		});

		jButtonStopM.setText("Stop");
		jButtonStopM.setToolTipText("Stop magserv via Autoproc");
		jButtonStopM.setMargin(new java.awt.Insets(2, 8, 2, 8));
		jButtonStopM.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jButtonStopMActionPerformed(evt);
			}
		});

		jComboBoxGetM.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Item 1", "Item 2", "Item 3", "Item 4" }));
		jComboBoxGetM.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jComboBoxGetMActionPerformed(evt);
			}
		});

		jButtonLdM.setText("Ld");
		jButtonLdM.setToolTipText("Initiate File Download to Magnet Server");
		jButtonLdM.setMargin(new java.awt.Insets(2, 4, 2, 4));
		jButtonLdM.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jButtonLdMActionPerformed(evt);
			}
		});

		javax.swing.GroupLayout jPanel6Layout = new javax.swing.GroupLayout(jPanel6);
		jPanel6.setLayout(jPanel6Layout);
		jPanel6Layout.setHorizontalGroup(jPanel6Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING).addGroup(jPanel6Layout.createSequentialGroup().addContainerGap().addGroup(jPanel6Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING).addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel6Layout.createSequentialGroup().addComponent(jButtonLdM, javax.swing.GroupLayout.DEFAULT_SIZE, 28, Short.MAX_VALUE).addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED).addComponent(jComboBoxGetM, javax.swing.GroupLayout.PREFERRED_SIZE, 83, javax.swing.GroupLayout.PREFERRED_SIZE).addGap(11, 11, 11)).addGroup(jPanel6Layout.createSequentialGroup().addGap(8, 8, 8).addGroup(jPanel6Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING).addGroup(jPanel6Layout.createSequentialGroup().addComponent(jButtonLinkM).addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED).addComponent(jButtonCloseM)).addGroup(jPanel6Layout.createSequentialGroup().addGroup(jPanel6Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING).addComponent(jButtonStartM).addComponent(jButtonPsM)).addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED).addGroup(jPanel6Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING).addComponent(jButtonStatM).addComponent(jButtonStopM)))))).addContainerGap()));
		jPanel6Layout.setVerticalGroup(jPanel6Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING).addGroup(jPanel6Layout.createSequentialGroup().addGroup(jPanel6Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE).addComponent(jButtonLinkM).addComponent(jButtonCloseM)).addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED).addGroup(jPanel6Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE).addComponent(jButtonStatM).addComponent(jButtonPsM)).addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED).addGroup(jPanel6Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE).addComponent(jButtonStopM).addComponent(jButtonStartM)).addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 16, Short.MAX_VALUE).addGroup(jPanel6Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE).addComponent(jButtonLdM).addComponent(jComboBoxGetM, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)).addContainerGap()));

		jPanel3.add(jPanel6);

		jPanel7.setBorder(javax.swing.BorderFactory.createTitledBorder("Tine (view files)"));

		jButtonFecM.setText("fec.log");
		jButtonFecM.setMargin(new java.awt.Insets(2, 8, 2, 8));
		jButtonFecM.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jButtonFecMActionPerformed(evt);
			}
		});

		jButtonGetM.setText("get.log");
		jButtonGetM.setToolTipText("log from file download script on Server");
		jButtonGetM.setMargin(new java.awt.Insets(2, 8, 2, 8));
		jButtonGetM.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jButtonGetMActionPerformed(evt);
			}
		});

		jButtonUrlM.setText("myurl");
		jButtonUrlM.setToolTipText("Http FileServer Address for files");
		jButtonUrlM.setMargin(new java.awt.Insets(2, 6, 2, 6));
		jButtonUrlM.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jButtonUrlMActionPerformed(evt);
			}
		});

		javax.swing.GroupLayout jPanel7Layout = new javax.swing.GroupLayout(jPanel7);
		jPanel7.setLayout(jPanel7Layout);
		jPanel7Layout.setHorizontalGroup(jPanel7Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING).addGroup(jPanel7Layout.createSequentialGroup().addContainerGap().addGroup(jPanel7Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false).addComponent(jButtonFecM).addComponent(jButtonGetM, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE).addComponent(jButtonUrlM, 0, 0, Short.MAX_VALUE)).addContainerGap(24, Short.MAX_VALUE)));

		jPanel7Layout.linkSize(javax.swing.SwingConstants.HORIZONTAL, new java.awt.Component[] { jButtonFecM, jButtonGetM, jButtonUrlM });

		jPanel7Layout.setVerticalGroup(jPanel7Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING).addGroup(jPanel7Layout.createSequentialGroup().addComponent(jButtonFecM).addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED).addComponent(jButtonGetM).addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED).addComponent(jButtonUrlM).addContainerGap(40, Short.MAX_VALUE)));

		jPanel3.add(jPanel7);

		jTabbedPane1.addTab("Mag", jPanel3);

		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
		this.setLayout(layout);
		layout.setHorizontalGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING).addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup().addContainerGap().addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING).addComponent(jScrollPane2, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 416, Short.MAX_VALUE).addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 416, Short.MAX_VALUE).addGroup(javax.swing.GroupLayout.Alignment.LEADING, layout.createSequentialGroup().addComponent(jTabbedPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 257, javax.swing.GroupLayout.PREFERRED_SIZE).addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED).addComponent(jButtonFecLog, javax.swing.GroupLayout.PREFERRED_SIZE, 0, javax.swing.GroupLayout.PREFERRED_SIZE).addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED).addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING).addGroup(layout.createSequentialGroup().addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING).addComponent(jScrollPane3, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 141, Short.MAX_VALUE).addComponent(jComboBoxContext, 0, 141, Short.MAX_VALUE)).addGap(6, 6, 6)).addComponent(jLabelDns, javax.swing.GroupLayout.PREFERRED_SIZE, 137, javax.swing.GroupLayout.PREFERRED_SIZE)))).addContainerGap()));
		layout.setVerticalGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING).addGroup(layout.createSequentialGroup().addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false).addGroup(layout.createSequentialGroup().addGap(30, 30, 30).addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false).addGroup(layout.createSequentialGroup().addComponent(jButtonFecLog).addGap(78, 78, 78)).addGroup(layout.createSequentialGroup().addComponent(jComboBoxContext, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE).addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED).addComponent(jScrollPane3, javax.swing.GroupLayout.PREFERRED_SIZE, 87, javax.swing.GroupLayout.PREFERRED_SIZE))).addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED).addComponent(jLabelDns)).addGroup(layout.createSequentialGroup().addContainerGap().addComponent(jTabbedPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 182, javax.swing.GroupLayout.PREFERRED_SIZE))).addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED).addComponent(jScrollPane2, javax.swing.GroupLayout.PREFERRED_SIZE, 77, javax.swing.GroupLayout.PREFERRED_SIZE).addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED).addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 93, Short.MAX_VALUE).addContainerGap()));
	}// </editor-fold>//GEN-END:initComponents
		// ---------------- ACTION METHODS for CDI SERVERS
		// -----------------------------

	private void enableTineButtons(boolean en) {
		jButtonFec.setEnabled(en);
		jButtonCdi.setEnabled(en);
		jButtonGet.setEnabled(en);
		jButtonUrl.setEnabled(en);
		jButtonLink.setEnabled(en);
		jButtonClose.setEnabled(en);
	}

	private void enableTineMButtons(boolean en) {
		jButtonFecM.setEnabled(en);
		jButtonCdi.setEnabled(en);
		jButtonGetM.setEnabled(en);
		jButtonUrlM.setEnabled(en);
		jButtonLinkM.setEnabled(en);
		jButtonCloseM.setEnabled(en);
	}

	private void enableApButtons(boolean en) {
		jButtonPs.setEnabled(en);
		jButtonStat.setEnabled(en);
		jButtonStart.setEnabled(en);
		jButtonStop.setEnabled(en);
		jButtonGetCdi.setEnabled(en);
	}

	private void enableMApButtons(boolean en) {
		jButtonPsM.setEnabled(en);
		jButtonStatM.setEnabled(en);
		jButtonStartM.setEnabled(en);
		jButtonStopM.setEnabled(en);
		jButtonGetM.setEnabled(en);
	}

	private void jButtonFecLogActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_jButtonFecLogActionPerformed
	// TODO add your handling code here:
		getTineFile("fec.log");
	}// GEN-LAST:event_jButtonFecLogActionPerformed

	private void jComboBoxContextActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_jComboBoxContextActionPerformed
		int jc = jComboBoxContext.getSelectedIndex();
		if (jc < 0 || !initOk)
			return;
		context = jComboBoxContext.getSelectedItem().toString();
		tineServer = null;
		serverType = null;
		enableTineButtons(false);
		enableTineMButtons(false);
		jLabelDns.setText("");
		setServers();
	}// GEN-LAST:event_jComboBoxContextActionPerformed

	private void jListServerValueChanged(javax.swing.event.ListSelectionEvent evt) {// GEN-FIRST:event_jListServerValueChanged
		if (jListServer.getSelectedValue() == null)
			return;// GEN-LAST:event_jListServerValueChanged
		tineServer = jListServer.getSelectedValue().toString();
		tineBinary = tineServerList.getBinName(tineServer);
		serverData = tineServerList.getServerData(tineServer);
		serverType = tineServerList.getServerType(tineServer);
		enableTineButtons(true);
		enableTineMButtons(true);
		apServer = dnsName(tineServer);
		jLabelDns.setText(apServer);
	}

	private boolean serverCheck() {
		if (tineServer == null) {
			jTextAreaAp.setText("First choose a server!");
			return false;
		}
		return true;
	}

	private void jButtonCloseActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_jButtonCloseActionPerformed
	// TODO add your handling code here:
		if (!serverCheck())
			return;
		closeApConnection();
	}// GEN-LAST:event_jButtonCloseActionPerformed

	private void jButtonLinkActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_jButtonLinkActionPerformed
	// TODO add your handling code here:
		if (!serverCheck())
			return;
		connectApServer();
	}// GEN-LAST:event_jButtonLinkActionPerformed

	private void jButtonPsActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_jButtonPsActionPerformed
	// TODO add your handling code here:
		if (!serverCheck())
			return;
		sendApCommand("ps");
	}// GEN-LAST:event_jButtonPsActionPerformed

	private void jButtonStatActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_jButtonStatActionPerformed
	// TODO add your handling code here:
		sendApCommand("st");
	}// GEN-LAST:event_jButtonStatActionPerformed

	private void jButtonStartActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_jButtonStartActionPerformed
	// TODO add your handling code here:
	// if (tineBinary.equals("cdiserv")|| tineBinary.equals("cdiraw"))
	// sendApCommand("start cdiserv");
	// else if (tineBinary.equals("cdiserv2"))
	// sendApCommand("start cdiserv2");
	// else jTextAreaFile.setText("Unallowed binary name: " + tineBinary +
	// "\n");
		sendApCommand("start " + serverData.getBinaryName());
	}// GEN-LAST:event_jButtonStartActionPerformed

	private void jButtonStopActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_jButtonStopActionPerformed
	// TODO add your handling code here
	// if (tineBinary.equals("cdiserv") || tineBinary.equals("cdiraw"))
	// sendApCommand("stop cdiserv");
	// else if (tineBinary.equals("cdiserv2"))
	// sendApCommand("stop cdiserv2");
	// else jTextAreaFile.setText("Unallowed binary name: " + tineBinary +
	// "\n");

		sendApCommand("stop " + serverData.getBinaryName());
	}// GEN-LAST:event_jButtonStopActionPerformed

	private void jButtonGetCdiActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_jButtonGetCdiActionPerformed
            // TODO add your handling code here:
            if (serverData.getServerType() == EServerType.CDI_1)
               sendApCommand("exec cdiget");
            else if (serverData.getServerType() == EServerType.CDI_2)
               sendApCommand("exec cdiget2");
            else 
                jTextAreaAp.setText("Unallowed binary name: " + tineBinary + "\n");
            //sendApCommand("exec cdiget"); // + serverData.getTransportCDIAddressScript());
	}// GEN-LAST:event_jButtonGetCdiActionPerformed

	private void jButtonCdiActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_jButtonCdiActionPerformed
	// TODO add your handling code here:
		getTineFile("cdi.log");
	}// GEN-LAST:event_jButtonCdiActionPerformed

	private void jButtonFecActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_jButtonFecActionPerformed
	// TODO add your handling code here:
		getTineFile("fec.log");
	}// GEN-LAST:event_jButtonFecActionPerformed

	private void jButtonGetActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_jButtonGetActionPerformed
	// TODO add your handling code here:
		getTineFile("cdiget.log");
	}// GEN-LAST:event_jButtonGetActionPerformed

	// ------------- ACTION METHODS for MAGNET SERVERS
	// -----------------------------

	private void jButtonLinkMActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_jButtonLinkMActionPerformed
	// TODO add your handling code here:
		connectApServer();// GEN-LAST:event_jButtonLinkMActionPerformed
	}

	private void jButtonCloseMActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_jButtonCloseMActionPerformed
	// TODO add your handling code here:
		closeApConnection();// GEN-LAST:event_jButtonCloseMActionPerformed
	}

	private void jButtonPsMActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_jButtonPsMActionPerformed
	// TODO add your handling code here:
		sendApCommand("ps");// GEN-LAST:event_jButtonPsMActionPerformed
	}

	private void jButtonStatMActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_jButtonStatMActionPerformed
	// TODO add your handling code here:
		sendApCommand("st");// GEN-LAST:event_jButtonStatMActionPerformed
	}

	private void jButtonStartMActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_jButtonStartMActionPerformed
	// TODO add your handling code here:
		sendApCommand("start magserv");// GEN-LAST:event_jButtonStartMActionPerformed
	}

	private void jButtonStopMActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_jButtonStopMActionPerformed
	// TODO add your handling code here:
		sendApCommand("stop magserv");// GEN-LAST:event_jButtonStopMActionPerformed
	}

	private void jButtonLdMActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_jButtonLdMActionPerformed
	// TODO add your handling code here:
		if (MagCmd != null)
			sendApCommand(MagCmd); // ???? IS THIS RIGHT ??
	}// GEN-LAST:event_jButtonLdMActionPerformed

	private void jComboBoxGetMActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_jComboBoxGetMActionPerformed
		if (jComboBoxGetM.getSelectedIndex() < 0)
			return;
		MagFile = jComboBoxGetM.getSelectedItem().toString();
		if (MagFile.equals(MAGNETS))
			MagCmd = "exec magget";
		else if (MagFile.equals(GROUPS))
			MagCmd = "exec grpget";
		else
			MagFile = null;
	}// GEN-LAST:event_jComboBoxGetMActionPerformed

	private void jButtonFecMActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_jButtonFecMActionPerformed
	// TODO add your handling code here:
		getTineFile("fec.log");
	}// GEN-LAST:event_jButtonFecMActionPerformed

	private void jButtonGetMActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_jButtonGetMActionPerformed
	// TODO add your handling code here:
		getTineFile("cdiget.log");
	}// GEN-LAST:event_jButtonGetMActionPerformed

	// - temporary solution with hard-coded binary names 07.11.08

	private void jButtonUrlActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_jButtonUrlActionPerformed
	// TODO add your handling code here:
		if (serverType != null && serverType == EServerType.CDI_1) // "cdiserv"
			getTineFile("/export/tine/server/cdiserv/database/myurl");
		else if (serverType != null && serverType == EServerType.CDI_2) // "cdiserv2"
			getTineFile("/export/tine/server/cdiserv2/database/myurl");
		else
			jTextAreaFile.setText("Unallowed binary name: " + tineBinary + ", server type=" + serverType + "\n");
	}

	/*
	 * this code was set to controlled! why?
	 * }                                          
	 */

	private void jButtonUrlMActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_jButtonUrlMActionPerformed
		getTineFile("/export/tine/server/magserv/database/myurl");
	}

	/*
	 * this code was set to controlled! why?
	 * }                                           
	 */

	// Variables declaration - do not modify//GEN-BEGIN:variables
	private javax.swing.JButton jButton1;
	private javax.swing.JButton jButton12;
	private javax.swing.JButton jButton2;
	private javax.swing.JButton jButton3;
	private javax.swing.JButton jButton7;
	private javax.swing.JButton jButtonCdi;
	private javax.swing.JButton jButtonStart;
	private javax.swing.JButton jButtonClose;
	private javax.swing.JButton jButtonCloseM;
	private javax.swing.JButton jButtonFec;
	private javax.swing.JButton jButtonFecLog;
	private javax.swing.JButton jButtonFecM;
	private javax.swing.JButton jButtonGet;
	private javax.swing.JButton jButtonGetCdi;
	private javax.swing.JButton jButtonGetM;
	private javax.swing.JButton jButtonLdM;
	private javax.swing.JButton jButtonLink;
	private javax.swing.JButton jButtonLinkM;
	private javax.swing.JButton jButtonPs;
	private javax.swing.JButton jButtonPsM;
	private javax.swing.JButton jButtonStartM;
	private javax.swing.JButton jButtonStat;
	private javax.swing.JButton jButtonStatM;
	private javax.swing.JButton jButtonStop;
	private javax.swing.JButton jButtonStopM;
	private javax.swing.JButton jButtonUrl;
	private javax.swing.JButton jButtonUrlM;
	private javax.swing.JComboBox jComboBoxContext;
	private javax.swing.JComboBox jComboBoxGetM;
	private javax.swing.JLabel jLabelDns;
	private javax.swing.JList jListServer;
	private javax.swing.JMenu jMenu1;
	private javax.swing.JMenu jMenu2;
	private javax.swing.JMenuBar jMenuBar1;
	private javax.swing.JPanel jPanel1;
	private javax.swing.JPanel jPanel2;
	private javax.swing.JPanel jPanel3;
	private javax.swing.JPanel jPanel4;
	private javax.swing.JPanel jPanel5;
	private javax.swing.JPanel jPanel6;
	private javax.swing.JPanel jPanel7;
	private javax.swing.JScrollPane jScrollPane1;
	private javax.swing.JScrollPane jScrollPane2;
	private javax.swing.JScrollPane jScrollPane3;
	private javax.swing.JTabbedPane jTabbedPane1;
	private javax.swing.JTextArea jTextAreaAp;
	private javax.swing.JTextArea jTextAreaFile;
	// End of variables declaration//GEN-END:variables
}
