/*
 *
 * File: IEData.java
 * Date: Feb 2, 2007; 1:09:45 PM
 */
package de.desy.mst.libs.accelerator;

/**
 * <p>Each panel has its enumeration type which derivates from this
 * interface.</p>
 * <p>Remember that there also exists different other interfaces which allow
 * to set special value for each TINE data property. Each of this interface
 * starts with a {@code IEData...} with a special name, which describes the
 * content. Currently there are interfaces for setting the range
 * ({@link IEDataRange}), the send and receive factor ({@link IEDataFactor}),
 * for meta values ({@link IEDataMetaValue} or {@link IEDataMetaDataType}), 
 * sending values with the output data ({@link IEDataSendNumber}), sending 
 * values with the output data but here returned as a TINE data type 
 * ({@link IEDataSendDataType}), finding out its crate address 
 * ({@link IEDataAddress}), sending the multiplexer channel of Linac2 RF 
 * ({@link IEDataMpxChannel}), and for setting all of these values to the data 
 * instance ({@link IEDataSetValues}).</p>
 *
 * @author <a href="marcus.walla(at)desy.de">walla</a>
 * @version $Id: IEData.java 22470 2009-03-05 16:49:54Z walla $
 */
public interface IEData extends IEEnum
{
	/**
	 * <p>Returns a logical value if the data component is adjusting its parameter
	 * just by dragging it around or not. With the word <b>drag</b> I mean that
	 * you take the component and drag it with the mouse button pressed down and
	 * push it to another location.</p>
	 *
	 * @return Returns the logical value if this data component is dragged or
	 * 		not. Most of the time this will return a {@code false} value while
	 * 		it can also return on some data components a {@code true} value
	 * 		(for example with a slider component). You can also ignore this
	 * 		value, which is in most of the cases okay!
	 */
	public abstract boolean isAdjusting();

	/**
	 * <p>Normally all elements of these enumeration interface will fire an event
	 * to modified it in the CDI server and therefore in the hardware and not
	 * only on screen. This is currently done if this method returns a
	 * {@code true} value. If not it will return a {@code false}.</p>
	 *
	 * @return Returns a {@code true} if the value should be set on hardware,
	 * 		otherwise a {@code false} is returned.
	 */
	public abstract boolean isFireEvent();

	/**
	 * <p>Tells the caller if this data element is part of a CDI server or not.
	 * </p>
	 *
	 * @return Returns {@code true} if it is part of a CDI server otherwise a
	 * 		{@code false} is given back.
	 */
	public abstract boolean isCDI();

	/**
	 * Returns the size of values each data component can get. Normally this method
	 * will response with 1 as a result. But there are also other values possible
	 * for returning. For example the temperature of the PETRA 3 accelerator ring
	 * will return 8, because each device contains 8 temperature measurement
	 * values per device.
	 *
	 * @return Returns the number of values per property.
	 */
	public abstract int size();


}
/* end of file */