/*
 *
 * File: EventServerDataExchange.java
 * Date: Aug 14, 2007; 4:14:33 PM
 */
package de.desy.mst.libs.accelerator.eventserver;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Properties;

import de.desy.mst.libs.accelerator.Utility;
import de.desy.mst.libs.accelerator.exception.InvalidKeyException;
import de.desy.mst.libs.framework.mstapp.logging.Logger;

/**
 * This is the event server class.
 *
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: EventServerDataExchange.java 23534 2009-03-27 17:00:15Z walla $
 */
public class EventServerDataExchange
{
	/** Is an instance of the property map for different entries. */
	private static Properties _map;

	/**
	 * Will get the property string. Associates a map.
	 *
	 * <p><b><u>CAUTION:</u></b> This static method doesn't exit the application
	 * in a correct way (I call the <tt>System.exit(-1)</tt>). If there exist a
	 * better way to exit the program you should change it here.</p>
	 *
	 * @param propertyString_ Is the key of the property.
	 * @return Is the value of the property.
	 */
	private static final String getProperty(final String propertyString_)
	{
		if (_map == null)
		{
			try
			{
				_map = new Properties();
				_map.load(new Object().getClass().getResourceAsStream(
						"/resources/properties/bitvalue.properties"));
			}
			catch(FileNotFoundException exception)
			{
				Logger.SWApp.error("The current property file of bit values cannot" +
						" be found.", exception);
				Utility.fatalErrorMessage("Property-Datei wurde nicht gefunden",
						"Bei der Initialisierung des Programms konnte die ntige<br>" +
						"Initialisierungsdatei der Bitwerte nicht gefunden werden!", 
						exception);
				System.exit(-1);
			}
			catch(IOException exception)
			{
				Logger.SWApp.error("An IO exception has been thrown on reading the" +
						" property file of EventServerDataExchange on RF.", exception);
				Utility.fatalErrorMessage("Ein-/Ausgabe Ausnahme wurde geworfen",
						"Bei der Initialisierung des Programms wurde eine I/O-Ausnahme<br>" +
						"geworfen!",
						exception);
				System.exit(-1);
			}

		}

		if (_map.containsKey(propertyString_))
			return _map.getProperty(propertyString_);
		throw new InvalidKeyException("The corresponding key of the bit value" +
				" is invalid.");
	}

	/**
	 * <p>Queries if the corresponding value of <tt>propertyString_</tt> is set
	 * in the property file of this application. This calls the method of
	 * {@link EventServerDataExchange#getProperty(String)}.</p>
	 *
	 * @param propertyString_ Is the string value you are querying for.
	 * @return Returns the current string value, if the corresponding value
	 * 		exists.
	 */
	public static final String getValue(final String propertyString_)
	{
		return getProperty(propertyString_);
	}
}
/* end of file */