/**
 * File: DeprecatedException.java
 * Date: Feb 8, 2010
 */
package de.desy.mst.libs.accelerator.exception;

/**
 * This exception is thrown in a constructor or method which is declared as
 * deprecated and will be removed in future versions.  
 *
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: DeprecatedException.java 32897 2010-02-08 15:46:15Z walla $
 */
public class DeprecatedException extends RuntimeException
{
	private static final long serialVersionUID = -2320005129440723056L;

	// --- CONSTRUCTORS -------------------------------------------------------- 
	
	/**
	 * Constructor.
	 * 
	 * @param clazz_ Calling clazz name.
	 */
	public DeprecatedException(final String clazz_)
	{
		super(checkClazz(clazz_));
	}

	/**
	 * Constructor.
	 * 
	 * @param clazz_ Calling clazz name.
	 * @param message_ Message.
	 */
	public DeprecatedException(final String clazz_, final String message_)
	{
		super(checkClazz(clazz_, message_));
	}

	/**
	 * Constructor.
	 * 
	 * @param clazz_ Calling clazz name.
	 * @param cause_ Cause.
	 */
	public DeprecatedException(final String clazz_, final Throwable cause_)
	{
		super(checkClazz(clazz_), cause_);
	}

	/**
	 * Constructor.
	 * 
	 * @param clazz_ Calling clazz name.
	 * @param message_ Message.
	 * @param cause_ Cause.
	 */
	public DeprecatedException(final String clazz_, final String message_, final Throwable cause_)
	{
		super(checkClazz(clazz_, message_), cause_);
	}
	
	private static final String STR_DEPRECATED = "class/method is deprecated";
	
	private static String checkClazz(final String clazz_)
	{
		return (clazz_!=null)?(clazz_ + " - " + STR_DEPRECATED):STR_DEPRECATED;
	}

	private static String checkClazz(final String clazz_, final String message_)
	{
		return (clazz_!=null)?(clazz_ + " - " + STR_DEPRECATED + " - " + message_):STR_DEPRECATED + " - " + message_;
	}
}
/* end of file */