/*
 *
 * File: EnumerationNotUsedException.java
 * Date: Oct 14, 2007; 3:26:15 PM
 */
package de.desy.mst.libs.accelerator.exception;


/**
 * This exception is thrown if the enumeration indeed exists but is not used
 * by the method.
 * 
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: EnumerationNotUsedException.java 34946 2010-04-12 08:33:15Z walla $
 */
public class EnumerationNotUsedException extends RuntimeException
{
	// --- CLASS FIELDS --------------------------------------------------------
	
	/** Is the unique serial version UID */
	final static long serialVersionUID = 1092348905403504238l;
	
	// --- INSTANCE FIELDS -----------------------------------------------------
	
	private final Enum<?> _enum;
	
	// --- CONSTRUCTORS --------------------------------------------------------
	
	/**
	 * Creates a new exception of a not used enumeration. This is thrown if
	 * the exception indeed exists but is not used.
	 */
	public EnumerationNotUsedException()
	{
		_enum = null;
	}
	
	/**
	 * Constructor.
	 * 
	 * @param enum_ The not used enumeration.
	 */
	public EnumerationNotUsedException(final Enum<?> enum_)
	{
		super("Enumeration indeed exist but is not used - " + enum_);
		_enum = enum_;
	}

	/**
	 * Creates a new exception of a not used enumeration. This is thrown if
	 * the exception indeed exists but is not used.
	 * 
	 * @param message_ Sets the message as its description.
	 */
	public EnumerationNotUsedException(final String message_)
	{
		super(message_);
		_enum = null;
	}

	/**
	 * Creates a new exception of a not used enumeration. This is thrown if
	 * the exception indeed exists but is not used.
	 *
	 * @param cause_ Is the cause of this exception.
	 */
	public EnumerationNotUsedException(final Throwable cause_)
	{
		super(cause_);
		_enum = null;
	}

	/**
	 * Creates a new exception of a not used enumeration. This is thrown if
	 * the exception indeed exists but is not used.
	 * 
	 * @param message_ Sets the message as its description.
	 * @param cause_ Is the cause of this exception.
	 */
	public EnumerationNotUsedException(final String message_, 
			final Throwable cause_)
	{
		super(message_, cause_);
		_enum = null;
	}
	
	// --- INSTANCE METHODS ----------------------------------------------------
	
	public enum EUnknown { UNKNOWN };

	/**
	 * Returns the enumeration instance. When you don't have initialized this
	 * class with an enumeration instance then the {@link EUnknown#UNKNOWN} is 
	 * returned.
	 * 
	 * @return Enumeration instance.
	 */
	public Enum<?> getEnum()
	{
		return (_enum != null) ? _enum : EUnknown.UNKNOWN;
	}
}
/* end of file */