/*
 *
 * File: IllegalChannelException.java
 * Creation date: May 30, 2008 - 3:21:03 PM
 */
package de.desy.mst.libs.accelerator.exception;

/**
 * Is thrown if you want to set or get a wrong channel number of a multiplexer.
 *
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: IllegalChannelException.java 12126 2008-05-30 16:42:25Z walla $
 */
public final class IllegalChannelException extends Exception
{
	/** Is the serial version UID to unify this class. */
	public static final long serialVersionUID = 3421059591934863439l;
	
	/** The current illegal channel number. */
	private int _channel;
	
	/**
	 * Constructs an illegal channel exception.
	 * 
	 * @param channel_ Illegal channel number.
	 */
	public IllegalChannelException(final int channel_)
	{
		super();
		_channel = channel_;
	}

	/**
	 * Constructs an illegal channel exception.
	 * 
	 * @param channel_ Illegal channel number.
	 * @param message_ Is the message.
	 */
	public IllegalChannelException(final int channel_, final String message_)
	{
		super(message_);
		_channel = channel_;
	}

	/**
	 * Constructs an illegal channel exception.
	 * 
	 * @param channel_ Illegal channel number.
	 * @param cause_ Is the throwable cause of this exception.
	 */
	public IllegalChannelException(final int channel_, final Throwable cause_)
	{
		super(cause_);
		_channel = channel_;
	}

	/**
	 * Constructs an illegal channel exception.
	 * 
	 * @param channel_ Illegal channel number.
	 * @param message_ Is the message.
	 * @param cause_ Is the throwable cause of this exception.
	 */
	public IllegalChannelException(final int channel_, final String message_, 
			final Throwable cause_)
	{
		super(message_, cause_);
		_channel = channel_;
	}
	
	public int getChannel()
	{
		return _channel;
	}
}
/* end of file */