/*
 *
 * File: TineAccessException.java
 * Date: Aug 27, 2007; 5:09:45 PM
 */
package de.desy.mst.libs.accelerator.exception;

import de.desy.mst.libs.accelerator.model.EAccess;

/**
 * This exception is thrown if the access via CDI or the DEVICE SERVER is not
 * right.
 *
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: TineAccessException.java 12066 2008-05-29 08:35:57Z walla $
 */
public class TineAccessException extends TineException
{
	/** Is the serial version UID to unify this class. */
	public static final long serialVersionUID = 6060304120310566232l;
	
	private final EAccess _access;
	
	/**
	 * The constructor of a TINE access exception. 
	 * 
	 * @param access_ The access which occurs on build this exception.
	 */
	public TineAccessException(final EAccess access_)
	{
		super(-1, "A TineAccessException has been thrown: access=" 
				+ access_.toString());
		_access = access_;
	}

	/**
	 * The constructor of a TINE access exception. 
	 * 
	 * @param message_ Gets a message as its description.
	 * @param access_ The access which occurs on build this exception.
	 */
	public TineAccessException(final String message_, final EAccess access_)
	{
		super(-1, message_ + ": access=" + access_.toString());
		_access = access_;
	}

	/**
	 * The constructor of a TINE access exception. 
	 * 
	 * @param cause_ Is the throwable cause of this exception.
	 * @param access_ The access which occurres on build this exception.
	 */
	public TineAccessException(final Throwable cause_, final EAccess access_)
	{
		super(-1, cause_ + ": access=" + access_.toString());
		_access = access_;
	}

	/**
	 * The constructor of a TINE access exception. 
	 * 
	 * @param message_ Gets a message as its description.
	 * @param cause_ Is the throwable cause of this exception.
	 * @param access_ The access which occurs on build this exception.
	 */
	public TineAccessException(final String message_, final Throwable cause_, 
			final EAccess access_)
	{
		super(-1, message_ + ": access=" + access_.toString(), cause_);
		_access = access_;
	}
	
	/**
	 * Returns the access.
	 * 
	 * @return Returns the access.
	 */
	public EAccess getAccess()
	{
		return _access;
	}

}
/* end of file */