/*
 *
 * File: IMonitorStructGroup.java
 * Date: Mar 6, 2009; 5:08:41 PM
 */
package de.desy.mst.libs.accelerator.model;

import de.desy.mst.libs.accelerator.IEData;
import de.desy.tine.structUtils.TTaggedStructure;

/**
 * <p>This is the java interface of monitoring all hardware values which are
 * delivered by a TINE tagged structure type, which all are inherited from the 
 * interface {@code TTaggedStructure}. These values are delivered by the method of 
 * {@link #getDataStructures(IEData)}. Remember that this is a <b>GROUPED TLink</b>
 * call!</p> 
 *
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: IMonitorStructGroup.java 22532 2009-03-06 16:16:36Z walla $
 */
public interface IMonitorStructGroup<T extends TTaggedStructure> extends IMonitor
{
	/**
	 * <p>Returns the TINE tagged structure elements of the initialized device 
	 * group directly from the TINE protocol. To deliver the right TINE tagged
	 * structure elements you have to tell these method which you want.</p>
	 * 
	 * @param data_ Enumeration of the data element.
	 * @return Returns a list of the current device name. This is a type which
	 * 		extends the class of {@code TTaggedStructure}.
	 */
	public abstract T[] getDataStructures(final IEData data_);
}
/* end of file */