/*
 *
 * File: IEnableListener.java
 * Date: Jan 25, 2008; 9:17:40 AM
 */
package de.desy.mst.libs.accelerator.view;

/**
 * The interface if a listener is enabled or not. More over there also exists
 * a set method for enabling the listener.
 * 
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: IEnableListener.java 12066 2008-05-29 08:35:57Z walla $
 */
public interface IEnableListener
{

	/**
	 * Disables or enables this listener by setting it to {@code false} or 
	 * {@code true}. This value has an influence on calling the
	 * {@code propertyChange(PropertyChangeEvent):void}, which will be executed
	 * if you set this value to {@code true} otherwise not.
	 * 
	 * @param isEnabled_ A boolean value.
	 */
	public abstract void setListenerEnabled(final boolean isEnabled_);
	
	/**
	 * Returns the boolean value if this listener is enabled or not. This value
	 * has an influence on calling {@code propertyChanged(PropertyChangeEvent)}.
	 * If this method returns a {@code true} then it will be executed otherwise
	 * not.
	 * 
	 * @return Returns if the listener is enabled or not.
	 */
	public abstract boolean isListenerEnabled();
	
}
/* end of file */