/*
 *
 * File: RFStatus.java
 * Date: Jan 28, 2007; 2:01:53 PM
 */
package de.desy.mst.libs.accelerator.view.container;

import java.awt.Dimension;
import java.util.ArrayList;
import java.util.List;

import javax.swing.GroupLayout;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.LayoutStyle;
import javax.swing.SwingConstants;

import de.desy.mst.libs.accelerator.EName;
import de.desy.mst.libs.accelerator.EViewElement;
import de.desy.mst.libs.accelerator.IComponentSubject;
import de.desy.mst.libs.accelerator.IEData;
import de.desy.mst.libs.accelerator.IEValue;
import de.desy.mst.libs.accelerator.view.CFrame;
import de.desy.mst.libs.accelerator.view.ComponentSubject;
import de.desy.mst.libs.accelerator.view.IViewContainer;
import de.desy.mst.libs.accelerator.view.generic.RFLabel;
import de.desy.mst.libs.accelerator.view.generic.RFPanel;

/**
 * <p>Will display the status bar of the current RF panel.</p>
 * <p>If there exists two data component elements which are the <i>status</i>
 * and the <i>near operation</i>, each of these elements must return by the
 * function of {@code IEData#toString()} or {@code IEData#getName(EName)}, with
 * the corresponding contents which includes the name of "<b>status</b>" or
 * "<b>near</b>", otherwise it will <b>not</b> be displayed in a special way.</p>
 *
 * @author <a href="marcus.walla(at)desy.de">walla</a>
 * @version $Id: RFStatus.java 39890 2010-10-15 15:22:25Z walla $
 */
@Deprecated
public abstract class RFStatus implements IViewContainer
{
	/** The serial version UID, which is used by the java.io.Serializable
	 * interface. */
	public static final long serialVersionUID = 5945993497902456456l;

	/** Is the panel which holds all entries of the status in it. */
	private JPanel _panel;

	/**
	 * Creates a status container. This container includes the length of the
	 * delivered entries of <tt>data_</tt> in a horizontal manner. There
	 * can also be a gap between the entries which has a length of <tt>width_</tt>.
	 *
	 * @param dim_ Is the dimension if this container. This can also be
	 * 		{@code null}.
	 * @param width_ Is the width between two shown entries. Must be greater
	 * 		or equal then zero.
	 * @param data_ Is the enumeration interface.
	 */
	public RFStatus(final Dimension dim_, final int width_,
			final IEData[] data_)
	{
		checkArguments(width_, data_);
		initialize(dim_, width_, data_);
	}

	/**
	 * Will check all arguments to their validity. If one argument is invalid an
	 * <tt>IllegalArgumentException<tt> is thrown.
	 *
	 * @param width_ Is the width between two shown entries. Must be greater
	 * 		or equal then zero.
	 * @param data_ Is the enumeration interface.
	 */
	private void checkArguments(final int width_,
			final IEData[] data_)
	{
		if (width_ < 0 || width_ > 40)
			throw new IllegalArgumentException("The width is invalid.");

	}

	/**
	 * <p>Initializes all enumeration interface entries in a horizontal manner.
	 * </p>
	 * <p>Here two different fonts stored per column. The first one is managed
	 * by {@code FONT_STATUS} and the second is {@code FONT_VALUE}. Both can
	 * by manipulated in the class {@link de.desy.mst.libs.accelerator.view.CFrame}.
	 * </p>
	 *
	 * @param dim_ Is the dimension if this container. This can also be
	 * 		{@code null}.
	 * @param width_ Is the width between two shown entries. Must be greater
	 * 		or equal then zero.
	 * @param data_ Is the enumeration interface.
	 */
	private void initialize(final Dimension dim_, final int width_,
			final IEData[] data_)
	{
		IComponentSubject subject = ComponentSubject.getInstance();

		if (dim_ != null)
			_panel = new RFPanel(dim_);
		else
			_panel = new JPanel();

    	CFrame.DEBUG_BORDER(_panel);
    	_panel.setOpaque(false);

    	List<IEData> data = new ArrayList<IEData>();

    	IEData near = null;
    	for(int i = 0; i < data_.length; i++)
    		if (data_[i].toString().toLowerCase().matches("\\w*near\\w*") ||
    				data_[i].getName(EName.DEVICENAME).toLowerCase().matches("\\w*near\\w*"))
    			near = data_[i];
    		else
    			data.add(data_[i]);

    	JPanel innerPanel[] = new JPanel[data.size()];
    	RFLabel lblTitle[] = new RFLabel[data.size()];

    	int i = 0;
    	// create all relevant elements
    	for(final IEData current: data)
    	{
    		lblTitle[i] = new RFLabel(current.getName(EName.NAME),
    				SwingConstants.CENTER, CFrame.FONT_STATUS, null);
    		CFrame.DEBUG_BORDER(lblTitle[i]);
    		lblTitle[i].setOpaque(false);
    		lblTitle[i].setHorizontalAlignment(SwingConstants.CENTER);

    		innerPanel[i] = new JPanel();
		    CFrame.DEBUG_BORDER(innerPanel[i]);

	        if (current.toString().toLowerCase().matches("\\w*status\\w*")
	        		&& near != null)
    		{
		        RFLabel lblValue = new RFLabel("[STATUS]", SwingConstants.CENTER, CFrame.FONT_VALUE, null);
			    CFrame.DEBUG_BORDER(lblValue);

			    RFLabel lblNear = new RFLabel("[REMOTE]", SwingConstants.CENTER, CFrame.FONT_VALUE, null);
			    CFrame.DEBUG_BORDER(lblNear);

			    // add both label to the panel ...
			    GroupLayout layout = new GroupLayout(innerPanel[i]);
			    innerPanel[i].setLayout(layout);
			    layout.setHorizontalGroup(
			    		layout.createSequentialGroup()
			    		.addComponent(lblValue, 150, 160, 200)
			    		.addGap(4)
			    		.addComponent(lblNear, 60, 60, 60));
			    layout.setVerticalGroup(
			    		layout.createParallelGroup(GroupLayout.Alignment.CENTER)
			    		.addComponent(lblValue)
			    		.addComponent(lblNear));

			    // ... and set it as the current one of status panel
		        subject.register(current, lblValue);
		        subject.register(near, lblNear);
		        subject.notify(current, EViewElement.OPAQUE, false);
//		        subject.notify(current, EViewElement.TOOLTIP_CDI, "Der Status des Senders.");
    		}
	        else
	        {
	        	RFLabel lblStatus;
		        if (current.getName(EName.PRECISION) == null)
		            lblStatus = new RFLabel(current.getName(EName.NAME), // was before: EName.VALUE
		            		SwingConstants.CENTER, CFrame.FONT_VALUE, null);
			    else
			        lblStatus = new RFLabel("0",
			        		SwingConstants.CENTER, CFrame.FONT_VALUE, null);
		        CFrame.DEBUG_BORDER(lblStatus);

		        innerPanel[i].add(lblStatus);

		        subject.register(current, lblStatus);
		        subject.notify(current, EViewElement.OPAQUE, false);
//		        subject.notify(current, EViewElement.TOOLTIP_CDI, null);
	        }

    		i++;
    	}

		GroupLayout layout = new GroupLayout(_panel);
		_panel.setLayout(layout);
		layout.setAutoCreateContainerGaps(true);
		layout.setAutoCreateGaps(true);

		// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
		
		GroupLayout.ParallelGroup groupVertical1 =
			layout.createParallelGroup(GroupLayout.Alignment.BASELINE);
		GroupLayout.ParallelGroup groupVertical2 =
			layout.createParallelGroup(GroupLayout.Alignment.BASELINE);
		GroupLayout.SequentialGroup groupHorizontal =
			layout.createSequentialGroup();

    	// add all above created elements to the outer group layout
    	for(int j = 0; j < data.size(); j++)
    	{
    		groupVertical1.addComponent(lblTitle[j]);
    		groupVertical2.addComponent(innerPanel[j]);

    		groupHorizontal.addGroup(
    				layout.createParallelGroup().
    					addComponent(lblTitle[j]).
    					addComponent(innerPanel[j]));
    		if (j < data.size() - 1)
    			groupHorizontal.addPreferredGap(LayoutStyle.ComponentPlacement.UNRELATED);
    	}

		// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    	layout.setVerticalGroup(layout.createSequentialGroup().
				addGroup(groupVertical1).
				addGap(10).
				addGroup(groupVertical2));
		layout.setHorizontalGroup(groupHorizontal);
	}

	/**
	 * <p><u><i>ATTENTION:</u></i> THIS METHOD IS CURRENTLY NOT USED AND WILL
	 * THROW A RUNTIME EXCEPTION!<p>
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewContainer#reset(de.desy.mst.libs.accelerator.IEValue)
	 */
	public void reset(final IEValue container_)
	{
		throw new RuntimeException("This method is currently not implemented.");
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewContainer#getPanel()
	 */
	public JComponent getPanel()
	{
		return _panel;
	}

	/**
	 * <p><u><i>ATTENTION:</u></i> THIS METHOD IS CURRENTLY NOT USED AND WILL
	 * THROW A RUNTIME EXCEPTION!<p>
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewContainer#getValue(de.desy.mst.libs.accelerator.IEValue)
	 */
	public Number getValue(final IEValue container_)
	{
		throw new RuntimeException("This method is not implemented.");
	}

	/**
	 * <p><u><i>ATTENTION:</u></i> THIS METHOD IS CURRENTLY NOT USED AND WILL
	 * THROW A RUNTIME EXCEPTION!<p>
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewContainer#setValue(de.desy.mst.libs.accelerator.IEValue, java.lang.Number)
	 */
	public void setValue(final IEValue container_, final Number value_)
	{
		throw new RuntimeException("This method is not implemented.");
	}

	/**
	 * <p><u><i>ATTENTION:</u></i> THIS METHOD IS CURRENTLY NOT USED AND WILL
	 * THROW A RUNTIME EXCEPTION!<p>
	 * <p>This allows to set a boolean value on the enumeration interface of
	 * {@link IEValue}. For example, you can get the enabling of the
	 * container by calling it with <tt>EViewContainer.BUTTON</tt> and the
	 * enumeration interface type of <tt>EViewElement.ENABLE</tt>.</p>
	 *
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewContainer#getBoolean(de.desy.mst.libs.accelerator.IEValue, de.desy.mst.libs.accelerator.EViewElement)
	 */
	public boolean getBoolean(final IEValue container_,
			final EViewElement element_)
	{
		throw new RuntimeException("This method is not implemented.");
	}

	/**
	 * <p><u><i>ATTENTION:</u></i> THIS METHOD IS CURRENTLY NOT USED AND WILL
	 * THROW A RUNTIME EXCEPTION!<p>
	 *
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewContainer#setBoolean(de.desy.mst.libs.accelerator.IEValue, de.desy.mst.libs.accelerator.EViewElement, boolean)
	 */
	public void setBoolean(final IEValue container_,
			final EViewElement element_, final boolean value_)
	{
		throw new RuntimeException("This method is not implemented.");
	}

}
/* end of file */