/*
 *
 * File: RFSwitch.java
 * Date: Jan 25, 2007; 10:07:07 AM
 */
package de.desy.mst.libs.accelerator.view.container;

import java.awt.Dimension;
import java.util.HashMap;
import java.util.Map;

import javax.swing.ButtonGroup;
import javax.swing.GroupLayout;
import javax.swing.JComponent;
import javax.swing.JPanel;

import de.desy.mst.libs.accelerator.EName;
import de.desy.mst.libs.accelerator.EViewElement;
import de.desy.mst.libs.accelerator.IComponentSubject;
import de.desy.mst.libs.accelerator.IEData;
import de.desy.mst.libs.accelerator.IEValue;
import de.desy.mst.libs.accelerator.view.CFrame;
import de.desy.mst.libs.accelerator.view.ComponentSubject;
import de.desy.mst.libs.accelerator.view.IViewComponent;
import de.desy.mst.libs.accelerator.view.IViewContainer;
import de.desy.mst.libs.accelerator.view.generic.RFButton;
import de.desy.mst.libs.accelerator.view.generic.RFPanel;

/**
 * This displays a status label, two buttons and a before leading label.
 * 
 * @author <a href="marcus.walla(at)desy.de">walla</a>
 * @version $Id: RFSwitch.java 12066 2008-05-29 08:35:57Z walla $
 */
public abstract class RFSwitch implements IViewContainer
{
	/** The serial version UID, which is used by the java.io.Serializable 
	 * interface. */
	public static final long serialVersionUID = 1070307234965134646l;

	/** Is the panel which holds all entries of the status in it. */
	private JPanel _panel;
	
	/** Is the map of the pairs IEData and IViewComponent for the 
	 * button on. */
	protected Map<IEData, IViewComponent> _mapButtonOn;
	
	/** Is the map of the pairs IEData and IViewComponent for the 
	 * button off. */
	protected Map<IEData, IViewComponent> _mapButtonOff;
	
	/** Is the map of the pairs IEData and IViewComponent for the 
	 * switch label. */
	protected Map<IEData, IViewComponent> _mapStatus;

	
	/**
	 * Constructs and initialize the two buttons and the label.
	 * 
	 * @param dim_ Is the dimension of the container. This can also be 
	 * 		{@code null}.
	 * @param data_ Is an array of the enumeration interfaces.
	 */
	public RFSwitch(final Dimension dim_, 
			final IEData[] data_)
	{
		checkArguments(data_);
		
		_mapButtonOn = new HashMap<IEData, IViewComponent>();
		_mapButtonOff = new HashMap<IEData, IViewComponent>();
		_mapStatus = new HashMap<IEData, IViewComponent>();

		if (dim_ != null)
			_panel = new RFPanel(new Dimension(dim_.width, dim_.height));
		else
			_panel = new JPanel();
		
		initialize(data_);
	}

	/**
	 * This will initialize all changeable switches and the status of them.
	 * 
	 * @param data_ Is an array of the enumeration interface.
	 */
	private void initialize(final IEData[] data_)
	{
		IComponentSubject subject = ComponentSubject.getInstance();

		ButtonGroup group = new ButtonGroup();
		
//		RFLabel label = new RFLabel(data_[0].getName(EName.NAME) 
//				+ ':', CFrame.FONT_SWITCH, null);
//		label.setOpaque(false);
//		
//		RFLabel status = new RFLabel("***", CFrame.FONT_SWITCH, null);
//		status.setBorder(BorderFactory.createLineBorder(CColor.BLACK, 2));
//		status.setOpaque(false);
//		status.setHorizontalAlignment(SwingConstants.CENTER);
//		status.setVerticalAlignment(SwingConstants.CENTER);
//		subject.register(data_[0], status);
//		_mapStatus.put(data_[0], status);
		
		RFButton toggle0 = 
			new RFButton(data_[0].getName(EName.BUTTON)
					, CFrame.FONT_SWITCH, null);
		group.add(toggle0);
		subject.register(data_[0], toggle0);
		_mapButtonOn.put(data_[0], toggle0);
		
		RFButton toggle1 = 
			new RFButton(data_[1].getName(EName.BUTTON)
					, CFrame.FONT_SWITCH, null); 
		group.add(toggle1);
		subject.register(data_[1], toggle1);
		_mapButtonOff.put(data_[1], toggle1);
		
		_panel.setOpaque(false);
		GroupLayout layout = new GroupLayout(_panel);
		_panel.setLayout(layout);
		
		layout.setHorizontalGroup(layout.createSequentialGroup()
				.addComponent(toggle0)
				.addGap(5)
				.addComponent(toggle1)
				);
		layout.setVerticalGroup(layout.createParallelGroup()
				.addComponent(toggle0)
				.addComponent(toggle1)
				);
	}
	
	/**
	 * <p><u><i>ATTENTION:</u></i> THIS METHOD IS CURRENTLY NOT USED AND WILL 
	 * THROW A RUNTIME EXCEPTION!<p>
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewContainer#reset(de.desy.mst.libs.accelerator.IEValue)
	 */
	public void reset(final IEValue container_)
	{
		throw new RuntimeException("This method is currently not implemented.");
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewContainer#getPanel()
	 */
	public JComponent getPanel()
	{
		return _panel;
	}
	

	/** 
	 * <p><u><i>ATTENTION:</u></i> THIS METHOD IS CURRENTLY NOT USED AND WILL 
	 * THROW A RUNTIME EXCEPTION!<p>
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewContainer#getBoolean(de.desy.mst.libs.accelerator.IEValue, de.desy.mst.libs.accelerator.EViewElement)
	 */
	public boolean getBoolean(final IEValue container_, final EViewElement element_)
	{
		throw new RuntimeException("This method is not implemented!");
	}

	/** 
	 * <p><u><i>ATTENTION:</u></i> THIS METHOD IS CURRENTLY NOT USED AND WILL 
	 * THROW A RUNTIME EXCEPTION!<p> 
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewContainer#getValue(de.desy.mst.libs.accelerator.IEValue)
	 */
	public Number getValue(final IEValue container_)
	{
		throw new RuntimeException("This method is not implemented!");
	}

	/** 
	 * <p><u><i>ATTENTION:</u></i> THIS METHOD IS CURRENTLY NOT USED AND WILL 
	 * THROW A RUNTIME EXCEPTION!<p> 
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewContainer#setBoolean(de.desy.mst.libs.accelerator.IEValue, de.desy.mst.libs.accelerator.EViewElement, boolean)
	 */
	public void setBoolean(final IEValue container_, final EViewElement element_,
			final boolean value_)
	{
		throw new RuntimeException("This method is not implemented!");
	}

	/** 
	 * <p><u><i>ATTENTION:</u></i> THIS METHOD IS CURRENTLY NOT USED AND WILL 
	 * THROW A RUNTIME EXCEPTION!<p> 
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewContainer#setValue(de.desy.mst.libs.accelerator.IEValue, java.lang.Number)
	 */
	public void setValue(final IEValue container_, final Number value_)
	{
		throw new RuntimeException("This method is not implemented!");
	}

	/**
	 * This will check all arguments for there validity. If their exists one 
	 * value which is invalid an {@code IllegalArgumentException} is thrown.
	 * 
	 * @param data_ These are the current data element for connecting to the
	 * 		TINE server.
	 */
	private void checkArguments(final IEData[] data_)
	{
		if (data_ == null)
			throw new NullPointerException("The data enumeration is null.");
		if (data_.length == 0)
			throw new IllegalArgumentException("The data enumeration shell" +
					" not be empty.");
		if (data_.length != 2)
			throw new IllegalArgumentException("The data enumeration should" +
					" have two elements.");
		if (data_[0] == null || data_[1] == null)
			throw new NullPointerException("The first or second data enumeration" +
					" is null.");
	}

}
/* end of file */