/*
 *
 * File: RFContainerSlider.java
 * Date: Jan 28, 2007; 10:06:19 AM
 */
package de.desy.mst.libs.accelerator.view.container;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.JComponent;

import de.desy.mst.libs.accelerator.EName;
import de.desy.mst.libs.accelerator.EViewElement;
import de.desy.mst.libs.accelerator.IComponentSubject;
import de.desy.mst.libs.accelerator.IEData;
import de.desy.mst.libs.accelerator.IEValue;
import de.desy.mst.libs.accelerator.Utility;
import de.desy.mst.libs.accelerator.view.CFrame;
import de.desy.mst.libs.accelerator.view.ComponentSubject;
import de.desy.mst.libs.accelerator.view.IViewContainer;
import de.desy.mst.libs.accelerator.view.generic.RFLabel;
import de.desy.mst.libs.accelerator.view.generic.RFPanel;

/**
 * This container will carry the whole enumeration array in a list. This list is
 * displayed on screen, with the title of the enumeration (which means its
 * name), a possible unit with an equality sign and value itself. The value is
 * also stored in the subject object, which allows to set the value on different
 * swing containers concurrent.
 *
 * @author <a href="marcus.walla at desy.de">walla</a>
 * @version $Id: RFTransmitter.java 39890 2010-10-15 15:22:25Z walla $
 */
@Deprecated
public abstract class RFTransmitter implements IViewContainer
{
	/** The serial version UID, which is used by the java.io.Serializable
	 * interface. */
	public static final long serialVersionUID = 5400102030504050005l;

	/** Is the panel which holds all entries of the status in it. */
	private RFPanel _panel;

	/**
	 * This constructor will contain in its outer container all elements of the
	 * delivered enumeration interface array. This a list of different
	 * enumeration types which are displayed in a list.
	 *
	 * @param data_ Is an array of the enumeration interface.
	 * @param dim_ Is the dimension if this container.
	 */
	public RFTransmitter(final IEData[] data_,
			final Dimension dim_)
	{
		checkArguments(data_, dim_);
		initialize(data_, dim_);
	}

	/**
	 * This private method will check if all arguments are valid. If one is
	 * invalid an <tt>IllegalArgumentException</tt> is thrown.
	 *
	 * @param data_ Is an array of the enumeration interface.
	 * @param dim_ Is the dimension if this container.
	 */
	private void checkArguments(final IEData[] data_,
			final Dimension dim_)
	{
		if (data_ == null)
			throw new IllegalArgumentException("Enumeration array is null.");
		for(int i = 1; i < data_.length; i++)
			if (data_[i] == null)
					throw new IllegalArgumentException("Enumeration element" +
							" of " + i + " is null.");
	}

	/**
	 * This will initialize the whole container panel with the length of the
	 * encapsulated enumeration types of {@code IEData}.
	 *
	 * @param data_ Is an array of the enumeration interface.
	 * @param dim_ Is the dimension if this container.
	 */
	private void initialize(final IEData[] data_, final Dimension dim_)
	{
		GridBagConstraints position = new GridBagConstraints();

		_panel = new RFPanel(dim_);
		_panel.setLayout(new GridBagLayout());

		IComponentSubject subject = ComponentSubject.getInstance();

		RFLabel label;

		for(int i = 0; i < data_.length; i++)
		{
			label = new RFLabel(data_[i].getName(EName.NAME_TABLE),
					CFrame.FONT_TRANSMITTER, null);
			label.setToolTipText(Utility.getAddress(data_[i]));
			position.gridx = 0;
			position.gridy = i;
			position.insets = new Insets(0,0,5,0);
			position.anchor = GridBagConstraints.LINE_START;
			position.ipadx = 0;
			position.ipady = 0;
			position.weightx = 0.0;
			position.weighty = 0.0;
			position.fill = GridBagConstraints.NONE;
			_panel.add(label, position);

			label = new RFLabel("[" + data_[i].getName(EName.UNIT)
					+ "] =", CFrame.FONT_TRANSMITTER, null);
			label.setToolTipText(Utility.getAddress(data_[i]));
			position.gridx = 1;
			position.gridy = i;
			position.anchor = GridBagConstraints.LINE_END;
			position.ipadx = 0;
			position.ipady = 0;
			position.weightx = 0.0;
			position.weighty = 0.0;
			position.fill = GridBagConstraints.NONE;
			_panel.add(label, position);

			label = new RFLabel(String.format(
					data_[i].getName(EName.PRECISION), 0.0)
					, CFrame.FONT_TRANSMITTER, null);
			position.gridx = 2;
			position.gridy = i;
			position.anchor = GridBagConstraints.LINE_END;
			position.insets = new Insets(0,10,5,0);
			position.ipadx = 0;
			position.ipady = 0;
			position.weightx = 0.0;
			position.weighty = 0.0;
			position.fill = GridBagConstraints.NONE;
			_panel.add(label, position);
			subject.register(data_[i], label);
			subject.notify(data_[i], EViewElement.TOOLTIP, "<html>" + Utility.getAddress(data_[i]) + "</html>");
		}
	}

	/**
	 * <p><u><i>ATTENTION:</u></i> THIS METHOD IS CURRENTLY NOT USED AND WILL
	 * THROW A RUNTIME EXCEPTION!<p>
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewContainer#reset(de.desy.mst.libs.accelerator.IEValue)
	 */
	public void reset(final IEValue container_)
	{
		throw new RuntimeException("This method is currently not implemented.");
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewContainer#getPanel()
	 */
	public JComponent getPanel()
	{
		return _panel;
	}

	/**
	 * <p><u><i>ATTENTION:</u></i> THIS METHOD IS CURRENTLY NOT USED AND WILL
	 * THROW A RUNTIME EXCEPTION!<p>
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewContainer#getValue(de.desy.mst.libs.accelerator.IEValue)
	 */
	public Number getValue(final IEValue container_)
	{
		throw new RuntimeException("This method is not implemented.");
	}

	/**
	 * <p><u><i>ATTENTION:</u></i> THIS METHOD IS CURRENTLY NOT USED AND WILL
	 * THROW A RUNTIME EXCEPTION!<p>
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewContainer#setValue(de.desy.mst.libs.accelerator.IEValue, java.lang.Number)
	 */
	public void setValue(final IEValue container_, final Number value_)
	{
		throw new RuntimeException("This method is not implemented.");
	}

	/**
	 * <p><u><i>ATTENTION:</u></i> THIS METHOD IS CURRENTLY NOT USED AND WILL
	 * THROW A RUNTIME EXCEPTION!<p>
     * <p>This allows to set a boolean value on the enumeration interface of
	 * {@link IEValue}. For example, you can get the enabling of the
	 * container by calling it with <tt>EViewContainer.BUTTON</tt> and the
     * enumeration interface type of <tt>EViewElement.ENABLE</tt>.</p>
	 *
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewContainer#getBoolean(de.desy.mst.libs.accelerator.IEValue, de.desy.mst.libs.accelerator.EViewElement)
	 */
	public boolean getBoolean(final IEValue container_,
			final EViewElement element_)
	{
		throw new RuntimeException("This method is not implemented.");
	}

	/**
	 * <p><u><i>ATTENTION:</u></i> THIS METHOD IS CURRENTLY NOT USED AND WILL
	 * THROW A RUNTIME EXCEPTION!<p>
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewContainer#setBoolean(de.desy.mst.libs.accelerator.IEValue, de.desy.mst.libs.accelerator.EViewElement, boolean)
	 */
	public void setBoolean(final IEValue container_,
			final EViewElement element_, final boolean value_)
	{
		throw new RuntimeException("This method is not implemented.");
	}
}
/* end of file */