/*
 *
 * File: RFPanelUnfixed.java
 * Date: Oct 11, 2007; 9:29:29 AM
 */
package de.desy.mst.libs.accelerator.view.generic;

import java.awt.Color;
import java.awt.LayoutManager;
import java.util.EventListener;

import javax.swing.JPanel;
import javax.swing.border.Border;

import de.desy.mst.libs.accelerator.EGraphicComponent;
import de.desy.mst.libs.accelerator.ESeverity;
import de.desy.mst.libs.accelerator.IEData;
import de.desy.mst.libs.accelerator.exception.NotImplementedException;
import de.desy.mst.libs.accelerator.exception.OperationNotAllowedException;
import de.desy.mst.libs.accelerator.view.CFrame;
import de.desy.mst.libs.accelerator.view.IViewComponent;

/**
 * This is a RF panel which is unfixed in the size as default.
 *
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: RFPanelUnfixed.java 36465 2010-06-04 08:04:07Z walla $
 */
public class RFPanelUnfixed extends JPanel implements IViewComponent
{
	// --- CLASS FIELDS --------------------------------------------------------
	
	/** This is the current unique serial version UID. It helps the application
	 * to control serialization compatibility between versions. */
	final static long serialVersionUID = 858829100959324592L;

	/** Class name. */
	public final static String CLAZZ = RFPanelUnfixed.class.getSimpleName();
	
	// --- CONSTRUCTOR ---------------------------------------------------------
	
    /**
     * Creates a new panel for RF with an unfixed size.
     */
    public RFPanelUnfixed()
    {
    	super();
    	debug();
    }

    /**
     * Creates a new JPanel with a specified buffering strategy.
     * If {@code isDoubleBuffered} is {@code true}, the {@code JPanel}  will
     * use a double buffer.
     *
     * @param layout_ The LayoutManager to use.
     * @param doubleBuffered_ A boolean, {@code true} for double-buffering, which uses
     * 			additional memory space to achieve fast, flicker-free updates.
     */
    public RFPanelUnfixed(final LayoutManager layout_,
    		final boolean doubleBuffered_)
    {
    	super(layout_, doubleBuffered_);
    	debug();
    }

    // --- INSTANCE METHODS ----------------------------------------------------
    
	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#debug()
	 */
	public void debug()
	{
		CFrame.DEBUG_BORDER(this);
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putSeverity(int)
	 */
	public void putSeverity(final ESeverity severity_)
	{
		throw new NotImplementedException();
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#fetchSeverity()
	 */
	public ESeverity fetchSeverity()
	{
		return ESeverity.NORMAL;
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#fetchGraphicComponent()
	 */
	public EGraphicComponent fetchGraphicComponent()
	{
		return EGraphicComponent.PANEL;
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#fetchLabel()
	 */
	public String fetchText(IEData view_)
	{
		return null;
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#isEnable()
	 */
	public boolean isEnable()
	{
		return super.isEnabled();
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#isVisual()
	 */
	public boolean isVisual()
	{
		return super.isVisible();
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#hasOpaque()
	 */
	public boolean hasOpaque()
	{
		return super.isOpaque();
	}

	/*
	 * Will always return the {@code false} value.
	 *
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#isValueAdjusting(de.desy.mst.libs.accelerator.model.IEDataComponent)
	 */
	public boolean isValueAdjusting(IEData view_)
	{
		return false;
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putListener(java.util.EventListener)
	 */
	public void putListener(final EventListener listener_)
	{
		throw new IllegalArgumentException("The listener model you are" +
				" adding doesn't exist for this instance: " +
				listener_.toString());
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putLabel(java.lang.String)
	 */
	public void putText(String label_, IEData view_)
	{
		throw new OperationNotAllowedException("The operation is not allowed.");
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putNumber(java.lang.Number, de.desy.mst.libs.accelerator.model.IEDataComponent)
	 */
	public void putNumber(Number number_, IEData view_)
	{
		throw new OperationNotAllowedException("The operation is not allowed.");
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putEnable(boolean)
	 */
	public void putEnable(boolean enable_)
	{
		super.setEnabled(enable_);
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putVisual(boolean)
	 */
	public void putVisual(boolean visual_)
	{
		super.setVisible(visual_);
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putOpaque(boolean)
	 */
	public void putOpaque(boolean opaque_)
	{
		super.setOpaque(opaque_);
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putForeground(java.awt.Color)
	 */
	public void putForeground(Color foreground_)
	{
		super.setForeground(foreground_);
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putBackground(java.awt.Color)
	 */
	public void putBackground(Color background_)
	{
		super.setBackground(background_);
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putTooltip(java.lang.String)
	 */
	public void putTooltip(String tooltip_)
	{
		this.setToolTipText(tooltip_);
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putSelected(boolean)
	 */
	public void putSelected(boolean isSelected_)
	{
		throw new RuntimeException("This method makes no sense here!");
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putBorder(javax.swing.border.Border)
	 */
	public void putBorder(final Border border_)
	{
		super.setBorder(border_);
	}
	
	@Override
	public String toString()
	{
		return CLAZZ;
	}
}
/* end of file */