/**
 *  Mar 23, 2007 - Apr  2, 2007	ws:	creation from scratch
 *  Sep  4, 2007 - Sep  4, 2007	ws:	cosmetics for unification Ini, PfadeIni
 *  Nov 28, 2007 - Nov 28, 2007	ws:	added equals, hashCoce
 *	Feb 15, 2008 - Feb 15, 2008 ws: added "@Override"es
 *	May  5, 2008 - May  5, 2008	ws:	use the Default in Type instead of null
 *	Jun  5, 2008 - Jun  5, 2008	ws:	added:	getShortFrameName()
 */
package de.desy.mst.libs.framework.mstapp;

import java.util.Arrays;

/**
 * Immutable class mit den Informationen zur Applikation, die sich
 * aus dem MstFrameMain Namen saugen lassen (Was siehe TYPE), aber insbesondere:
 * <li> System
 * <li> SubSystem
 * <li> Application
 *
 * @author ws
 * version 1.0.0
 */
public class ApplicationInfo {
	/**
	 * de.desy.mst.libs.framework.mstapp.MstFrameMain<br>
	 * wird folgendermassen zerlegt
	 */
	public enum TYPE {
			  /**z.B. de				*/ COUNTRY		("de")
			, /**z.B. desy				*/ INSTITUTION	("desy")
			, /**z.B. mst				*/ GROUP		("mst")
			, /**z.B. desy2, libs, ...	*/ ACCELERATOR	("BadFacility")
			, /**z.B. framework			*/ SUBSYSTEM	("BadSUBSYSTEM")
			, /**z.B. mstapp			*/ APPLICATION	("BadAPPLICATION")
			, /**z.B. MstFrameMain		*/ CLASSNAME	("BadCLASSNAME")
			;
		private final String sDefault;
		private TYPE(final String sDefault) {this.sDefault = sDefault;}
		//---------------------------------------------------------------------
		public String getDefault() {return this.sDefault;}
	}
	public static final ApplicationInfo __SYSTEM_APP_INFO;
	public static final ApplicationInfo _UNKNOWN_APP_INFO;
//-----------------------------------------------------------------------------
	static{
		__SYSTEM_APP_INFO  = createSystemAppInfo("_System");
		 _UNKNOWN_APP_INFO = createSystemAppInfo("~~~UNKNOWN APPNAME~~~");
	}

	/**
	 * @param pseudoAppName		not null
	 * @throws NullPointerException, falls mstFrameMainName oder appName null
	 */
	public static ApplicationInfo createSystemAppInfo
												(final String pseudoAppName) {
		return createAppInfo(MstFrameMain.class.getName(), pseudoAppName);
	}
	/**
	 * @param frameMainName	not null
	 * @param appName		not null
	 * @throws NullPointerException, falls mstFrameMainName oder appName null
	 */
	public static ApplicationInfo createAppInfo(final String frameMainName, final String appName) {
		return new ApplicationInfo(frameMainName, appName);
	}


//=============================================================================

	private final String frameName;
	private final String appName;
	private final String[] frameNameArray;

//=============================================================================

	/**
	 * @param mstFrameMainName 	not null
	 * @param appName			not null
	 * @throws NullPointerException, falls mstFrameMainName oder appName null
	 */
	private ApplicationInfo(final String mstFrameMainName, final String appName){
		super();
		if(mstFrameMainName == null) {
			final String msg = "The mstFrameMainName may not be null.";
			throw new NullPointerException(msg);
		}
		if(appName == null) {
			final String msg = "The appName may not be null.";
			throw new NullPointerException(msg);
		}
		this.frameName = mstFrameMainName;
		this.appName   = appName;
		this.frameNameArray = calcFrameNameArray(this.frameName, " ");
	}

//=============================================================================

	public final String getInfo(final TYPE type, final String sDefault){
		final int index = type.ordinal();
		return this.frameNameArray.length > index 	?
												this.frameNameArray[index]
												:
												sDefault	//false
												;
	}
	public final String getInfo(final TYPE type){return getInfo(type, type.getDefault());}

	public final String   getAppName()			{return this.appName;}
	public final String   getFrameName() 		{return this.frameName;}
	public final String[] getFrameNameArray()	{return this.frameNameArray.clone();}


	public final String getSubSystem()	{return getInfo(TYPE.SUBSYSTEM,   "");}
	public final String getAccelerator(){return getInfo(TYPE.ACCELERATOR, "");}
	/**Klassen Name ohne das Paket*/
	public String getShortFrameName() { 
		return this.frameNameArray[this.frameNameArray.length - 1];
		//getInfo(TYPE.CLASSNAME); waere weniger robust
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((this.appName == null) ? 0 : this.appName.hashCode());
		result = prime * result
				+ ((this.frameName == null) ? 0 : this.frameName.hashCode());
		result = prime * result + Arrays.hashCode(this.frameNameArray);
		return result;
	}
	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override public boolean equals(final Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (!(obj instanceof ApplicationInfo))
			return false;
		final ApplicationInfo other = (ApplicationInfo) obj;
		if (this.appName == null) {
			if (other.appName != null)
				return false;
		} else if (!this.appName.equals(other.appName))
			return false;
		if (this.frameName == null) {
			if (other.frameName != null)
				return false;
		} else if (!this.frameName.equals(other.frameName))
			return false;
		if (!Arrays.equals(this.frameNameArray, other.frameNameArray))
			return false;
		return true;
	}
//=============================================================================

	private String[] calcFrameNameArray(final String frameName, final String separator) {
		if(frameName.contains(separator)){
			return calcFrameNameArray(frameName, separator + " ");
		}
		return frameName.replace(".", separator).split(separator);
	}
}
