/*
 * Feb 07, 2005 - Apr 22, Jul 15, 2005	ws: Created on
 * Jun 02, 2006 - Jun 22, 2006 	ws:	first class support fuer die Maschine
 * Mar 19, 2007 - Mar 30, 2007	ws: added: ApplicationInfo
 * Jun 20, 2007 - Jun 20, 2007	ws: changed regEx extractAppNameFromClassName [.]
 * Nov 28, 2007 - Nov 28, 2007	ws:	neuer Kontruktor
 * Dec 11, 2009 - Dec 16, 2009	ws:	added debugSwitches
 * Nov 30, 2007 - Nov 30, 2007	ws:	field title
 * Feb 27, 2012 - Feb 27, 2012	ws:	retrieveTitle catch Exception (-> MaWa eMail 2012-02-24)
 */
package de.desy.mst.libs.framework.mstapp;

//import de.desy.mst.libs.framework.mstapp.ini.LocalHome;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import de.desy.mst.libs.framework.mstapp.appinf.ApplikationKind;
import de.desy.mst.libs.framework.mstapp.appinf.IApp;
import de.desy.mst.libs.framework.mstapp.mode.Maschine;
import de.desy.mst.libs.framework.mstapp.mode.MaschineMgr;

/**
 * @author schuette
 *
 * Description: Decodes input parameters of main in MstApp to handy properties
 * Here and only here are the input parameters and the help text defined.
 * Complete management of parameters in the before ini file stage
 * Reserved Parameters are:
 * <ol>
 * <li> Name of the MainFrame	(required)								</li>
 * <li> Name of the Applikation (Default MainFrame Name ohne package) 	</li>
 * <li> Name of the Maschine    (Default Maschine.getDefault())			</li>
 * </ol>
 * @link MstApp.main
 *
 */
public class ParamDecoder {
    /**0-basierte Postion des Parameters: mstFrameMainName == Hauptfenster der Applikation*/
	public final static int MAIN_FRAME_POSITION = 0;
	/**0-basierte Postion des Parameters: appName == eindeutiger Applikationsname, der auch Basis eines Dateinamens sein kann*/
	public final static int APPNAME_POSITION = 1;
	/**0-basierte Postion des Parameters: maschinenName == Kurz, Lang oder Logbuch wird erkannt siehe Klasse Maschine*/
	public final static int MASCHINE_NAME_POSITION = 2;
	/**Anzahl der vom System reservierten Parameter, User Parameter knnen danach kommen*/
	public final static int RESERVED_PARAMETER_COUNT = MASCHINE_NAME_POSITION + 1;

	public final String 	originalString;
	public final String[]	originalAppParams;

	/** helpNeeded == erstes argument von args enthaelt ?*/
	public final boolean helpNeeded;

	/**
	 * Name der Hauptform inklusive Package Prefix. <br>
	 * Z.B. 'de.desy.mst.MeineForm' <br>
	 * Setzen via: Erster Applikations-Start-Parameter
	 */
	public final String  mstFrameMainName;

	/** Name der Applikation. <br>
	 * Default: mstFrameMainName ohne Package Prefix<br>
	 * Z.B. 'MeineForm' <br>
	 * Optional setzen via: zweitem Applikations-Start-Parameter<br>
	 * Z.B. "PeMagDevMgr"
	 * Der appName <b>muss<b>
	 * <li> gltig in jedem Filesystem sein
	 * <li> muss eindeutig im ganzen Kontrollsystem-Scope sein
	 */
	public final String  appName;

	/**
	 *	via IApp mechanism from AnLa retrieved title or as a backup appName 
	 */
	public final String  title;
	
	/**
	 * Maschinen Manager aus drittem Parameter
	 */
	public final MaschineMgr maschineMgr;
	/**
	 * AppName und Hauptframe zerlegt um auf Facility, Subsystem zu schliessen
	 */
	public final ApplicationInfo applicationInfo;
	/**
	 * User Bereich der Startparameter <br>
	 * Startparemeter mit Index >= RESERVED_PARAMETER_COUNT
	 */
	private final String[]	parameter;
	private boolean parameterAppNameWasSupplied  = true;
	private boolean parameterMaschineWasSupplied = true;


     //TINE KONSTANTEN
	 @Deprecated public static final String     TINE_SERVICE_NAME       = "Tine";
	 @Deprecated public static final String TINE_VERZEICHNIS_NAME       = "Database";
	 @Deprecated public static final String TINE_DATABASE_SRV_HOME__KEY = "tine.home";
	 @Deprecated public static final String       TINE_DEBUG_LEVEL__KEY = "debug.level";

//=============================================================================

	/**
	 * @param args	<li> full qualified mstFrameMainName (inklusive Paketname)
	 * 				<li> <b>optionale Parameter folgen:</b>
	 * 				<li> ApplikationsName
	 * 				<li> Maschine z.B. "PE"
	 * 				<li> Weitere Parameter sind Applikations Entwickler
	 * 					 Parameter und knnen mit aus dem Entwicklerframe mit
	 * 					 getMstApp().getStartParameter().userParameter(int index)
	 *  				 index = 0, 1, ... wieder geholt werden.
	 */
	public ParamDecoder(final String[] args) {
		this(null, args);
	}
    public ParamDecoder(final String mstFrameMainName, final String appName, final String maschinenName, final String[] userParameter) {
		this(xTorParamsToMainForm(mstFrameMainName, appName, maschinenName, userParameter));
	}
	private static String[] xTorParamsToMainForm(final String mstFrameMainName, final String appName, final String maschinenName, final String[] userParameter) {
		final List<String> list = new ArrayList<String>();
		list.add(mstFrameMainName);
		list.add(appName);
		list.add(maschinenName);
		list.addAll(Arrays.asList(userParameter));
		return list.toArray(new String[0]);
	}
	/** Convenience Konstruktor: falls mstFrameMainName nichttrivial und nicht in args wird es virtuell dazugetan*/
	public ParamDecoder(final String mstFrameMainName, String ... args) {
		super();

		this.originalString    = mstFrameMainName;
		this.originalAppParams = args;

		if(args == null){args = new String[]{};}
		this.mstFrameMainName = extractMstFrameMainName(mstFrameMainName, args);
		this.parameter = extractParameter(args);

		//check for help needed
		if(internalHelpIsNeeded()){
			this.helpNeeded = true;
			this.appName = "";
			this.maschineMgr = null;
		}
		else{
			this.helpNeeded = false;
			this.appName = extractMstAppName(this.mstFrameMainName, args);
			final String maschine = extractMaschinenName();
			this.maschineMgr = new MaschineMgr(maschine, this.getParameterMaschineWasSupplied());
            //initTine();
		}

		this.title = retrieveTitle(ApplikationKind.Automatic, this.appName);
		
		this.applicationInfo =
		ApplicationInfo.createAppInfo(this.mstFrameMainName, this.appName);
	}

//=============================================================================

	public String helpText(){
		return new String("Please supply the name of the MstFrameMain Class."
						+ "\nFor example 'de.desy.mst.mstapp.MstFrameMain' (application run duration constant)."
						+ "\nAn ApplicationName ist optional (application run duration constant)."
						+ "\nThe third Argument is the optional MaschineName to start with (might be changed during the run of the application)."
						+ "\nIn Eclipse this is part of 'debug ...' or 'run ...' (x)=Arguments.");
	}

	/**
	 * Gibt einen userParameter oder null zurueck
	 * @param 	index 	0 basierter Index des Parameters
	 * @return 			Parameter oder null
	 */
	public String userParameter(final int index) {
		if(index >= 0 && index < this.parameter.length){
			return this.parameter[index];
		}else{
			return null;
		}
	}


//=============================================================================

	private String extractMstFrameMainName(final String mstFrameMainName, final String[] args) {
		if(mstFrameMainName != null)
			return mstFrameMainName;

		if(args.length > 0)
			return args[0];

		return null;
	}
	private String extractMstAppName(final String mstFrameMainName, final String[] args) {
		final String xmlSuffix = ".XML";
		int appNamePosition = APPNAME_POSITION;	//if no first Param this moves one ahead
		if(args.length > MAIN_FRAME_POSITION && args[MAIN_FRAME_POSITION] != mstFrameMainName){
			appNamePosition += -1;
		}
		String appName;
		if(args.length > appNamePosition ){
			appName = args[appNamePosition];
			if(appName == null || appName.length() == 0){
				appName = extractAppNameFromClassName(this.mstFrameMainName);
			}else if(appName.toUpperCase().endsWith(xmlSuffix)){
				final int beginIndex = 0;
				final int endIndex   = appName.length() - xmlSuffix.length();
				appName = appName.substring(beginIndex, endIndex);
			}
		}else{
			appName = extractAppNameFromClassName(mstFrameMainName);
		}
		return appName;
	}
	private String extractMaschinenName() {
		if(this.originalAppParams != null && this.originalAppParams.length > 0){
			final int offset = (this.originalAppParams[0] != this.mstFrameMainName) ? 1 : 0;
			final int correctedMaschinePosition = MASCHINE_NAME_POSITION - offset;
			if(this.originalAppParams.length > correctedMaschinePosition){
				final String maschinenName = this.originalAppParams[correctedMaschinePosition];
				this.parameterMaschineWasSupplied = true;
				return maschinenName;
			}
		}
		this.parameterMaschineWasSupplied = false;
		return Maschine.getDefaultMaschine().getLangenNamen();
	}
	private String extractAppNameFromClassName(final String string) {
		this.parameterAppNameWasSupplied = false;
		final String regex = "[.]";//in character classes ist "." kein Meta Character
		final String[] strings = this.mstFrameMainName.split(regex);
		return strings[strings.length - 1];
	}

	private String[] extractParameter(final String[] args) {
		if(args.length < RESERVED_PARAMETER_COUNT){
			return new String[]{};
		}else if(args[0] != this.mstFrameMainName){
			return tailOfArgs(args, RESERVED_PARAMETER_COUNT - 1);
		}else return tailOfArgs(args, RESERVED_PARAMETER_COUNT);
	}

	/**
	 * @param 	args != null and length > 0
	 * @return	args[1..end] non null
	 */
	private String[] tailOfArgs(final String[] args, final int numberOfIndexesToRemove) {
		final int newLength = Math.max(0, args.length - numberOfIndexesToRemove);
		final String[] retval = new String[newLength];
		for (int i = 0; i < retval.length; i++) {
			retval[i] = args[i + numberOfIndexesToRemove];
		}
		return retval;
	}

	/**
	 * required: mstFrameMainName is set
	 * Kriterium: ? am ende von mstFrameMainName == erster Parameter
	 */
	private boolean internalHelpIsNeeded() {
		if(this.mstFrameMainName == null || this.mstFrameMainName.length() == 0){
			return true;
		}
		else if(this.mstFrameMainName.endsWith( new String( new char[]{'?'}) ) ){
			return true;
		}
		else {
			return false;
		}
	}

	//.........................................................................
	private String retrieveTitle(final ApplikationKind appKind, final String defaultTitle) {
		try {final String title = retrieveTitle(getAnLaApplicationInformation(appKind));
			 return (title == null || title.isEmpty()) ? defaultTitle : title;
		} catch (final Exception e) {return null;}
		
	}
	private IApp getAnLaApplicationInformation(final ApplikationKind appKind) {
		try						  {return appKind.getApplicationInformation();}
		catch(final Throwable th) {/*silently ignore*/ return null;}
	}
	private String retrieveTitle(final IApp appInfo) throws Exception{
		return appInfo == null ? null : appInfo.getTitle();
	}
	//.........................................................................

	
//    /**Set standard VM Properies of TINE*/
//	@Deprecated private void initTine() {
//        //-Dtine.home=L:\database
//        //-Ddebug.level=1
//        if(!LocalHome.noLocalHomeWanted()){
//            String tineVerz = LocalHome.calcLocalDirectory(TINE_SERVICE_NAME, TINE_VERZEICHNIS_NAME);
//            LocalHome.tryToCreateAllParentDirs(tineVerz);
//            if(!hasDatabaseVerzeichnis()){setDatabaseVerzeichnis(tineVerz);} //Do not overwrite explicit start parameter
//        }
//    }

    @Deprecated /**liefert startparameter korrekt, danach nicht mehr*/
	public final String getTineDBVerz() {
        return System.getProperty(TINE_DATABASE_SRV_HOME__KEY);
    }
    @Deprecated /**funktioniert nur bevor Tine gestartet ist*/
    final void setDatabaseVerzeichnis(final String tineVerz) {
        System.setProperty(TINE_DATABASE_SRV_HOME__KEY, tineVerz);
    }
//    @Deprecated /**liefert startparameter korrekt, danach nicht mehr*/
//    private boolean hasDatabaseVerzeichnis() {
//		return System.getProperties().containsKey(TINE_DATABASE_SRV_HOME__KEY);
//	}
    @Deprecated /**liefert startparameter korrekt, danach nicht mehr*/
    public final int getTineDebugLevel() {
        final String debugLevelString = System.getProperty(TINE_DEBUG_LEVEL__KEY);
        try                 {return Integer.parseInt(debugLevelString);}
        catch(final Exception e)  {return -999;}
    }
    @Deprecated /**funktioniert nur bevor Tine gestartet ist*/
    public final void setTineDebugLevel(final int tineDebugLevel) {
        System.setProperty(TINE_DEBUG_LEVEL__KEY, Integer.toString(tineDebugLevel) );
    }
	public final MaschineMgr getMaschineMgr() {
		return this.maschineMgr;
	}

	public final boolean getParameterAppNameWasSupplied() {
		return this.parameterAppNameWasSupplied;
	}
	public final boolean getParameterMaschineWasSupplied() {
		return this.parameterMaschineWasSupplied;
	}

//===================================================================
	/**Do not use - do not touch. For debug and system use only. Only for use in unit tests*/
	@Deprecated public static final class _ImplementationSwitches{
		/**Do not use - do not touch. For debug and system use only. Only for use in unit tests*/
		@Deprecated public static enum DEBUGSWITCHES{
			  /**Masterswitch for debug mode */ ISDEBUG
			, /**on...no webaccess */ LOCALSECURITY
			, /**on...no webaccess */ NOINIFILES
			, /**on...no fileaccess*/ NOFILELOGGING
			, /**on...no fileaccess*/ NOPREFERENCES
			, /**on...ignore Spider*/ NOSPIDER
			, /**on...ignore CAMFrameControl*/ NOCAMFRAMECONTROL
			;
			@Deprecated private static final String PREFIX = "DEBUGSWITCH.";
			@Deprecated public static final void switchAllTO(final boolean isOn){
				for (final DEBUGSWITCHES debugSwitch : values())
					debugSwitch.setOn(isOn);
			}
			@Deprecated public  static final void switchAllOff(){switchAllTO(false);}
			@Deprecated public  static final void switchAllOn (){switchAllTO(true);}
			//static{initFromEnvironment();}	//falls ein environment var suppport gewuenscht
			///**pp*/ static void initFromEnvironment() {
			//	for (DEBUGSWITCHES debugSwitch : DEBUGSWITCHES.values())
			//		initFromEnvironment(debugSwitch);
			//}
			//private static void initFromEnvironment(final DEBUGSWITCHES debugSwitch) {
			//	try {
			//		final String property = System.getProperty(debugSwitch.toString());
			//		final boolean on = (property != null && property.toLowerCase().equals("true"));
			//		debugSwitch.setOn(on);
			//	} catch (Exception ex) {
			//		Logger.SWFW.warn("Der DebugSwitch " + debugSwitch + " konnte nicht initialisert werden.", ex);
			//	}
			//}
			//-----------------------------------------------------------------
			private volatile boolean on = false;
			@Deprecated public final boolean setOn(final boolean on){final boolean old = isOn(); this.on = on; return old;}
			@Deprecated public final boolean  isOn()                {return this.on && ISDEBUG.on;}
			@Override   public final String  toString()             {return (PREFIX + super.toString()).toLowerCase();}
		}
	} 
}