/**
 * 
 * Last changed: $Date$
 * By: $Author$
 * URL: $HeadURL$
 * $Revision$ 
 *
 */
package de.desy.mst.libs.framework.mstapp.appinf;

import java.io.IOException;
import java.util.jar.JarFile;




/**
 * @author labudda
 *
 */
public class Lib implements ILib {
	private final int major;
	private final int minor;
	private final int revision;
	private final int build;
	private final String name;
	private final String buildTime;
	
	/**
	 * @param jar
	 * @throws IOException
	 */
	public Lib(final JarFile jar) throws IOException {
		this(jar, jar.getName());
	}

	/**
	 * @param jar
	 * @throws IOException 
	 */
	public Lib(final JarFile jar, final String name) throws IOException {
		assert jar != null: "Jar must not be null";
		this.name = name;
		final java.util.zip.ZipEntry ze = jar.getEntry("build.Number");
		if (ze == null) {
			this.buildTime = Lib.getChangedTimestamp(this.name);
			this.major = 0;
			this.minor = 0;
			this.revision = 0;
			this.build = 0;
		} else {
			this.buildTime = (new java.util.Date(ze.getTime())).toString();
			final java.io.InputStream is = jar.getInputStream(ze);
			assert is != null: "inputstram must not be null";
			final java.util.Properties properties = new java.util.Properties();
			properties.load(is);
				
			this.major = Integer.parseInt(properties.getProperty("major", "1"));
			this.minor = Integer.parseInt(properties.getProperty("minor", "0"));
			this.revision = Integer.parseInt(properties.getProperty("revision", "0"));
			this.build = Integer.parseInt(properties.getProperty("build.number", "1"));
			is.close();
		}
	}

	/**
	 * @param filename
	 * @return date of file last modified
	 */
	private static String getChangedTimestamp(final String filename) {
		long ts;
		final java.io.File file = new java.io.File(filename);
		ts = file.lastModified();
		final java.util.Date date = new java.util.Date(ts);
		return date.toString();
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.auxiliary.devl.versions.information.ILib#getBuild()
	 */
	@Override public int getBuild() {
		return this.build;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.auxiliary.devl.versions.information.ILib#getMajor()
	 */
	@Override public int getMajor() {
		return this.major;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.auxiliary.devl.versions.information.ILib#getMinor()
	 */
	@Override public int getMinor() {
		return this.minor;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.auxiliary.devl.versions.information.ILib#getName()
	 */
	@Override public String getName() {
		return this.name;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.auxiliary.devl.versions.information.ILib#getRevision()
	 */
	@Override public int getRevision() {
		return this.revision;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.auxiliary.devl.versions.information.ILib#getDate()
	 */
	@Override public String getDate() {
		return this.buildTime;
	}

}
