/**
 * 
 * Last changed: $Date$
 * By: $Author$
 * URL: $HeadURL$
 * $Revision$ 
 *
 */
package de.desy.mst.libs.framework.mstapp.appinf.server;

import java.io.IOException;

import de.desy.mst.libs.framework.mstapp.appinf.AApp;
import de.desy.mst.libs.framework.mstapp.appinf.ContactInformation;
import de.desy.mst.libs.framework.mstapp.appinf.IContactInformation;
import de.desy.mst.libs.framework.mstapp.appinf.ILib;
import de.desy.mst.libs.framework.mstapp.appinf.Lib;

/**
 * @author labudda
 *
 */
class ServerApp extends AApp {
	private final java.util.jar.Manifest manifest;
	private final java.util.jar.JarFile mainClassJarFile;
	/**
	 * @param mainClassJarFile 
	 * @param manifest
	 * @throws IOException 
	 */
	ServerApp(final java.util.jar.JarFile mainClassJarFile) throws IOException {
		this.mainClassJarFile = mainClassJarFile;
		this.manifest = mainClassJarFile.getManifest();
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.IApp#getDescription()
	 */
	@Override public String getDescription() {
		return this.manifest. getMainAttributes().getValue("Implementation-Description");
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.IApp#getHelp()
	 */
	@Override public String getHelp() {
		return this.manifest. getMainAttributes().getValue("Implementation-Help");
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.IApp#getJars()
	 */
	@Override public java.util.Collection<ILib> getJars() {
		final String[] jarNames = this.manifest.getMainAttributes().getValue("Class-Path").split(" ");
		final java.util.ArrayList<ILib> jars = new java.util.ArrayList<ILib>(); 
		for(final String jarName: jarNames) {
			try {
				final java.util.jar.JarFile jarFile = new java.util.jar.JarFile(jarName); 
				jars.add(new Lib(jarFile));
			} catch (final java.io.IOException e) {
				e.printStackTrace();
			}
		}
		return jars;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.IApp#getName()
	 */
	@Override public String getName() {
		return this.getProjectName();
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.IApp#getVersion()
	 */
	@Override public String getVersion() {
		return this.manifest.getMainAttributes().getValue("Implementation-Version");
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.IApp#getContact()
	 */
	@Override public IContactInformation getContact() {
		final String vendor = this.manifest.getMainAttributes().getValue("Implementation-Vendor");
		return new ContactInformation(vendor);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.IApp#getIcon()
	 */
	@Override public javax.swing.ImageIcon getIcon() {
		final String entryName = "resources/webstart/" + this.getProjectName() + ".png";
		System.out.println("Look for " + entryName);
		final java.util.zip.ZipEntry zipEntry = this.mainClassJarFile.getEntry(entryName);
		if (zipEntry == null) {
				System.out.println(entryName + " not found");
				return null;
			} else {
				try {
					final java.io.InputStream stream = this.mainClassJarFile.getInputStream(zipEntry);
					final java.awt.image.BufferedImage loaded = javax.imageio.ImageIO.read(stream);
					return new javax.swing.ImageIcon(loaded);
				} catch (final java.io.IOException e){
					return null;
				}
			}
	}

	/**
	 * @return name of project / name of ja file without path or extension
	 */
	private String getProjectName() {
		final java.io.File jarFile = new java.io.File(this.mainClassJarFile.getName());
		System.out.println(jarFile.getName());
		for (final String element :jarFile.getName().split(".")) {
			System.out.println(element);
		}
		for (final String element :jarFile.getName().split("\\.")) {
			System.out.println(element);
		}
		final String name = jarFile.getName().split("\\.")[0];
		System.out.println(name);
		return name;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.IApp#getTitle()
	 */
	@Override public String getTitle() {
		return this.manifest. getMainAttributes().getValue("Implementation-Title");
	}

}
