/**
 * ws:	Mar 29, 2007 - Apr 11, 2007 creation from scratch
 * ws:	Sep  4, 2007 - Sep  4, 2007 added SEP
 * ws:	Sep 10, 2007 - Sep 30, 2007 add: getCacheDateiPfad(), getCacheRoot()
 */
package de.desy.mst.libs.framework.mstapp.hilfe;

import java.io.File;

import de.desy.mst.libs.framework.mstapp.ApplicationInfo;
import de.desy.mst.libs.framework.mstapp.ApplicationInfo.TYPE;
import de.desy.mst.libs.framework.mstapp.ini.LocalHome;
import de.desy.mst.libs.framework.mstapp.logging.Logger;

/**
 * Liefert die Hilfepfade:
 *
 * @author ws
 * version 1.0.0
 */
public abstract class AHilfePfade implements IHilfePfade{
	protected static final String SEP	 = "/";	//Web Path Separator
	public final ApplicationInfo applicationInfo;

//=============================================================================

	/** @throws applicationsInfo may not be null*/
	public AHilfePfade(final ApplicationInfo applicationInfo) {
		super();
		if(applicationInfo == null){
			final String msg = "Das applicationInfo darf nicht null sein.";
			throw new NullPointerException(msg);
		}
		this.applicationInfo = applicationInfo;
	}

//=============================================================================
	abstract public String getAppOrALName(final ApplicationInfo applicationInfo);
//=============================================================================

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.hilfe.IHilfePfade#getSubsystemVerzeichnis(de.desy.mst.libs.framework.mstapp.ApplicationInfo)
	 */
	@Override public final String getSubsystemVerzeichnis(final ApplicationInfo applicationInfo) {
		final String accelerator = applicationInfo.getInfo(TYPE.ACCELERATOR);
		final String subSystem = applicationInfo.getInfo(TYPE.SUBSYSTEM);

		return getSubsystemVerzeichnis(accelerator, subSystem);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.hilfe.IHilfePfade#getDateiPfad(de.desy.mst.libs.framework.mstapp.ApplicationInfo)
	 */
	@Override public final String getDateiPfad(final ApplicationInfo applicationInfo) {
		final String accelerator = applicationInfo.getInfo(TYPE.ACCELERATOR);
		final String subSystem = applicationInfo.getInfo(TYPE.SUBSYSTEM);
		final String appName = getAppOrALName(applicationInfo);

		return getDateiPfad(accelerator, subSystem, appName);
	}


	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.hilfe.IHilfePfade#getDateiPfad()
	 */
	@Override public final String getDateiPfad() {return getDateiPfad(this.applicationInfo);}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.hilfe.IHilfePfade#getSubsystemVerzeichnis()
	 */
	@Override public final String getSubsystemVerzeichnis() {
		return getSubsystemVerzeichnis(this.applicationInfo);
	}


	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.hilfe.IHilfePfade#getCacheDateiPfad()
	 */
	@Override public final String getCacheDateiPfad() {
//		LocalHome.calcLocalPath(SRV_NAME, cleanAppname(appName), FILE_SUFFIX)
		if(!isCachable()){
			final String msg = "Nach Datei Cache gefragt,"
							 + "aber der Benutzer erlaubt keinen. "
							 + "Liefer NullString";
			Logger.SWFW.warn(msg);
			return "";
		}
		final String dateiPfad = getDateiPfad();
		final String BEGIN_OF_WEBADDRESS_MARKER = "//";
		final int beginIndex0 = dateiPfad.indexOf(BEGIN_OF_WEBADDRESS_MARKER)
							  + BEGIN_OF_WEBADDRESS_MARKER.length();
		final int beginIndex1 = dateiPfad.indexOf(SEP, beginIndex0)
							  + SEP.length();
		final String dateiPfadSpezifisch = dateiPfad.substring(beginIndex1);
		final String cacheDateiPfad =
			getCacheRoot() + dateiPfadSpezifisch.replace(SEP, File.separator);

		return cacheDateiPfad;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.hilfe.IHilfePfade#getCacheRoot()
	 */
	@Override public String getCacheRoot() {return LocalHome.getHome();}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.hilfe.IHilfePfade#isCachable()
	 */
	@Override public final boolean isCachable() {return !(LocalHome.noLocalHomeWanted());}

//=============================================================================
	public final ApplicationInfo getApplicationInfo(){
		return this.applicationInfo;
	}
//=============================================================================

	/**
	 * Die Blanks im appName werden fuer Web und File Bezeichnungen
	 * rausgenommen.
	 */
	public final static String cleanName(final String appName) {
		return appName.replace(" ", "");
	}
}