/*
 * Apr 19, 2005 - Jun 30, 2005	ws: creation
 * Oct 25, 2005 - ...			ws: new logging system
 * Feb 23, 2006 - ...			ws: Flache Dirs keine extra \
 * Nov  7, 2006 - ...			ws: added calcLocalUserPrefPath(...)
 * Mar 29, 2007 - ...			ws: versuche fuer Linux PrefStorage Probleme
 * 									abzufangen - kein local cach dann
 * May  9, 2007 - May 11, 2007	ws: askTheUser public, + decorate inside
 * 									added: setHome()
 * Feb  8, 2008 - Feb 11, 2008	ws:	noLocalHomeWanted() robuster,
 * 									isLocalHomeConfiguered()
 * Aug  8, 2008 - Aug  8, 2008	ws: HOME_DIR_NAME to public from pp for File_LogSourceFactory
 * Sep 16, 2008 - Sep 30, 2008	ws:	added: homeProtokoll, isAbweichendesProtokoll
 * Oct 16, 2008 - Oct 17, 2008	ws:	changed init() check for home starts with "null"
 * Dec 10, 2008 - Dec 10, 2008	ws:	MstSecurityMgr -> MstSecurity
 */
package de.desy.mst.libs.framework.mstapp.ini;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.prefs.Preferences;

import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;

import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.security.MstSecurity;

/**
 * Fuer Initialisierung und andere Dinge ist es nuetzlich Platz auf der
 * lokalen UserPlatte zu haben. Diese Klasse managed den Ort.
 * Der User kann das schreiben auf die lokale Platte verweigern.
 * Diese Klasse ist rein statisch und applikationsunabhaengig.
 * Sie ist nur MstApp spezifisch.
 *
 * Die Klasse ist package privat, da nur als teil des lokalen
 * bootstraps fuer ConfigMgr gebraucht.
 *
 *
 * @author schuette
 * @version 1.0.0
 */
public class LocalHome {
	/**Unter Folgendem Key ist das MstApp Home Directory auf dem lokalen PC zu finden*/
	private static final String           HOME_DIR_KEY     =           "home_directory";
	private static final String PROTOKOLL_HOME_DIR_KEY     = "protokoll_home_directory";
	private static final String PROTOKOLL_HOME_DIR_ABW_KEY = "protokoll_home_directory_abw";

	/**falls keine lokalen Daten gewuenscht ist, dies der Wert von mstApp_Home*/
	private static final String NOHOME_VALUE = "<keep_off/>";	//beachte kein legaler Verz. Name

	/**erstes Angebot an Benutzer fuer das Oberverzeichnies des Home Verzeichnisses*/
	private static final String DEFAULT_HOME_PARENTDIR = "c:\\";
	/**Wurzel des lokalen Verzeichnisses mit MstApp Daten*/
	public  static final String HOME_DIR_NAME = "mst_app";

    /**Falls no local storage wantet, returned by 'calcLocalPath(...)'*/
    private static final String NO_PATH = "NullDevice";

	/**Prefix fuer Preference User Storage*/
    private static final String PREF = "Pref_";
    //-------------------------------------------------------------------------
	/**
	 * Root Verzeichnis von MstApp Daten oder falls lokal keine Daten gewuenscht
	 * ist dieser Wert = noHome
	 */
	private static String home;
	private static String homeProtokoll;
	private static boolean isAbweichendesProtokoll;

	/** nie null, immer getrimmt
	 * PermanantStorage Storage on local PC at default location.
	 * In diesem Fall java.util.prefs.Preferences
	 * fr Windows ist dies die Registry und sieht auf meinem PC so aus:
	 * 	<li>Key Name:	"HKEY_LOCAL_MACHINE\SOFTWARE\JavaSoft\Prefs\de\desy\mst\mstapp\ini"
	 * 	<li>Value name:	"/Home/Directory"
	 * 	<li>Value data:	"/C://temp///Mst/App/Dt"
	 *
	*/
	final static class PermanentStorage{
		public static final String loadStringByKey(final String theKey, final String defaultValue) {
			try {
				return ("" + getPreferences().get(theKey, "" + defaultValue)).trim();

			} catch (final Exception e) {
				final String msg =
					"Kann die lokalen mstApp Systemeinstellungen nicht ffnen.";
				Logger.SWFW.warn(msg, e);
				return ("" + defaultValue).trim();
			}
		}
		public static final void storeStringByKey(final String theKey, final String theString) {
			try {
				getPreferences().put(theKey, "" + theString);//remember user-setting
	            final String msg = "PC weite Konfigurationsnderung: Bitte alle Applikationen neu starten!";
				JOptionPane.showMessageDialog(null, msg, "Konfigurationsnderung", JOptionPane.WARNING_MESSAGE);
			} catch (final Exception e) {
				final String msg =
					"Kann die lokalen mstApp Systemeinstellung ("
				  + theKey + ", " + theString + ") nicht schreiben. "
				  + "Lokales cashing ist "
				  + (noLocalHomeWanted()? "nicht ":"")
				  + "eingeschaltet.";
				Logger.SWFW.warn(msg, e);
			}
		}
		private static final Preferences getPreferences() {
			final Preferences preferences = Preferences.userNodeForPackage(LocalHome.class);
			return preferences;
		}
	}
	/**
	 * Access to the EnvironmentVariables of the PC as shown via the Java Virtual Machine
	 * A useful list of keys can be found in:
	 * Peter van der Linden "Just Java2" Table 19-6 on page 494
	 */
	final static class EnvironmentVariables{
		private static String fileSeparator;
		public static final String getFileSeparator() {
			if(fileSeparator == null){
				fileSeparator = loadStringByKey("file.separator", "/");
			}
			return fileSeparator;
		}
		/**
		 * equivalent to System.getProperty(key, defaultValue)
		 * but all Exception's go to logger and the default will be returned
		 */
		public static final String loadStringByKey(final String key, final String defaultValue) {
			try {
				return System.getProperty(key, defaultValue);
			} catch (final Exception e) {
				Logger.SWFW.warn("can not load '" + key + "'", e);
				return defaultValue;
			}
		}
	}

//=============================================================================

	public final static boolean noLocalHomeWanted(){
		final String home = getHome_Unchecked();
		return (home == null)
			|| (home.isEmpty())
			|| (home.startsWith(NOHOME_VALUE));
	}

	public final static boolean isLocalHomeConfiguered(){
		final String home = getHome_Unchecked();
		return  (home != null)
			&& !(home.isEmpty() );
	}

	/**
	 * @return Returns the mstApp_Home, falls "noLocalHomeWanted == false"
	 * @throws IllegalStateException falls "noLocalHomeWanted == true"
	 */
	public final static String getHome() {
		if(noLocalHomeWanted()){
			throw new IllegalStateException("The user does not want lokal Data.");
		}
		return getHome_Unchecked();
	}
	/**
	 * @return Returns the mstApp_Home, falls "noLocalHomeWanted == false"
	 * @throws IllegalStateException falls "noLocalHomeWanted == true"
	 */
	public static String getHomeProtokoll() {
		if(noLocalHomeWanted()){
			throw new IllegalStateException("The user does not want lokal Data.");
		}
		return getHomeProtokoll_Unchecked();
	}
	public static boolean hatAbweichendesProtokollVerzeichnis() {
		if(noLocalHomeWanted())
			return false;
		return isAbweichendesProtokoll;
	}


    /**
     * this routine asks the user for a home directory.
     * No movement is done.
     * But the new home directory is stored in the Java Prefs.
     */
    public static void changeHomeByUserDialog() {
        final String newHome = askTheUser(NOHOME_VALUE);
        setHome(newHome);
    }

    private static void setHome(final String newHome) {
		if(newHome != home){
            home = newHome;
            PermanentStorage.storeStringByKey(HOME_DIR_KEY, newHome);
        }
	}
    /**package privat*/
	static void setHome(final String newHome, final boolean isCachingErlaubt) {
		setHome(isCachingErlaubt ? decorateWithFileSeparator(newHome.trim())
								 : NOHOME_VALUE);
	}
	public static void setHomeProtokoll(String newHomeProtokoll, final boolean newIsAbweichendesProtokoll) {
		newHomeProtokoll = (newHomeProtokoll == null) ? "" : decorateWithFileSeparator(newHomeProtokoll.trim());
		if(newHomeProtokoll != homeProtokoll){
            homeProtokoll = newHomeProtokoll.equalsIgnoreCase(home) ? "" : newHomeProtokoll;
            PermanentStorage.storeStringByKey(PROTOKOLL_HOME_DIR_KEY, homeProtokoll);
        }
		if(newIsAbweichendesProtokoll != isAbweichendesProtokoll){
			isAbweichendesProtokoll = newIsAbweichendesProtokoll;
			PermanentStorage.storeStringByKey(PROTOKOLL_HOME_DIR_ABW_KEY, booleanToString(isAbweichendesProtokoll));
		}
	}

	private static String booleanToString(final boolean b) {
		return b ? "T" : "F";
	}
	private static boolean stringToBoolean(final String s) {
		if(s == null || s.isEmpty())
			return false;
		return s.toUpperCase().startsWith("T");
	}

    /**
     * @param srvName ...like "Ini"
     * @param appName ...application of the local path
     * @param suffix ....like ".xml"
     */
    public static final String calcLocalPath(final String srvName, final String appName, final String suffix) {
        return calcLocalPath(srvName, appName, appName, suffix);  //default filename is appname
    }
    /**
     * @param srvName ...like "Ini"
     * @param appName ...application of the local path
     * @param suffix ....like ".xml"
     */
    public static final String calcLocalUserPath(final String srvName, final String appName, final String suffix) {
        return calcLocalPath(srvName, appName, MstSecurity.get().getUser(), suffix);  //default filename is appname
    }
    /**
     * @param srvName ...like "Ini"
     * @param appName ...application of the local path
     * @param suffix ....like ".xml"
     */
    public static final String calcLocalUserPrefPath(final String srvName, final String appName, final String suffix) {
        return calcLocalPath(srvName, appName, PREF + MstSecurity.get().getUser(), suffix);
    }
    /**
     * @param srvName  ...like "Ini"
     * @param appName  ...application name for the path
     * @param fileName ...name of the file without path and suffix, like "mySpecialData"
     * @param suffix   ...like ".xml"
     */
    public static final String calcLocalPath(final String srvName, final String appName, final String fileName, final String suffix) {
        if (LocalHome.noLocalHomeWanted()) {
            return NO_PATH ;
        }else{
            return (calcLocalDirectory(srvName, appName) + fileName + suffix);
        }
    }

    /**
     * @param srvName ...like "Ini"
     * @param appName ...application name for the path
     * @return the complete path to directory
     */
    public static String calcLocalDirectory(final String srvName, final String appName) {
        if (LocalHome.noLocalHomeWanted()) {
            return NO_PATH ;
        }else{
            String appDirBez;
            if(appName != null && appName.length() > 0){
            	appDirBez = appName + File.separator;	//Tabellendaten werden flach abgespeichert
            }else{
            	appDirBez = "";
            }
			return (getHome() + srvName + File.separator + appDirBez );
        }
    }


    /**
     * @param lokalesVerz
     */
    public static void tryToCreateAllParentDirs(final String lokalesVerz) {
        if(LocalHome.noLocalHomeWanted()){return;}  //EXIT

        final File  file = new File(lokalesVerz);
        if (file.exists()) {return;}                //EXIT

        file.mkdirs();
        Logger.SWFW.info("Created new Directory: '" + lokalesVerz + "'");
    }

    /**
     * @param menuParent unter diesem Menu richtet sich das Local Home ein
     */
    public static void initAndRegisterJMenuForAllowLocalHome(final JMenu menuParent) {
        final JMenuItem item = new JMenuItem("Setze Basis Verzeichnis fr das lokale Speichern.");
        item.addActionListener(new ActionListener() {
        	@Override public void actionPerformed(final ActionEvent e) {
                changeHomeByUserDialog();
                final Object sourceAsObject = e.getSource();
                if(sourceAsObject instanceof Component){
                    final Component component = ((Component)sourceAsObject);
                    component.setEnabled(LocalHome.noLocalHomeWanted());
                }
                JOptionPane.showMessageDialog(
                        null
                      , "Bitte die Anwendung neu starten ein neues lokales "
                      + "Verzeichnis ist eingerichtet worden ('" + home + "'). "
                      + "Die Menues zum schreiben verweisen auf veraltete Pfade."
                      , "Bitte um Neustart.", JOptionPane.PLAIN_MESSAGE);
            }
        });
        item.setEnabled(LocalHome.noLocalHomeWanted());
        menuParent.add(item);
    }

//=============================================================================

	/**internal version that does not throw an error and is never null, hopefully*/
	private final static String getHome_Unchecked() {
		if(home == null){
			init();
		}
		return home;	//never null, hopefully
	}

	private static final void init() {
		home = null;
		homeProtokoll = null;
		try {
			final String defaultValue = null;	//Beachte: 'null != NOHOME'
			home = PermanentStorage.loadStringByKey(HOME_DIR_KEY, defaultValue);
			if(home == null || home.isEmpty() || home.startsWith("null"))
				changeHomeByUserDialog();

			homeProtokoll = PermanentStorage.loadStringByKey(PROTOKOLL_HOME_DIR_KEY, defaultValue);
			if(homeProtokoll.isEmpty() || homeProtokoll.equals(home) || homeProtokoll.startsWith("null"))
				homeProtokoll = null;

			final String defaultB = booleanToString((homeProtokoll == null) ? false : !homeProtokoll.equals(home));
			final String stringValue = PermanentStorage.loadStringByKey(PROTOKOLL_HOME_DIR_ABW_KEY, defaultB);
			isAbweichendesProtokoll = stringToBoolean(stringValue);
		} catch (final Exception e) {
			home = NOHOME_VALUE;	//Speicher dieses Setting aber nicht ab
		}
	}

	public static final String askTheUser(final String defaultHome) {
		final JFileChooser fileChooser = new JFileChooser(defaultParentDir(defaultHome));
		fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		fileChooser.setApproveButtonText("OK (Select this Parent Directory)");
		fileChooser.setDialogTitle("Root Directory for 'MstApp Files Directory'. Default: '"
				                   + DEFAULT_HOME_PARENTDIR + "'.");
		if(JFileChooser.APPROVE_OPTION == fileChooser.showOpenDialog(new JFrame())){

			//waits here until user clicks ok or cancel

			final File dir = fileChooser.getSelectedFile(); //ja, dirs sind files
			if(dir == null || dir.getPath() == ""){
				return defaultHome; //nothing selected, so stay with original
			}
			final String parentDirectoryName = dir.getPath();
			return decorateWithFileSeparator(calcHome(parentDirectoryName));
		}else
			//user pressed cancel, stay with original value for now,
			// but do not remember after lifetime of this app
			return decorateWithFileSeparator(defaultHome);
			//Possible Improvement:
			//use an Accessory (siehe Harold "JAVA IO" p. 376f) to ask user,
			//if he wants NOHOME to be kept permanently
	}

	private static String calcHome(final String parDir) {
		if(parDir == NOHOME_VALUE)
			return NOHOME_VALUE;
		else
			return decorateWithFileSeparator(parDir) + decorateWithFileSeparator(HOME_DIR_NAME);
	}

	private static final String defaultParentDir(final String defaultHome) {
		if(defaultHome == NOHOME_VALUE || defaultHome == null || defaultHome == "")
			return DEFAULT_HOME_PARENTDIR;
		else
			return parentDir(defaultHome, DEFAULT_HOME_PARENTDIR);
	}

	private static final String parentDir(String dir, final String defaultDir) {
		dir = defaultDir;
		try {
			final File file = new File(dir);
			dir = file.getParent();
			if(dir != null)
				dir += EnvironmentVariables.getFileSeparator();
		} catch (final Exception ignore) {}

		return dir;
	}

	/**@throws NullpointerException*/
	private static final String decorateWithFileSeparator(final String dirName){
		final String fileSeparator = EnvironmentVariables.getFileSeparator();
		if(NOHOME_VALUE.equalsIgnoreCase(dirName)
		|| dirName.endsWith(fileSeparator))
			return dirName;

		return dirName + fileSeparator;
	}


	private static String getHomeProtokoll_Unchecked() {
		if(homeProtokoll == null
		|| homeProtokoll.isEmpty()
		|| homeProtokoll.equals("null")
		|| !isAbweichendesProtokoll)
			return getHome_Unchecked();	//Default same as Home
		return homeProtokoll;	//never null, hopefully
	}
}
