/**
 * Dec  6, 2007 - Dec 19, 2007	ws:	Creation from scratch
 * Jun  3, 2008 - Jun  3, 2008	ws: catch Exception statt only RunTimeException
 * 									Feedback Problem mit .error statt .warn
 * 					  27			createBadDeformatMsg mit throwable
 * Aug  1, 2008 - Aug  4, 2008	ws:	added:	hasMassiveDecodingProblems()
 * Apr 29, 2009 - Apr 30, 2009	ws:	added:	getFormatter()
 * 									updated:deformatSpaltenTitel()
 * 									updated:updateIndexes() mit TEXTKONSTANTEN
 * May 19, 2009 - May 19, 2009	ws:	added:	createLogMsg()
 */
package de.desy.mst.libs.framework.mstapp.logging;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.UUID;
import java.util.concurrent.atomic.AtomicLong;
import de.desy.mst.libs.framework.mstapp.util.MstDateFormatISOParser;

/**
 * Liest das Ergebnis des ILogFormatter's zurueck.
 * @author ws
 * @version 1.0.0
 */
public class EinfacherLogDeformatter {
	private final int NCOLUMNS = 9;	//Default Columns
	/**The nrSurrogate is negative to distinguish from proper ones*/
	private static final AtomicLong nrSurrogate = new AtomicLong(Long.MIN_VALUE);
//=============================================================================
	private static long getNextNrSurrogate() {
		return nrSurrogate.incrementAndGet();
	}
//=============================================================================
	private class SpaltenIndex {
		public final static int INVALID = Integer.MIN_VALUE;
		//---------------------------------------------------------------------
		private int nr;
		private int level;
		private int datum;
		private int applikation;
		private int pcName;
		private int kategorie;
		private int meldung;
		private int runId;
		private boolean hasBadHeader = false;
		//---------------------------------------------------------------------
		public SpaltenIndex() {super(); invalidate();}
		//---------------------------------------------------------------------
		public void updateIndexes(final String[] dieTitel) {
			this.hasBadHeader = false;
			//Nr	Level	Datum	Kategorie	Meldung	Applikation	RunId	PCName
			final HashMap<String, Integer> map = new HashMap<String, Integer>();
			for (int i = 0; i < dieTitel.length; i++) {
				map.put(dieTitel[i], Integer.valueOf(i));
			}
			this.nr 			= updateIndex(map, EinfacherLogFormatter.HEADER_NR);
			this.level 		= updateIndex(map, EinfacherLogFormatter.HEADER_LEVEL);
			this.datum 		= updateIndex(map, EinfacherLogFormatter.HEADER_DATUM);
			this.kategorie 	= updateIndex(map, EinfacherLogFormatter.HEADER_KATEGORIE);
			this.meldung 	= updateIndex(map, EinfacherLogFormatter.HEADER_MELDUNG);
			this.applikation	= updateIndex(map, EinfacherLogFormatter.HEADER_APPLIKATION);
			this.runId 		= updateIndex(map, EinfacherLogFormatter.HEADER_RUN_ID);
			this.pcName 		= updateIndex(map, EinfacherLogFormatter.HEADER_PCNAME);

			if(map.size() > 0){
				final String msg =
					  "EinfacherLogReader$SpaltenIndex.updateIndexes"
					+ " hat unbekannte Titel gefunden."
					+ " Folgende Titel werden ignoriert: "
					+ map.toString();
				Logger.SWFW.warn(msg);
			}
		}
		private int updateIndex(final HashMap<String, Integer> map, final String key) {
			if(map.containsKey(key)){
				final Integer retVal = map.get(key);
				map.remove(key);
				return  retVal.intValue();
			}else{
				this.hasBadHeader = true;
				return INVALID;
			}
		}
		private void invalidate(){
			final String[] empty = {};
			updateIndexes(empty);
		}
		public boolean hasBadHeader() {return this.hasBadHeader;}
	}

	private final SpaltenIndex index = new SpaltenIndex();
	private final EinfacherLogFormatter formatter;
	private       String[] spaltenTitel = {};

//=============================================================================
	/**
	 * @param  formatter not Null
	 * @throws NullPointerException
	 */
	public EinfacherLogDeformatter(final EinfacherLogFormatter formatter) {
		super();
		if(formatter == null){
			final String msg = "Der LogFormatter darf nicht null sein.";
			throw new NullPointerException(msg);
		}
		this.formatter = formatter;
	}
//=============================================================================
	/**Ordnet den Spalten die richtigen Werte zu*/
	public void deformatSpaltenTitel(final String dieSpaltenTitel) {
		final String sep = this.formatter.getSEPARATOR_InMSG();
		final String escapedSep = sep.length() == 1 ? squareBracket(sep) : sep;
		this.spaltenTitel = dieSpaltenTitel.split(escapedSep);
		this.index.updateIndexes(this.spaltenTitel);
	}
	private String squareBracket(final String sep) {return "[" + sep + "]";}

	/**Tries to reconstruct the original Message*/
	public LogMsg deformatMessageEntry(final String logMsgSerialized) {
		if(logMsgSerialized == null ||logMsgSerialized.isEmpty()){
			return null;
		}
		final List<String> colValues = splitMsgToList(logMsgSerialized, this.formatter.getSEPARATOR_InMSG());
		if(colValues.size() <= 1)
			throw new IllegalArgumentException("Message hat zu wenige Spalten (" + colValues.size() + ").");
		final LogMsg logMsg = deformatMessageEntry(colValues);
		return logMsg;
	}
	public boolean hasBadHeader() {
		return this.index.hasBadHeader(); 
	}

//=============================================================================

	/**pp for testing*/List<String> getSpaltenTitel() {
		return Arrays.asList(this.spaltenTitel);
	}

//=============================================================================

	private List<String> splitMsgToList(final String logMsgSerialized, final String sep) {
		//normales String.split(regEx) geht nicht, da bei leerem item kein Array
		//Eintrag gemacht wird
		final List<String> list = new ArrayList<String>(this.NCOLUMNS);
		String string = logMsgSerialized;
		while(true){
			final int beginIndex = string.indexOf(sep);
			final boolean noMoreFound = (beginIndex == -1);
			if(noMoreFound){
				list.add(string);
				break;
			}
			final String item = string.substring(0, beginIndex);
			list.add(item);
			string = string.substring(beginIndex + sep.length());
		}
		return list;
	}

	private LogMsg deformatMessageEntry(final List<String> colValues) {
		final LogMsg logMsg = createLogMsg(
			deformat_Nr(		extractOrEmpty(colValues, this.index.nr))
		  , deformat_Level(		extractOrEmpty(colValues, this.index.level))
		  , deformat_Date(		extractOrEmpty(colValues, this.index.datum))
		  , 					extractOrEmpty(colValues, this.index.applikation)
		  , 					extractOrEmpty(colValues, this.index.pcName)
		  , deformat_Kategorie(	extractOrEmpty(colValues, this.index.kategorie))
		  , deformat_Meldung(	extractOrEmpty(colValues, this.index.meldung))
		  , null	//			XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX.throwable
		  , deformat_RunId(		extractOrEmpty(colValues, this.index.runId))
									   );
		return logMsg;
	}
	private String extractOrEmpty(final List<String> colValues, final int i) {
		return (	0 <= i
				&&	i < colValues.size()) ?
				colValues.get(i) :
				"";
	}
	private long deformat_Nr(final String value) {
		return (value == null ||
				value.isEmpty()  ) ?
						getNextNrSurrogate() :
						new Long(value).longValue();
	}
	private LogLvl deformat_Level(final String value) {
		try {
			return LogLvl.valueOf(value);
		} catch (final IllegalArgumentException e) {
			return null;	//finde das level nicht, LogMst XTor loest null auf
		}
	}
	private Date deformat_Date(final String value) {
		Date date;
		try {
			date = MstDateFormatISOParser.parse(value);
		} catch (final ParseException e) {
			Logger.SWFW.warn( "EinfacherLogReader.deformat_Date("
							+ value
							+ ") kann nicht parsen setze zu new Date(0).");
			date = new Date(0);
		}
		return date;
	}
	private Logger deformat_Kategorie(final String value) {
		try {
			return Logger.valueOf(value);
		} catch (final IllegalArgumentException e) {
			return  null;	//finde die Kateg. nicht, LogMst XTor loest Any auf
		}
	}
	protected String deformat_Meldung(final String value) {
		final String decodedMeldung = this.formatter.decode(value);
		final String deDecoratedMeldung = deDecorate(decodedMeldung);
		return deDecoratedMeldung;
	}
	/**pp for testing, remove " at the ends if they exist*/
	String deDecorate(final String string) {
		if(string == null || string.isEmpty() || string.equals("\"")){
			return "";
		}
		final int beginIndex = (string.charAt(0) == '\"') ? 1 : 0;
		final int lMin1 = string.length() - 1;
		final int endIndex = (string.charAt(lMin1)=='\"')?(lMin1) :(lMin1+1);
		final String dedecoratedString = string.substring(beginIndex, endIndex);
		return dedecoratedString;
	}
	private UUID deformat_RunId(final String value) {
		try {
			return  UUID.fromString(value);
		} catch (final IllegalArgumentException e) {
			return null;
		}
	}
	public LogMsg createBadDeformatMsg(final String line, final Throwable e) {
		final String msg = "Eine LogMsg kann nicht konstruiert werden aus: '"
						 + line + "'.";
		return new LogMsg(LogLvl.error, Logger.SWFW, msg, e);
	}
	protected EinfacherLogFormatter getFormatter() {return this.formatter;}
	/**for internal use of deformatters only*/
	@SuppressWarnings("deprecation")
	protected final LogMsg createLogMsg(final long nr
									  , final LogLvl 	level
									  , final Date 		date
									  , final String 	applikation
									  , final String 	pcName
									  , final Logger 	kategorie
									  , final String 	message
									  , final Throwable throwable
									  , final UUID	runId
	){return new LogMsg(nr, level, date, applikation, pcName, kategorie, message, throwable, runId);}
}
