/**
 * Jun 16, 2009 - Jun 24, 2009	ws:	Creation from scratch
 */
package de.desy.mst.libs.framework.mstapp.logging.appender.file;

import java.io.File;
import java.util.LinkedList;
import java.util.List;
import de.desy.mst.libs.framework.mstapp.ApplicationInfo;
import de.desy.mst.libs.framework.mstapp.logging.EinfacherLogFormatter;
import de.desy.mst.libs.framework.mstapp.logging.ILogFormatter;
import de.desy.mst.libs.framework.mstapp.logging.LMTag;
import de.desy.mst.libs.framework.mstapp.logging.filter.HauptLogFilter;

/**
 * Paket Private Klasse
 * <br> Enthaelt die LMTag Spezifische Zusatzinfo um ein LogAppenderFile zusammen
 * mit dem Application Info konfigurieren zu koennen.
 * <br> Design Idee: (1) Tags so simpel wie Moeglich (Name, invers)
 * <br> Design Idee: (2) LogAppenderFile sollte moeglich nichts spezifisches
 * 						 jenseits eines Defaults enthalten.
 * <br> Last: muss fuer jeden neuen Datei Tag mitgeplegt werden.
 * @author ws
 * @version 1.0.0
 */
class __LMTagFilePeer {
	/**synchronized(__LMTagFilePeer.class)*/
	private static final List<__LMTagFilePeer> instantiatedPeers = new LinkedList<__LMTagFilePeer>();
	//-----------------------------------------------------------------------------
	public static final __LMTagFilePeer PNachDateiStdLog = new __LMTagFilePeer(LMTag.NachDateiStdLog);

	/** @return vorher registrierten peer oder null*/
	public static __LMTagFilePeer get(final LMTag tag) {
		synchronized(__LMTagFilePeer.class) {
			for (__LMTagFilePeer peer : instantiatedPeers)
				if(peer.getTag().equals(tag))
					return peer;
		}
		return null;
	}
//=============================================================================
	private final LMTag tag;
//-----------------------------------------------------------------------------
	__LMTagFilePeer(final LMTag tag) throws NullPointerException{
		if(tag == null)
			throw new NullPointerException("Der Tag darf nicht null sein.");
		this.tag = tag;
		synchronized(__LMTagFilePeer.class) {instantiatedPeers.add(this);}
	}
//-----------------------------------------------------------------------------
	public final LMTag getTag() {return this.tag;}
	public IFileTargetsByDate createFileTargetsByDate(){
		return new FTWochentag(null);
	}
	public String calcPath(ApplicationInfo info) {
		final String mitDateiName = new PfadeProtokoll(info).getCacheDateiPfad();
		if(mitDateiName.isEmpty())
			return "";
		final int beginDateiName = mitDateiName.lastIndexOf(File.separator);
		if(beginDateiName < 0)
			return "";
		final String dateiName = mitDateiName.substring(beginDateiName);
		final String ohneDateiName = mitDateiName.replace(dateiName, "");
		return ohneDateiName;
	}
	public ILogFormatter createLogFormatter() {
		return EinfacherLogFormatter.createNopadNothrowableFormatter();
	}
	public HauptLogFilter createHauptLogFilter() {
		return new HauptLogFilter(getTag());
	}
	public final LogAppenderFile createLogAppenderFile(final ApplicationInfo info){
		return 	new LogAppenderFile(createFileTargetsByDate()
								  , calcPath(info)
								  , createLogFormatter()
								  , createHauptLogFilter());
	}
}

