/**
 * May 5, 2009 - May 6, 2009	ws:	Creation from scratch
 */
package de.desy.mst.libs.framework.mstapp.logging.appender.tinelogsrv;

import java.text.DateFormat;
import java.util.Date;
import java.util.Observable;
import java.util.concurrent.atomic.AtomicLong;

import de.desy.mst.libs.framework.mstapp.util.MstDate;
import de.desy.mst.libs.framework.mstapp.util.MstDateFormat;

/**
 * Managed die Timinginformation, Aenderungen nur via updateForNextHeartBeat():
 * <li>HeartBeat	(Zaehlt durch)
 * <li>StartTime	(Anfangszeitpunkt)
 * <li>Intervall	(wann der Sollheartbeatzeitpunkt)
 * <li>Wann wieder HeartBeat (im Raster StartTime + n * Intervall)
 * @author ws
 * @version 1.0.0
 */
/**pp*/ class HeartBeat extends Observable{
	private static final long MSINTERVAL = 24L*3600L*1000L;	//Default 24h
	private static final DateFormat DATEFORMAT = MstDateFormat.ISO;
//=============================================================================
	private final AtomicLong hbCount  = new AtomicLong();
	private final AtomicLong msHBZeit = new AtomicLong();
	private final Date daStartzeit;
	private final long msInterval;
//=============================================================================
	public HeartBeat(final long msStartzeit){this(msStartzeit, MSINTERVAL);}
	/**@param msStartzeit ... Programm Start
	 * @param msInterval  ... Zeitraum zwischen zwei Heartbeats*/
	public HeartBeat(final long msStartzeit, final long msInterval) {
		this.daStartzeit = new Date(msStartzeit);
		this.msInterval  = msInterval;

		this.hbCount .set(0);
		this.msHBZeit.set(msStartzeit);
	}
//=============================================================================
	/**einzige modifizierende Funktion: hbCount++, msHBZeit geaendert*/
	public final void updateForNextHeartBeat(final long currentTimeMillis)	{
		this.msHBZeit.set(nextHBZeit(getMSHBZeit(), currentTimeMillis, getMsInterval()));
		this.hbCount.incrementAndGet();
	}
	public final long   getHBCount ()	{return this.hbCount.get();}
	public final long   getMSHBZeit()   {return this.msHBZeit.get();}

	public final long   getMsInterval() {return this.msInterval;}

	public final Date   getDaStartzeit(){return this.daStartzeit;}
	public final long   getMsStartzeit(){return getDaStartzeit().getTime();}
	public final String getTxStartzeit(){return format(getDaStartzeit());}

	/**Dauer ... Zeitraum Programm Start bis End typisch currentTimeMillis*/
	public final long 	getMSDauer(final long msEnd  ) {return msEnd - getMsStartzeit();}
	public final String getTxDauer(final long msDauer) {return MstDate.msIntervalToText(getMSDauer(msDauer));}
//=============================================================================
	/**
	 * Berechnung der naechsten HBZeit
	 * <br>package privat und now explizit zum testen
	 * @param old 	alte HBZeit
	 * @param now 	jetzt
	 * @param delta	Intervall
	 * @return		erste Sollzeit mit (old + n * delta) >= now
	 *              wobei n = 0,1,2,...
	 *              (festes Raster, nicht notwendig in synch mit getHBCount(),
	 *              koennte ja HeartBeats ausgelassen haben)
	 */
	static final long nextHBZeit(final long old, final long now, final long delta) {
		if(now < old)
			return old;		//keine neue Zeit noetig

		final long cand = old + delta;
		if(cand >= now)
			return cand;	//erste Sollzeit in der Zukunft

		final long nIntervall = 1L + (now - old) /delta;
		return old + nIntervall * delta;	//erste Sollzeit nach now
	}
//=============================================================================
	private final DateFormat getDateFormat() {return HeartBeat.DATEFORMAT;}
	private final String format(final Date d){return getDateFormat().format(d);}
}
