/**
 * Apr 20, 2009 - May  5, 2009	ws:	Creation from scratch
 */
package de.desy.mst.libs.framework.mstapp.logging.appender.tinelogsrv;

import static de.desy.tine.server.logger.ClsLog.ClogPriority.CLOG_PRIORITY_IMPORTANT;
import static de.desy.tine.server.logger.ClsLog.ClogPriority.CLOG_PRIORITY_NONE;

import java.io.IOException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import de.desy.mst.libs.framework.mstapp.Main;
import de.desy.mst.libs.framework.mstapp.logging.LogLvl;
import de.desy.mst.libs.framework.mstapp.logging.LogMsg;
import de.desy.mst.libs.framework.mstapp.logging.appender.ALogAppender;
import de.desy.mst.libs.framework.mstapp.logging.filter.HauptLogFilter;
import de.desy.tine.server.equipment.TEquipmentModuleFactory;
import de.desy.tine.server.logger.Clog;
import de.desy.tine.server.logger.ClogFilter;
import de.desy.tine.server.logger.ClsLog;
import de.desy.tine.server.logger.ClsLog.ClogPriority;
import de.desy.tine.server.logger.ClsLog.ClogStatus;

/**
 * MstLogging Appender for Output to TineServer
 * @author ws
 * @version 1.0.0
 */
public class LogAppenderTine extends ALogAppender{
    public  final static String LOGGER = "MstAppTineLog";
    private final static String NAME   = "LogAppenderTine";
    private final static String txtSep = " | ";
//=============================================================================
	public LogAppenderTine(final HauptLogFilter hauptLogFilter) {
		super(hauptLogFilter);
	}
//=============================================================================
	@Override public String getName() {return NAME;}
	@Override public void flush() {}	//wird ungepuffert geschrieben
	@Override public Element getBeschreibungAsElement(final Document document) {
		return getBeschreibungAsElement(document, NAME);
	}
	@Override public void doWrite(final LogMsg logMsg) {
		final String text = extractClogText(logMsg);	//maximum erste 256 chars
		final String context = calcContext();
		final String tag = Main.getAppName();
		final String caller = LOGGER;
		final ClogPriority priority = (logMsg.getLevel() == LogLvl.fatal) ? CLOG_PRIORITY_IMPORTANT : CLOG_PRIORITY_NONE;
		final ClogStatus status = levelToStatus(logMsg.getLevel());
		ClsLog.log(text, context, tag, caller, priority, status);
	}
//=============================================================================
	/**pp*/ @SuppressWarnings("deprecation") String calcContext() {
		final String context = TEquipmentModuleFactory.getInstance().getFecContext();
		if(context != null)
			return context;
		if(Main.getStartParameter() == null)
			return "TEST";
		return "?CONTEXT?";
	}
	/**pp*/ ClogStatus levelToStatus(final LogLvl level) {
		final int ordinal = level.ordinal();
		final ClogStatus[] values = ClogStatus.values();
		final int index = Math.min(ordinal, values.length - 1);
		return values[index];
	}
	/**pp*/ Clog[] read(final long start, final long stop) throws IOException {
		final String text = null;
		final String context = null;
		final String tag = null;
		final String caller = LogAppenderTine.LOGGER;
		final ClogFilter clogFilter = new ClogFilter(text
												   , context
												   , tag
												   , caller
												   , 0	//CLOG_PRIORITY_NONE.ordinal()
												   , 0	//ClogStatus.CLOG_STATUS_NONE.ordinal()
												   , start
												   , stop);
		final Clog[] entries = ClsLog.getEntries(new ClogFilter[] {clogFilter});
		return entries;
	}
	/**pp ohne msg.getDate()*/ String extractClogText(final LogMsg logMsg) {
		//public final long   nr;		//local count um Messages nach Erzeugung sortieren zu koennen
		//public final LogLvl level;
		//public final Date	date;
		//public final String applikation;
		//public final String pcName;
		//public final Logger kategorie;
		//public final String message;
		//public final Throwable throwable;
		//public final UUID   runId;	//UUID dieser Message
		return    ""     + logMsg.getNr()
				+ txtSep + logMsg.getRunId()
				+ txtSep + logMsg.getKategorie()
				+ txtSep + logMsg.getMessage()
				;
	}
}
