/**
 *  Created on	Jan 12, 2006 - Feb 02, 2006
 *  			Jul 30, 2006 - Aug 01, 2006, ws: implement: set, equals, hashCode
 *	Feb 15, 2008 - Feb 15, 2008 ws: added "@Override"es
 *  Oct 30, 2008 - Oct 30, 2008	ws:	added setFromSaveString(...), getSaveString(...)
 */
package de.desy.mst.libs.framework.mstapp.logging.filter;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import de.desy.mst.libs.framework.mstapp.ini.PrefStorage;
import de.desy.mst.libs.framework.mstapp.logging.LogLvl;
import de.desy.mst.libs.framework.mstapp.logging.LogMsg;
import de.desy.mst.libs.framework.mstapp.logging.Logger;

/**
 * <b>Leaffilter fuer Logger Logging Levels (Debug, Fatal etc.).</b><br>
 * Laesst alle Messages ab dem angegebenen LogLvl einschlieslich durch<br>
 * Also ein level info laesst info aber auch error Messages durch. Debug Messages werden ausgefiltert.<br>
 * Will man alles durchlassen setzt man den LogLvl zu null oder debug<br>
 * <br>
 * Einige Spezialfaelle:<br>
 * (LogLvl == null)       pass liefert (true)<br>
 * (LogLvl == Logger.Any) pass liefert (Message   != null)<br>
 * (Message         == null)       pass liefert (Kategorie == null)<br>
 *
 * @author ws
 * @version 1.0.0
 */
public class LogFilterLogLvl extends ALogFilter {
	private static final String NAME = "LogLevelFilter";
	private static final String ATTRIBUTNAME = "LogLevel";

	private static final LogLvl DEFAULT_LOG_LVL = LogLvl.debug;
	private LogLvl logLvl; //never null
/*	  debug	//0
	, info	//1
	, warn	//2
	, error	//3
	, fatal	//4
*/
//=============================================================================

	public LogFilterLogLvl() {this(DEFAULT_LOG_LVL);}
	public LogFilterLogLvl(LogLvl lvl) {
		super();
		this.setLogLvl(lvl);
	}
	public LogFilterLogLvl(LogFilterLogLvl logLogFilterLogLvl) {
		this((LogLvl) null);
		this.set(logLogFilterLogLvl);
	}

//=============================================================================

	/* (non-Javadoc)
	 * @see de.desy.mst.mstapp.logging.filter.ALogFilter#passOnStandardModus(de.desy.mst.mstapp.logging.LogMsg)
	 */
	@Override protected boolean passOnStandardModus(final LogMsg logMsg){
		return logMsg.getLevel().istErnsterAls(this.getLogLvl());
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.mstapp.logging.filter.ALogFilter#getName()
	 */
	@Override protected String getName() {return NAME;}
	/* (non-Javadoc)
	 * @see de.desy.mst.mstapp.logging.filter.ALogFilter#decorateElement(org.w3c.dom.Document, org.w3c.dom.Element)
	 */
	@Override protected void decorateElement(Document document, Element element) {
		element.setAttribute(ATTRIBUTNAME, getLogLvl().toString());
	}

//=============================================================================

	/**never null*/
	public LogLvl getLogLvl() {return this.logLvl;}
	public void setLogLvl(LogLvl logLvl) {
		this.logLvl = (logLvl == null ? DEFAULT_LOG_LVL : logLvl);
	}
	/**Copy Semantik*/
	public void set(final LogFilterLogLvl logFilterLogLvl){
		super.set(logFilterLogLvl);
		this.setLogLvl(logFilterLogLvl.getLogLvl());
	}

	@Override public boolean equals(Object obj) {
		if(obj instanceof LogFilterLogLvl){
			LogFilterLogLvl filter = (LogFilterLogLvl) obj;
			return super.equals(filter)
			    && this.getLogLvl().ordinal() == filter.getLogLvl().ordinal()
			    ;
		}
		return false;
	}
	@Override public int hashCode() {
		return this.getLogLvl().ordinal()
		     ;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.filter.ALogFilter#getSaveString(java.lang.String, java.lang.String)
	 */
	@Override protected String getSaveString(final String appenderGruppe, final String saveString) {
		final String key = makeKey(appenderGruppe);
		final String value = getLogLvl().toString();
		return PrefStorage.serialize(saveString, key, value);
	}
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.filter.ALogFilter#setFromSaveString(java.lang.String, java.lang.String)
	 */
	@Override protected void setFromSaveString(final String appenderGruppe, final String saveString, final Object optionalDefault) {
		final String key = makeKey(appenderGruppe);
		final String value = PrefStorage.extract(saveString, key);
		try {
			final LogLvl lvl = (value == null || value.isEmpty()) ?
							   (LogLvl) optionalDefault : LogLvl.valueOf(value);
			this.setLogLvl(lvl);
		} catch (final IllegalArgumentException ex) {
			Logger.SWFW.warn("Kann den Filter " + this.getName()
						   + " nicht zu dem LogLvl '" + value
						   + "' setzen ignorier den Wert und belasse ihn auf '"
						   + getLogLvl().toString() + "'.");
		} catch (final NullPointerException ex) {
			Logger.SWFW.warn("Kann den Filter " + this.getName()
						   + " nicht zu dem LogLvl 'null' setzen"
						   + ", belasse ihn auf '" + getLogLvl().toString()
						   + "'.");
		}
	}
	//-------------------------------------------------------------------------
	private String makeKey(final String appenderGruppe) {
		return makeKey(appenderGruppe, ATTRIBUTNAME);
	}
}
