/**
 *  Created on	Feb  2, 2006 - Feb 27, 2006
 *  Jul 26, 2006 - Aug  1, 2006	ws: neuer Copy XTor StdLogFilter(StdLogFilter filter), set
 *  Oct 15, 2008 - Nov  4, 2008	ws:	added toString(...), setFromSaveString(...), getSaveString(...)
 * 	Jun  8, 2009 - Jun 26, 2009	ws:	add Tagging Support, BlocksAnyLevel Support
 * 	Jan 11, 2010 - Jan 11, 2010	ws:	getSaveString() now only serializes explicitly non default levels and kategories
 */
package de.desy.mst.libs.framework.mstapp.logging.filter;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import de.desy.mst.libs.framework.mstapp.ini.PrefStorage;
import de.desy.mst.libs.framework.mstapp.logging.LMTag;
import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.logging.LogLvl;
import de.desy.mst.libs.framework.mstapp.logging.LogMsg;

/**
 * Enthlt alle Standard Unterabfragen des LogFilters.
 * Falls mehr gebraucht werden muss genau diese Klasse
 * schnittstellen-konform erweitert werden.
 * <br><br>
 * Pass im Standard Mode, falls
 * <br> logMsg.hasTag(getTag())
 * oder smtliche folgende SubFilter passen.
 * <br><br>
 * SubFilter nach:
 * <li>(not logMsg.hasTag(getTag().invertiert))
 * <li>Kategorie
 * <li>LogLvl
 *
 * @author ws
 * @version 1.0.0
 */
public class StdLogFilter extends ALogTaggedFilter {
	public static final String NAME = "StdLogFilter";

	public static final Logger DEFAULT_LOG_KATEGORIE	= Logger.Any;
	public static final LogLvl DEFAULT_LOG_LVL			= LogLvl.info;

	public static final Logger BYPASS_FILTER_DEFAULT_LOG_KATEGORIE	= Logger.SWFW;
	public static final LogLvl BYPASS_FILTER_DEFAULT_LOG_LVL		= LogLvl.info;

//-----------------------------------------------------------------------------
	private static final String ATTRIBUTNAME_BAL = "BlocksAnyLevel";

	private final LogFilterKategorie	filterKategorie = new LogFilterKategorie();
	private final LogFilterLogLvl 		filterLogLvl    = new LogFilterLogLvl();

//=============================================================================

	public StdLogFilter() {
		this(new LogFilterKategorie(DEFAULT_LOG_KATEGORIE), new LogFilterLogLvl(DEFAULT_LOG_LVL));
	}
	public StdLogFilter(final LMTag tag) {
		this(new LogFilterKategorie(DEFAULT_LOG_KATEGORIE), new LogFilterLogLvl(DEFAULT_LOG_LVL), tag);
	}
	public StdLogFilter(final Logger kategorie) {
		this(new LogFilterKategorie(kategorie), new LogFilterLogLvl(DEFAULT_LOG_LVL));
	}
	public StdLogFilter(final LogLvl logLvl) {
		this(new LogFilterKategorie(DEFAULT_LOG_KATEGORIE), new LogFilterLogLvl(logLvl));
	}
	public StdLogFilter(final Logger kategorie, final LogLvl logLvl) {
		this(kategorie, logLvl, null);
	}
	/**@throws NullPointerException, falls kategorie oder lvl == null ist.*/
	public StdLogFilter(final LogFilterKategorie kategorie, final LogFilterLogLvl lvl) {
		this(kategorie, lvl, null);
	}
	/**@throws NullPointerException, falls kategorie oder lvl == null ist. LMTag can be null*/
	public StdLogFilter(final Logger kategorie, final LogLvl lvl, final LMTag tag) {
		this(new LogFilterKategorie(kategorie), new LogFilterLogLvl(lvl), tag);
	}
	/**@throws NullPointerException, falls kategorie oder lvl == null ist. LMTag can be null*/
	public StdLogFilter(final LogFilterKategorie kategorie, final LogFilterLogLvl lvl, final LMTag tag) {
		super(tag);
		setFilterKategorie	(kategorie);
		setFilterLogLvl		(lvl);
	}
	/**Copy XTor: deep clone des StdLogFilter filter @throws NullpointerException, falls filter == null*/
	public StdLogFilter(final StdLogFilter filter) {
		super(filter.getTag(TagState.ON));
		this.set(filter);
	}

//=============================================================================

	/**nie null, beachte: setBlockAnyLevel(), blocksAnyLevel()*/
	public LogLvl getLogLvl() 					{return getFilterLogLvl().getLogLvl();}
	/**beachte: setBlockAnyLevel(), blocksAnyLevel()*/
	public void   setLogLvl(final LogLvl logLvl)		{       getFilterLogLvl().setLogLvl(logLvl);}

	/**nie null*/
	public Logger getKategorie() 				{return getFilterKategorie().getKategorie();}
	public void   setKategorie(final Logger kategorie){		getFilterKategorie().setKategorie(kategorie);}

	/**d.h passes only on modus or tag, but not LogLvl or Katergorie if true*/
	public boolean blocksAnyLevel() {
		return getFilterLogLvl().getModus() == FilterModus.NEVER_PASS;
	}
	/**falls true => (passOnStandardModusIgnoreTag(final LogMsg logMsg) == false), nur noch Modus und Tag wirksam*/
	public void setBlockAnyLevel(final boolean blocksAnyLevel) {
		getFilterLogLvl().setModus(blocksAnyLevel ? FilterModus.NEVER_PASS
												  :	FilterModus.STANDARD);
	}

	/**Clones the Setting of the wegWirfFilter*/
	public void set(final StdLogFilter wegWirfFilter) {
		//modus in ALogFilter, tag in ALogTaggedFilter
		super.set(wegWirfFilter);
		//Kategorie
		getFilterKategorie().set(wegWirfFilter.getFilterKategorie());
		//LogLvl
		getFilterLogLvl().set(wegWirfFilter.getFilterLogLvl());
	}

	public LogLvl getDefaultLogLvl   () {return StdLogFilter.BYPASS_FILTER_DEFAULT_LOG_LVL;}
	public Logger getDefaultKategorie() {return StdLogFilter.BYPASS_FILTER_DEFAULT_LOG_KATEGORIE;}
//=============================================================================
	@Override public boolean passOnStandardModusIgnoreTag(final LogMsg logMsg) {
		return getFilterLogLvl   ().pass(logMsg)
		    && getFilterKategorie().pass(logMsg);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.mstapp.logging.filter.ALogFilter#getName()
	 */
	@Override protected String getName() {return NAME;}
	/* (non-Javadoc)
	 * @see de.desy.mst.mstapp.logging.filter.ALogFilter#decorateElement(org.w3c.dom.Document, org.w3c.dom.Element)
	 */
	@Override protected void decorateElement(final Document document, final Element element) {
		super.decorateElement(document, element);
		element.appendChild(getFilterKategorie().getBeschreibungAsElement(document));
		element.appendChild(getFilterLogLvl   ().getBeschreibungAsElement(document));
	}

	@Override public boolean equals(final Object obj) {
		if(obj instanceof StdLogFilter){
			final StdLogFilter filter = (StdLogFilter) obj;
			return super.equals(filter)
			    && this.getKategorie().ordinal() == filter.getKategorie().ordinal()
			    && this.getLogLvl()   .ordinal() == filter.getLogLvl()   .ordinal()
			    && this.blocksAnyLevel()         == filter.blocksAnyLevel()
			    ;
		}
		return false;
	}
	@Override public int hashCode() {
		return super.hashCode()
		     + this.getKategorie().ordinal()
		     + this.getLogLvl().ordinal()
		     ;
	}
	@Override public String toString() {
		if(this.getModus() != FilterModus.STANDARD)
			return "(" + this.getModus () + ")";
		final String tagString = getTag(TagState.ON) == null ? ""
						: getTag(TagState.ON).getName() + ", " + getTagState();
		final String lvlKatstring = blocksAnyLevel() ? ""
						: ", " + this.getLogLvl() + ", " + this.getKategorie();
		return  "(" + tagString + lvlKatstring + ")";
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.filter.ALogFilter#getSaveString(java.lang.String, java.lang.String)
	 */
	@Override public String getSaveString(final String appenderGruppe, final String saveString) {
		String retval = super.getSaveString(appenderGruppe, saveString);
		if(!getDefaultLogLvl().equals(getFilterLogLvl().getLogLvl()))
			retval = 	getFilterLogLvl().getSaveString(appenderGruppe, retval);
		if(!getDefaultKategorie().equals(getFilterKategorie().getKategorie()))
			retval = getFilterKategorie().getSaveString(appenderGruppe, retval);
		retval = getSaveStringBlocksAnyLevel(appenderGruppe, retval);
		return retval;
	}
	private String getSaveStringBlocksAnyLevel(final String appenderGruppe, final String saveString) {
		final String key = makeKey(appenderGruppe, ATTRIBUTNAME_BAL);
		final String value = blocksAnyLevel() ? "true" : "false";
		return PrefStorage.serialize(saveString, key, value);
	}
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.filter.ALogFilter#setFromSaveString(java.lang.String, java.lang.String)
	 */
	@Override public void setFromSaveString(final String appenderGruppe, final String saveString, final Object optionalDefault) {
		getFilterKategorie().setFromSaveString(appenderGruppe, saveString, getDefaultKategorie());
		getFilterLogLvl()	.setFromSaveString(appenderGruppe, saveString, getDefaultLogLvl());
		this.setFromSaveStringBlocksAnyLevel(appenderGruppe, saveString);
		super				.setFromSaveString(appenderGruppe, saveString, null);
	}
	private void setFromSaveStringBlocksAnyLevel(final String appenderGruppe, final String saveString) {
		final String keyTag = makeKey(appenderGruppe, ATTRIBUTNAME_BAL);
		final String valueTag = PrefStorage.extract(saveString, keyTag);
		final boolean blocks = "true".equals(valueTag);
		this.setBlockAnyLevel(blocks);
	}
	/**ohne optionalDefault da eh ignoriert*/
	public final void setFromSaveString(final String appenderGruppe, final String saveString) {
		setFromSaveString(appenderGruppe, saveString, null);
	}

//=============================================================================

	/**nie null*/private LogFilterLogLvl	getFilterLogLvl()	{return this.filterLogLvl;}
	/**nie null*/private LogFilterKategorie	getFilterKategorie(){return this.filterKategorie;}
	/**@throws NullPointerException, falls der LogFilterLogLvl null ist.*/
	private void setFilterLogLvl(final LogFilterLogLvl filterLogLvl) {
		if(filterLogLvl == null) {throw new NullPointerException("Der LogFilterLogLvl darf nicht null sein.");}
		getFilterLogLvl().set(filterLogLvl);
	}
	/**@throws NullPointerException, falls die LogFilterKategorie null ist.*/
	private void setFilterKategorie(final LogFilterKategorie filterKategorie) {
		if(filterKategorie == null) {throw new NullPointerException("Die LogFilterKategorie darf nicht null sein.");}
		getFilterKategorie().set(filterKategorie);
	}
//=============================================================================
}
