/**
 * Apr 23, 2008 - Apr 24, 2008	ws:	Creation from scratch
 * May  7, 2008 - Jun  5, 2008	ws:	ALogSource exklusiver Owner vom AMsgModel ..
 * Jun 10, 2008 - Jun 16, 2008	ws: support reloadData(...)
 * Jul  3, 2008 - Jul  8, 2008	ws:	support Chunks for too long sources
 * 				  Jul 28, 2008	ws:	reloadAllData reports possible structure change
 * Aug  1, 2008 - Aug  1, 2008	ws:	added: hasMassiveDecodingProblems()
 * Aug 11, 2008 - Aug 11, 2008	ws: added: SplitTextRow (extracted from CSVRow)
 * Aug 12, 2008 - Aug 25, 2008	ws:	added Schreib Unterstuetzung
 * Aug 28, 2008 - Aug 28, 2008	ws: color coding nicht nur Message sondern auch
 * 							feldspezifisch (Kommentare Anzeige unterdruecken)
 * Sep  1, 2008 - Sep  1, 2008	ws:	implements ITabelle zur Veroeffentlichung
 * Oct  6, 2008 - Oct  6, 2008	ws:	changed: toString so ok for PrintOutput
 * 									added: getBezeichnung
 *	Dec 10, 2008 - Dec 10, 2008	ws:	MstSecurityMgr -> MstSecurity
 *	Feb 15, 2009 - Feb 15, 2009 ws: serializable
 *	Jul 23, 2009 - Jul 23, 2009	ws:	changed List<L> getMessages() -> IRList<L> ... 
 *	Jul 28, 2009 - Jul 30, 2009	ws:	added: getAutoAppendServer() etc.
 *	Aug  3, 2009 - Aug  3, 2009	ws:	added: getAccessory()
 */
package de.desy.mst.libs.framework.mstapp.logging.gui;

import java.awt.Component;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Serializable;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;

import de.desy.mst.libs.framework.mstapp.Main;
import de.desy.mst.libs.framework.mstapp.ini.ITabelle;
import de.desy.mst.libs.framework.mstapp.logging.ILogProvider;
import de.desy.mst.libs.framework.mstapp.logging.LogMsg;
import de.desy.mst.libs.framework.mstapp.logging.LogUtil;
import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.logging.gui.LogMsgAuswertung;
import de.desy.mst.libs.framework.mstapp.logging.gui.__IAutoAppend.IAAServer;
import de.desy.mst.libs.framework.mstapp.logging.reader.LogReaderProvider;
import de.desy.mst.libs.framework.mstapp.security.MstSecurity;
import de.desy.mst.libs.framework.mstapp.util.gui.AppColor;

/**
 * Adapter zu diversen Logging Quellen:
 * <li> System
 * <li> Local Cache
 * <li> Tine Logging Server
 * <br> logging.gui Package Privat
 * @author ws
 * @version 1.0.0
 */
@SuppressWarnings({"rawtypes", "unchecked"})
abstract class ALogSource <L, M extends AMsgModel<L>> implements ITabelle {
	public static ALogSource<LogMsg, MsgModel_MstLog> createLogSource_ReaderProvider(final File file) throws IOException{
		final LogReaderProvider readerProvider = LogReaderProvider.createFrom(file);
		return new LogSourceReaderProvider(readerProvider, MsgModel_MstLog.getInstance()){
			@Override public File getDefaultFile() {return file;}
		};
	}

	public static <L, M extends AMsgModel<L>> ALogSource<L, M> createListLogSource(final List<L> list, final M logMsgModel){
		return new ListLogSource<L, M>(list, logMsgModel);
	}

//=============================================================================
	private final M logMsgModel;
	private final IAAServer<L> autoAppendServer;
	private LogMsgAuswertung<L, M> auswertung;
//=============================================================================
	ALogSource(final M logMsgModel, final IAAServer<L> autoAppendServer) {
		this.logMsgModel      = logMsgModel;
		this.autoAppendServer = autoAppendServer == null ? 
							new __AutoAppendServer_Null<L>() : autoAppendServer;
	}
//=============================================================================
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.ini.ITabelle#reloadAllData()
	 */
	@Override  
	abstract public void reloadAllData() throws IOException;
	abstract public __IRList<L> getMessages();
	abstract public boolean hasMassiveDecodingProblems();
	@Override
	abstract public String toString();	//fuer ordentliche Fehlertexte
	abstract public String getBezeichnung();//fuer Printout Footer

	@SuppressWarnings("deprecation")
	/**helper*/String getBezeichnung(final String quelle) {
		return quelle + " von " + Main.getApplicationInfo().getAppName()
					  + " auf " + MstSecurity.get().getHostName();
	}


	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime
				* result
				+ ((this.logMsgModel == null) ? 0 : this.logMsgModel.hashCode())
				+ getSize()
				+ ((getSize() > 0) ? getMessage(getSize() - 1).hashCode() : 0);
		return result;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
//	@SuppressWarnings("unchecked")
	@Override public boolean equals(final Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final ALogSource other = (ALogSource) obj;
		if (this.logMsgModel == null) {
			if (other.logMsgModel != null)
				return false;
		} else if (!this.logMsgModel.equals(other.logMsgModel))
			return false;
		return this.getMessages().equals(other.getMessages());
	}

	public L 	   	getMessage(final int index){return getMessages().get(index);}
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.ini.ITabelle#getSize()
	 */
	@Override public int		getSize() 	 		 {return getMessages().size();    }
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.ini.ITabelle#getAnzahlSpalten()
	 */
	@Override public int      getAnzahlSpalten()   {return getMsgModel().getSize();}
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.ini.ITabelle#get(int, int)
	 */
	@Override public Object   get(final int iZeile, final int iSpalte) {
		final L message = getMessage(iZeile);
		return getMsgModel().getField(iSpalte).get(message);
	}
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.ini.ITabelle#get(int, java.lang.String)
	 */
	@Override public Object   get(final int iZeile, final String spaltenTitel) {
		final L message = getMessage(iZeile);
		return getMsgModel().getField(spaltenTitel).get(message);
	}
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.ini.ITabelle#getSpaltenTitel()
	 */
	@Override public List<String> getSpaltenTitel(){return getMsgModel().getFieldNames();}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.ini.ITabelle#getTyp(int)
	 */
	@Override public Class<?> getTyp(final int iSpalte) {
		return getMsgModel().getField(iSpalte).getClazz();
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.ini.ITabelle#isKommentar(int)
	 */
	@Override public boolean istKommentar(final int iZeile){
		return getMsgModel().isComment(getMessage(iZeile));
	}
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.ini.ITabelle#getKommentar(int)
	 */
	@Override public String getKommentar(final int iZeile){
		return getMsgModel().getComment(getMessage(iZeile));
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.ini.ITabelle#isDirty()
	 */
	@Override public boolean isDirty(){
		if(canWrite())
			return getWriter().isDirty();
		else
			return false;
	}
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.ini.ITabelle#istEditierBar()
	 */
	@Override public boolean istEditierBar(){ return canWrite();}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.ini.ITabelle#set(int, int, java.lang.Object)
	 */
//	@SuppressWarnings("unchecked")
	@Override public void set(final int iZeile, final int iSpalte, final Object wert){
		final AField field = getMsgModel().getField(iSpalte);
		getWriter().setValueAt((Serializable)wert, iZeile, field);
	}
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.ini.ITabelle#set(int, java.lang.String, java.lang.Object)
	 */
//	@SuppressWarnings("unchecked")
	@Override public void set(final int iZeile, final String spaltenTitel, final Object wert){
		final AField field = getMsgModel().getField(spaltenTitel);
		getWriter().setValueAt((Serializable)wert, iZeile, field);
	}
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.ini.ITabelle#write(java.io.File)
	 */
	@Override public void write(final File file) throws IOException{
		getWriter().write(file);
	}



	/**
	 * zu lange Datensaetze werden nur Chunk-weise geladen. Chunks koennen leer
	 * sein und die Anzahl kann sich beim lesen weiter erhoehen.
	 * <br> Default: genau einen Chunk
	 */
	public long getChunkAnzahl() 		{return 1L;}
	public long getChunkIndexOfCurrent(){return 0L;}
	public void setChunk(final long index) throws IOException {reloadAllData();}

	public M getMsgModel() 	 	 {return this.logMsgModel;}
	public AppColor getAppColor(final int index, final int spalte) {
		final L message = getMessage(index);
		return getMsgModel().getAppColor(message, spalte);
	}

	public final LogMsgAuswertung<L, M> getAuswertung() {return this.auswertung;}
	public boolean hasAuswertung() {return getAuswertung() != null;}
	/**never null, immutable*/
	public final IAAServer<L> getAutoAppendServer() {return this.autoAppendServer;}
	/**
	 * normalerweise null, sonst GUI um zusaetzliches spezifisches zu koennen
	 * z.B. In Tine for MstLog Uebersicht einzelne Apps zu feuern
	 */
	public Component getAccessory() {return null;}
	//-------------------------------------------------------------------------
	/**zum Ueberschreiben falls schon an der Quelle filtern*/
	public void onFilterChanged(final AndOfFieldsFilter filter
					   		  , final LoggingTable<L, M> table) {;}
	/**zum Ueberschreiben falls schon an der Quell filtern*/
	public void initFilter(final HauptFilter hauptFilter
						 , final LoggingTable<L, M> table) {updateAuswertung();}
	public void onNewEntries(){updateAuswertung();}

	public boolean 				canWrite () {return getWriter() != null;}
	public ILogSourceWriter<L>	getWriter() {return null;}

	//-------------------------------------------------------------------------
	/**good Place for FileOutput, may be null*/
	public File getDefaultFile() {return null;}
	/**Die Messages inklusive Header als string, so dass es wieder gelesen werden kann*/
//	@SuppressWarnings("unchecked")
	public String serialize() {
		final AField[] fields = getMsgModel().getFields().values().toArray(new AField[0]);
		if(fields.length == 0)
			return "";

		final String separator = getSeparator() + " ";
		final int capacity = fields.length * 8 * (getSize() + 1);
		final StringBuilder sb = new StringBuilder(capacity);

		serializeHeader(separator, fields, sb);
		final __IRList<L> messages = getMessages();
		serializeMessages(separator, fields, messages, sb);
		return sb.toString();
	}

	public String getSeparator() 			{return "\t";}
	public String getNewMessageSeparator()	{return "\n";}

	public void updateAuswertung() {updateAuswertung(getAuswertung());}
//-------------------------------------------------------------------------
	void serializeHeader(final String separator, final AField<?, L>[] fields, final StringBuilder sb) {
		sb.append(fields[0].getName());
		for (int i = 1; i < fields.length; i++) {
			sb.append(separator)
			  .append(fields[i].getName());
		}
	}
	void serializeMessages(final String separator, final AField<?, L>[] fields, final __IRList<L> messages, final StringBuilder sb) {
		for (final L logMsg : messages) { 
			sb.append(getNewMessageSeparator());
			if(getMsgModel().isComment(logMsg))
				serializeComment(separator, fields, logMsg, sb);
			else
				serializeEntry(separator, fields, logMsg, sb);
		}
	}
	void serializeEntry(final String separator, final AField<?, L>[] fields, final L logMsg, final StringBuilder sb) {
		serializeFieldValue(fields[0], logMsg, sb);
		for (int i = 1; i < fields.length; i++){
			sb.append(separator);
			serializeFieldValue(fields[i], logMsg, sb);
		}
	}
	void serializeFieldValue(final AField<?, L> field, final L logMsg, final StringBuilder sb) {
		sb.append(field.serialize(logMsg));
	}
	void serializeComment(final String separator, final AField<?, L>[] fields, final L logMsg, final StringBuilder sb) {
		serializeEntry(separator, fields, logMsg, sb);	//default
	}
	private void updateAuswertung(final LogMsgAuswertung<L, M> oldAuswertung) {
		this.auswertung  = new LogMsgAuswertung<L, M>(this, oldAuswertung);
	}
//=============================================================================

	/**
	 * MAIN_COMMENT_HERE
	 * @author ws
	 * @version 1.0.0
	 */
	private static final class ListLogSource<L, M extends AMsgModel<L>>
					   extends ALogSource   <L, M> {
		private final List<L> list;
		//---------------------------------------------------------------------
		private ListLogSource(final List<L> list, final M logMsgModel) {
			super(logMsgModel, null);
			this.list = Collections.unmodifiableList(list);
		}
		//---------------------------------------------------------------------
		/* (non-Javadoc)
		 * @see de.desy.mst.libs.framework.mstapp.logging.gui.ALogSource#getMessage(int)
		 */
		@Override public L getMessage(final int index) {return getMessages().get(index);}
		/* (non-Javadoc)
		 * @see de.desy.mst.libs.framework.mstapp.logging.gui.ALogSource#getMessages()
		 */
		@Override public __IRList<L> getMessages() {return new __ListAdapt<L>(this.list);}
		/* (non-Javadoc)
		 * @see de.desy.mst.libs.framework.mstapp.logging.gui.ALogSource#getSize()
		 */
		@Override public int getSize() {return getMessages().size();} 
		/* (non-Javadoc)
		 * @see de.desy.mst.libs.framework.mstapp.logging.gui.ALogSource#reloadAllData()
		 */
		@Override public void reloadAllData() {;}
		/* (non-Javadoc)
		 * @see de.desy.mst.libs.framework.mstapp.logging.gui.ALogSource#toString()
		 */
		@Override public String toString() {
			return "ListLogSource " + System.identityHashCode(this);
		}
		@Override public String getBezeichnung() {
			return "ListLogSource auf " + MstSecurity.get().getHostName();
		}
		/* (non-Javadoc)
		 * @see de.desy.mst.libs.framework.mstapp.logging.gui.ALogSource#hasMassiveDecodingProblems()
		 */
		@Override public boolean hasMassiveDecodingProblems() {return false;}
	}
	private static class LogSourceReaderProvider extends ALogSourceILogProvider {
		private long chunkIndex;
		private long chunkAnzahl;	//aktuelle Schaetzung (und Minimum)
		private final long chunkLength;
		private Future<Long> futureCalcChunkAnzahl;
		//---------------------------------------------------------------------
		private LogSourceReaderProvider(final LogReaderProvider provider, final MsgModel_MstLog logMsgModel) {
			super(provider, null, logMsgModel);
			this.chunkIndex  = 0L;
			this.chunkLength = provider.getStop()- provider.getStart() + 1;
			this.chunkAnzahl = provider.hasMoreMessages() ? 2L : 1L;
			if(provider.hasMoreMessages())	//nutz die Zeit fuer HintergrundZaehlen
				this.futureCalcChunkAnzahl =
						Main.getExecutorService()
						    .submit(updateChunkAnzahlInBackground(provider));
		}
		//-------------------------------------------------------------------------
		@Override LogReaderProvider getLogProvider() {
			return (LogReaderProvider) super.getLogProvider();
		}
		//-------------------------------------------------------------------------
		@Override public long getChunkAnzahl() 			{
			updateChunkAnzahlFromFuture();
			return this.chunkAnzahl;
		}
		@Override public long getChunkIndexOfCurrent()	{return this.chunkIndex;}
		@Override public void setChunk(final long index) throws IOException {
			this.chunkIndex = index;
			reloadAllData();
		}
		@Override public void reloadAllData() throws IOException {
			final LogReaderProvider provider = getLogProvider();
			updateStartStop(provider);
			provider.reloadAllData();
			updateChunkAnzahl(provider);
		}
		@Override public boolean hasMassiveDecodingProblems() {
			return getLogProvider().hasMassiveDecodingProblems();
		}

		//-------------------------------------------------------------------------
		private long getChunkLength() {return this.chunkLength;}
		private void updateStartStop(final LogReaderProvider provider) {
			final long start = getChunkIndexOfCurrent() * getChunkLength();
			final long stop  = start + getChunkLength() - 1;
			provider.setStart(start);
			provider.setStop (stop);
		}
		private void updateChunkAnzahl(final LogReaderProvider provider) {
			if(provider.hasMoreMessages())
				this.chunkAnzahl = Math.max(getChunkAnzahl()
										  , getChunkIndexOfCurrent() + 2
										   );//2 = 1(0 start) + 1(>= einen mehr)
		}
		private Callable<Long> updateChunkAnzahlInBackground(final LogReaderProvider provider) {
			return new Callable<Long>(){
				@Override public Long call() throws IOException {
					return provider.countTotalAmountOfMessages();
				}
			};
		}
		private void updateChunkAnzahlFromFuture() {
			if(this.futureCalcChunkAnzahl == null)
				return;	//schon gemacht
			final long _start = System.currentTimeMillis();
			try {
				final long nMsgs = this.futureCalcChunkAnzahl.get(2, TimeUnit.SECONDS);
				final long chunkAnzahl = (nMsgs + this.chunkLength/2) / this.chunkLength;
				this.chunkAnzahl = chunkAnzahl;
				this.futureCalcChunkAnzahl = null;	//fertig
				final String msg = "LogSourceReaderProvider.updateChunkAnzahlInBackground(...) "
							     + nMsgs + " Meldungen in " + chunkAnzahl
							     + " Abschnitten zu je maximal " + this.chunkLength
							     + " Meldungen. " + getLogProvider().toString();
				Logger.SWFW.debug(LogUtil.timedMsg(_start, msg));
			} catch (final InterruptedException e) {
				updateChunkAnzahlInBackground(getLogProvider());//just retry
			} catch (final ExecutionException e) {
				Logger.SWFW.warn("LogSourceReaderProvider.updateChunkAnzahlInBackground(...) Problem: " + e);
			} catch (final TimeoutException e) {
				Logger.SWFW.warn("LogSourceReaderProvider.updateChunkAnzahlInBackground(...) braucht zu lange, abgebrochen, spter besser.");
			}
		}
	}
}
//=============================================================================
abstract class ALogSourceILogProvider extends ALogSource<LogMsg, MsgModel_MstLog>{
	private final ILogProvider logProvider;
	//-------------------------------------------------------------------------
	ALogSourceILogProvider(final ILogProvider logProvider, final IAAServer<LogMsg> autoAppendServer, final MsgModel_MstLog logMsgModel) {
		super(logMsgModel, autoAppendServer);
		this.logProvider = logProvider;
	}
	//-------------------------------------------------------------------------
	/**pp*/ILogProvider getLogProvider() {return this.logProvider;}
	//-------------------------------------------------------------------------
	@Override public void reloadAllData() throws IOException {getLogProvider().reloadAllData();}
	@Override public LogMsg 	  getMessage(final int index) {return getLogProvider().getMessage(index);}
	@Override public __IRList<LogMsg> getMessages()		{return new __ListAdapt<LogMsg>(getLPMessages());}
	          private    List<LogMsg> getLPMessages()	{return getLogProvider().getMessages();}
	@Override public int 	      getSize() 			{return getLogProvider().getSize();}
	@Override public String toString() {
		return "ALogSourceILogProvider (" + getLogProvider().toString() + ") " +
										  + System.identityHashCode(this);
	}
	@Override public String getBezeichnung() {
		return getBezeichnung(getLogProvider().toString());
	}
	
}// END class: ALogSourceILogProvider
//=============================================================================
/**
 * Basis Klasse fuer Text Zeilen einer ALogSource wie zb CSVRow
 * @author ws
 * @version 1.0.0
 */
abstract class SplitTextRow{
	private final String[] splitLine;
//-----------------------------------------------------------------------------
	SplitTextRow(final String[] splitLine) {this.splitLine = splitLine;}
//-----------------------------------------------------------------------------
	abstract public String getName();
	//-----------------------------------------------------------------------------
	public String get(final int iCol) {return getSplitLine()[iCol];}
	public <E extends Serializable> String get(final CSVField<E> field) {return get(field.getIndex());}
	public int getColumnCount() {return getSplitLine().length;}
	@Override public boolean equals(final Object other) {
		if(other instanceof SplitTextRow)
			return Arrays.equals(this.splitLine,((SplitTextRow)other).splitLine);
		return false;
	}
	@Override public int hashCode() {return Arrays.hashCode(this.splitLine);}
	@Override public String toString() {
		return getName() + ": " + Arrays.asList(this.splitLine).toString();
	}
	public boolean hasDecodingProblem() {
		final String[] splitLine = getSplitLine();
		if(splitLine == null)
			return true;
		return splitLine.length <= 1
			|| countEmptyStrings(splitLine) > splitLine.length / 2;
	}
	private int countEmptyStrings(final String[] splitLine) {
		int count = 0;
		for (final String string : splitLine)
			if(string.isEmpty())
				count++;
		return count;
	}
	public void set(final int index, final String stringValue) {
		getSplitLine()[index] = stringValue;
	}
	//-------------------------------------------------------------------------
	private final String[] getSplitLine() {return this.splitLine;}
}// END class: SplitTextRow


//=============================================================================
/**Schreibfunktionalitaet to File jenseits von canWrite, getWriter*/


interface ILogSourceWriter<L> {
	public void write(final File file) throws IOException;
	public <E extends Serializable> void setValueAt(final E value, final int index, final AField<E, L> field);

	public boolean isDirty();
	public void setDirty(final boolean isDirty);
}// END interface: ILogSourceWriter

abstract class ALogSourceWriter<L> implements ILogSourceWriter<L>{
   	private static final String DEFAULT_CHARSETNAME = "UTF-8";
	private final ALogSource<L , ? extends AMsgModel<L>> logSource;
	private boolean isDirty;
	//-------------------------------------------------------------------------
	ALogSourceWriter(final ALogSource<L, ? extends AMsgModel<L>> logSource) {
		super();
		this.logSource = logSource;
		this.isDirty   = false;
	}
	//-------------------------------------------------------------------------
	abstract public <E extends Serializable> void updateMessage(final Object value, final int index, final AField<E, L> field);
	//-------------------------------------------------------------------------
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.gui.ILogSourceWriter#setValueAt(java.lang.Object, int, de.desy.mst.libs.framework.mstapp.logging.gui.AField)
	 */
	@Override 
	public final <E extends Serializable> void setValueAt(final E newValue, final int index, final AField<E, L> field){
		try {
			if(!isDirty() && !getOldValue(index, field).equals(newValue))
				setDirty(true);
			updateMessage(newValue, index, field);
		} catch (final Exception e) {/*update failed, just ignore it*/}
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.gui.ILogSourceWriter#setDirty(boolean)
	 */
	@Override 
	public final void setDirty(final boolean isDirty) {this.isDirty  = isDirty;}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.gui.ILogSourceWriter#isDirty()
	 */
	@Override 
	public final boolean isDirty() {return this.isDirty;}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.gui.ILogSourceWriter#write(java.io.File)
	 */
	@Override 
	public void write(final File file) throws IOException{
		final OutputStream fos = new FileOutputStream(file);
			final OutputStreamWriter osw = new OutputStreamWriter(fos, DEFAULT_CHARSETNAME);
				final BufferedWriter buf = new BufferedWriter(osw);
					buf.write(getLogSource().serialize());
					setDirty(false);
				buf.close();
			osw.close();
		fos.close();
	}
	//-------------------------------------------------------------------------
	public ALogSource<L, ? extends AMsgModel<L>> getLogSource() {return this.logSource;}
	//-------------------------------------------------------------------------
	private <E extends Serializable> E getOldValue(final int index, final AField<E, L> field) {
		return field.get(getLogSource().getMessage(index));
	}
}// END class: ALogSourceWriter
