/**
 * 	Jul 16, 2008 - Aug  5, 2008	ws:	Creation from scratch (2'ter Anlauf)
 * 	Aug 11, 2008 - Aug 11, 2008	ws:	abstrahiere ALogSource_File heraus
 * 	Aug 12, 2008 - Aug 14, 2008	ws:	added Schreib Unterstuetzung
 * 	Aug 18, 2008 - Aug 18, 2008	ws: set in CSVField cleans the String
 * 	Aug 27, 2008 - Aug 28, 2008	ws: calcRows supports comments
 * 	Oct  6, 2008 - Oct  6, 2008	ws:	added getBezeichnung
 *	Jan 15, 2009 - Jan 15, 2009	ws:	removed trivial hashCode() in CSVRowModel
 *									serializable
 *	Jul 23, 2009 - Jul 23, 2009	ws:	changed List<L> getMessages() -> IRList<L> ... 
 */
package de.desy.mst.libs.framework.mstapp.logging.gui;

import java.awt.Color;
import java.io.File;
import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import de.desy.mst.libs.framework.mstapp.logging.gui.AFieldFilterGUIFactory.AsJList;
import de.desy.mst.libs.framework.mstapp.logging.gui.AFieldFilterGUIFactory.AsJTextArea;
import de.desy.mst.libs.framework.mstapp.util.MstParser;
import de.desy.mst.libs.framework.mstapp.util.gui.AppColor;

/**
 * <li> E.... Typ der Spalte
 * <li> L ... CSVRow
 * <li> M ... CSVMsgModel
 * @author ws
 * @version 1.0.0
 */
class CSVSource extends ALogSource_File<CSVRow, CSVRowModel>{
	public static CSVSource create(final File file) throws IOException {
		return create(ALogSource_File.calcInputLines(file), file);
	}
	public static CSVSource create(final String[] inputLines) {
		return create(inputLines, (File) null);
	}
	private static CSVSource create(final String[] inputLines, final File file) {
		return new CSVSource(file, new CSVAnalyser(inputLines));
	}
	private String separator;
	private List<CSVRow> leadingCommentLines;
	private CSVRow header;
	private int    columnCount;
	private CSVRow emptyMessage;
	private List<CSVRow> messages;
	private CSVRowModel rowModel;
	private final ILogSourceWriter<CSVRow> writer;

//-----------------------------------------------------------------------------
	private CSVSource(final File file, final CSVAnalyser analyser) {
		super(analyser.getCVSRowModel(), file);
		this.writer = new CSVLogSourceWriter(CSVSource.this);
		applyAnalyser(analyser);
	}
//-----------------------------------------------------------------------------
	@Override public void reloadAllData() throws IOException {
		final File file = getFile();
		if(file != null) {
			reloadAllData(file);
		}
	}
	@Override public String toString() {
		final File file = getFile();
		return "CSV Log Source (Separator = '" + getSeparator()
			 + ((file == null) ? "" : "', Datei = '" + file.getAbsolutePath())
			 + "') " + System.identityHashCode(this);
	}
	@Override public String getBezeichnung() {
		final File file = getFile();
		final String fileName = (file == null) ? ""
									: "', Datei = '" + file.getAbsolutePath();
		return getBezeichnung(fileName);
	}

	@Override public CSVRowModel  getMsgModel() {return this.rowModel;}
	@Override public boolean hasMassiveDecodingProblems() {return false;}
	@Override public ILogSourceWriter<CSVRow> getWriter() {return this.writer;}
	@Override public String 	  getSeparator() {return this.separator;}
	@Override public __IRList<CSVRow> getMessages () {return new __ListAdapt<CSVRow>(this.messages);}

	@Override void serializeComment(final String separator, final AField<?, CSVRow>[] fields, final CSVRow row, final StringBuilder sb) {
		sb.append(row.serializeComment()); 
	}

	@Override void serializeHeader(final String separator, final AField<?, CSVRow>[] fields, final StringBuilder sb) {
		final int begin = sb.length();
		final List<CSVRow> leadingCommentLines = getLeadingCommentLines();
		serializeMessages(separator, fields, new __ListAdapt<CSVRow>(leadingCommentLines), sb);
		if(sb.length() > begin){
			pruneLeadingNewMsgSeparator(sb, begin);
			sb.append(getNewMessageSeparator());
		}
		super.serializeHeader(separator, fields, sb);
	}
	//-----------------------------------------------------------------------------
	public List<CSVRow> getLeadingCommentLines(){return this.leadingCommentLines;}
	public CSVRow getHeader()					{return this.header;}
	public int    getColumnCount()				{return this.columnCount;}
	public CSVRow getEmptyMessage()				{return this.emptyMessage;}

//-----------------------------------------------------------------------------

	private void pruneLeadingNewMsgSeparator(final StringBuilder sb, final int begin) {
		sb.replace(begin, begin + getNewMessageSeparator().length(), "");
	}
	private void applyAnalyser(final CSVAnalyser analyser) {
		this.separator		= analyser.getSeparator();
		this.leadingCommentLines = analyser.getLeadingCommentLines();
		this.header			= analyser.getHeader();
		this.columnCount	= analyser.getColumnCount();
		this.emptyMessage   = analyser.getEmptyRow();
		this.messages 		= analyser.getRows();
		this.rowModel       = analyser.getCVSRowModel();
		getWriter().setDirty(false);
	}
	/**@throws NullPointerException for null file*/
	private void reloadAllData(final File file) throws IOException {
		applyAnalyser(new CSVAnalyser(calcInputLines(file)));
	}

//=============================================================================
	/**
	 * Idee :
	 * <li>erste Zeile enthaelt (ungetrimmte) Spaltentitel mit Separator getrennt
	 * <li>weitere Zeilen Daten als text mit gleichem Separator und Spaltenzahl
	 * <li>Typ String, Double, Long, Integer je nach vorhandenen Daten
	 */
	@SuppressWarnings({"rawtypes", "unchecked"})
	static class CSVAnalyser{
		private final static String   SEP_DEFAULT    = ",";
		private final static String[] SEP_KANDIDATEN = {"\t", ";", ","};
		private final static String[] EMPTY_INPUT = {""};//one line one col
	//=========================================================================
		private final String separator;
		private final List<CSVRow> leadingCommentLines;
		private final CSVRow header;
		private final int 	 columnCount;
		private final CSVRow emptyRow;
		private final List<CSVRow> rows;
		private final CSVRowModel  rowModel;
//		/**Anzahl der Spalten in erster Zeile (Header)*/
//		private final int    nCol;
//		private final String[] 		 headerArray;
//		private final String[] 		 emptyDataArray;
//		private final List<String[]> dataArrayList;
//		private final List<AColumn<?>> columns;
//		private final MsgModel_CSV   msgModel;

	//=========================================================================
		public CSVAnalyser(String ... inputLines) {
			super();
			if(inputLines == null || inputLines.length == 0)
				inputLines = EMPTY_INPUT;

			if(inputLines[0] == null)
				inputLines[0] = "";

			int headerLineIndex = 0;
			for (int i = 0; i < inputLines.length; i++) {
				final String line = inputLines[i];
				if(!CSVRow.isComment(line)){
					headerLineIndex = i;
					break;
				}
			}
			final String headerLine = inputLines[headerLineIndex];
			this.separator		= calcSeparator  (headerLine);
			this.columnCount    = MstParser.countSubstringsIn(getSeparator(), headerLine) + 1;
			this.emptyRow		= new CSVRow(getSeparator(), "",         getColumnCount());
			final boolean noHeaderGiven =  getColumnCount() == 1
										&& inputLines.length > 1
										&& headerLine.length() > 30
										&& headerLine.length() >= inputLines[1].length() * .8
										;
			this.header = noHeaderGiven ?
					new CSVRow(getSeparator(), "Kein Spaltentitel gefunden", getColumnCount()) :
					new CSVRow(getSeparator(), headerLine, getColumnCount());
			this.leadingCommentLines = calcLeadingCommentLines(getSeparator(),inputLines, getColumnCount());
			this.rows			= calcRows(getSeparator(), inputLines, getEmptyRow(), getColumnCount(), noHeaderGiven, headerLineIndex);
			this.rowModel       = calcRowModel(getColumnCount(), getHeader(), getRows());
		}
		//---------------------------------------------------------------------
		public String		getSeparator() 		{return this.separator;}
		public List<CSVRow> getLeadingCommentLines(){return this.leadingCommentLines;}
		public CSVRow		getHeader() 		{return this.header;}
		public int 			getColumnCount() 	{return this.columnCount;}
		public CSVRow 		getEmptyRow()		{return this.emptyRow;}
		public List<CSVRow>	getRows()			{return this.rows;}

		public CSVRowModel	getCVSRowModel()	{return this. rowModel;}
		//---------------------------------------------------------------------
		private String calcSeparator(final String headerLine) {
			for (int i = 0; i < SEP_KANDIDATEN.length; i++)
				if(headerLine.contains(SEP_KANDIDATEN[i]))
					return SEP_KANDIDATEN[i];
			return SEP_DEFAULT;
		}
		private List<CSVRow> calcLeadingCommentLines(final String separator
												   , final String[] inputLines
												   , final int nCol) {
			final ArrayList<CSVRow> lines = new ArrayList<CSVRow>();
			for (final String line : inputLines) {
				if(CSVRow.isComment(line))
					lines.add(new CSVRow(separator, line, nCol));
				else
					break;
			}
			return lines;
		}
		private List<CSVRow> calcRows(final String separator
									, final String[] inputLines
									, final CSVRow defaultRow
									, final int columnCount
									, final boolean noHeaderGiven
									, final int headerLineIndex) {
			final ArrayList<CSVRow> messages = new ArrayList<CSVRow>(inputLines.length - 1);
			for (int i = (noHeaderGiven ? 0 : 1) + headerLineIndex; i < inputLines.length; i++) {
				final String line = inputLines[i];
				final CSVRow row = (line == null || line.isEmpty())? defaultRow
								: new CSVRow(getSeparator(), line, columnCount);
				messages.add(row);
			}
			return messages;
		}
		private CSVRowModel calcRowModel(final int columnCount, final SplitTextRow header, final List<CSVRow> rows) {
			final Map<String, CSVField<?>> map = new LinkedHashMap<String, CSVField<?>>(columnCount);
			CSVField<String> coreInfoField = null;
			final SplitTextRow firstRow = (rows.size() > 0) ? rows.get(0) : getEmptyRow();
			for (int iCol = 0; iCol < columnCount; iCol++) {
				final CSVField field = CSVFieldType.calcCSVField(getSeparator(), iCol, header, rows);
				putField(map, field);
				coreInfoField = updCoreInfoField(coreInfoField, field, firstRow);
			}
			final Map<String, CSVField<?>> uMap = Collections.unmodifiableMap(map);
			return new CSVRowModel(uMap, coreInfoField);
		}
		void putField(final Map<String, CSVField<?>> map, final CSVField<?> field) {
			final String name = field.getName();
			if(map.containsKey(name))
				putField(map, name + "_" + field.getIndex(), field);
			else
				putField(map, name , field);
		}
		private void putField(final Map<String, CSVField<?>> map, final String name, final CSVField<?> field) {
			if(map.containsKey(name))
				putField(map, name + "_" + field.getIndex(), field);
			else
				map.put(name, field);
		}
		private CSVField updCoreInfoField(final CSVField coreInfoField, final CSVField kandidat, final SplitTextRow row) {
			if(	   coreInfoField == null
				|| row.get(coreInfoField).length() < row.get(kandidat).length()
			  )
				return kandidat;
			else
				return coreInfoField;
		}
	}//END CSVLogSource.CSVAnalyser
}//END CSVLogSource   =========================================================



//=============================================================================
//=============================================================================
//=============================================================================



class CSVRow extends SplitTextRow{
	private static final String   COMMENT_PREFIX = "# ";
	private static final int COMMENT_SPALTE = 0;

	static boolean isComment(final String line) {
		return line != null && line.startsWith(COMMENT_PREFIX);
	}
	static boolean isCmtSpalte(final int spalte) {
		return spalte == COMMENT_SPALTE;
	}

	//-------------------------------------------------------------------------
	private final boolean isComment;
	//-------------------------------------------------------------------------
	//Standart Zeile (kein Kommentar sondern Daten oder Header)
	public CSVRow(final String separator, final String line, final int nCol) {
		super(MstParser.normalize(
				isComment(line) ? splitForComment(line, nCol)
								: MstParser.split(separator, line, nCol)
								  ));
		this.isComment = isComment(get(COMMENT_SPALTE));
	}
	@Override public String getName() {return "CSVRow";}
	private static String[] splitForComment(final String line, final int columnCount) {
		final String[] split = new String[columnCount];
		split[COMMENT_SPALTE] = line;
		return split;
	}
	public boolean isComment() {return this.isComment;}
	public String getComment() {
		return isComment() ?
					get(COMMENT_SPALTE).substring(COMMENT_PREFIX.length() - 1)
				  : "";
	}
	public String serializeComment() {return get(COMMENT_SPALTE);}
}//END CSVRow   ===============================================================



//=============================================================================
//=============================================================================
//=============================================================================



@SuppressWarnings({"rawtypes"})
class CSVRowModel extends AMsgModel<CSVRow>{
	private static final Color	  VERYLIGHT_YELLOW  = new Color(255, 255, 217);
	private static final AppColor APP_COLOR_COMMENT =
						new AppColor("Comment", VERYLIGHT_YELLOW, Color.BLACK);
	private static final AppColor APP_COLOR_COMMENT_INVISIBLE =
		new AppColor("Comment Invisible", VERYLIGHT_YELLOW, VERYLIGHT_YELLOW);
	//-------------------------------------------------------------------------

	private final CSVField<?> coreInfoField;
	//-------------------------------------------------------------------------
	CSVRowModel(final Map<String, CSVField<?>> map, final CSVField<?> coreInfoField) {
		super(map);
		this.coreInfoField = coreInfoField;
	}
	//-------------------------------------------------------------------------
	@Override public CSVField<?> getCoreInfoField() {return this.coreInfoField;}
	@Override public CSVField<Date> getDateField() 	{return null;}
	@Override public boolean equalsInStructure(final AMsgModel otherMsgModel) {
		if(this == otherMsgModel)
			return true;
		if( !(otherMsgModel instanceof CSVRowModel))
			return false;
		if(this.getSize() != otherMsgModel.getSize())
			return false;
		final CSVRowModel other = (CSVRowModel) otherMsgModel;
		for (int index = 0; index < this.getSize(); index++) {
			final AField<?, CSVRow> thisField = this.getField(index);
			final AField<?, CSVRow> otherField = other.getField(index);
			if( !thisField.equals(otherField))
				return false;
		}
		return true;
	}
	@Override public String toString() {
		return "CSV Row Model " + System.identityHashCode(this)
								+ ", "
								+ getFields().keySet().toString();
	}
	@Override public AppColor getAppColor(final CSVRow message, final int spalte) {
		final AppColor aColor = message.isComment() ?
			(CSVRow.isCmtSpalte(spalte) ? APP_COLOR_COMMENT : APP_COLOR_COMMENT_INVISIBLE)
		  : AppColor._Info;
		return aColor;
	}
	@Override public boolean isComment(final CSVRow row) {
		return row != null && row.isComment();
	}
	@Override public String getComment(final CSVRow row) {
		return row.getComment();
	}

}//END CSVMsgModel   ==========================================================



//=============================================================================
//=============================================================================
//=============================================================================



@SuppressWarnings({"rawtypes", "unchecked"})
class CSVField<E extends Serializable> extends AField<E, CSVRow>{
	private static final long serialVersionUID = 1L;

	private final CSVFieldType<E> fieldType;
	private final String separator;
	private final boolean isDiscrete;
	//-------------------------------------------------------------------------
	public CSVField(final CSVFieldType<E> fieldType, final String separator, final int index, final String name, final E valueForPrefSize, final boolean isWideEditor, final boolean isDiscrete) {
		super(index, name, name, fieldType.getClazz(), valueForPrefSize, createFieldFilterFactory(name, isWideEditor, isDiscrete));
		this.separator = separator;
		this.fieldType = fieldType;
		this.isDiscrete = isDiscrete;
	}
	private static AFieldFilterGUIFactory createFieldFilterFactory(final String name, final boolean isWideEditor, final boolean isDiscrete) {
		return isDiscrete  ? new AsJList(name, name)
						   : new AsJTextArea(name, name, isWideEditor);
	}
	//-------------------------------------------------------------------------
	@Override E get(final CSVRow row) {
		final int index = getIndex();
		final String stringValue = row.get(index);
		if(row.isComment())
			if(CSVRow.isCmtSpalte(index))
				return getFieldType().convert(stringValue);
			else
				return getFieldType().getEmptyValue();
		else
			return getFieldType().convert(stringValue);
	}
	/**Kommentarzeilen koennen nicht editiert werden*/
	@Override public void set(final CSVRow row, final E value) {
		if(row.isComment())
			return;
		row.set(getIndex(), clean(asString(value)));
	}
	@Override public boolean isDiscrete() {return this.isDiscrete;}
	@Override public int hashCode() {
		return 31 * super.hashCode()
				  + ((this.fieldType == null) ? 0 : this.fieldType.hashCode());
	}
	@Override public boolean equals(final Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final CSVField other = (CSVField) obj;
		if (this.fieldType == null) {
			if (other.fieldType != null)
				return false;
		} else if (!this.fieldType.equals(other.fieldType))
			return false;
		return super.equals(obj);
	}
	@Override public String toString() {
		return getName() + "(" + getIndex() + ", " + getFieldType() + ")";
	}
	public CSVFieldType<E> getFieldType() {return this.fieldType;}
	//-------------------------------------------------------------------------
	/**separator is only allowed within "'s */
	private String clean(final String string) {
		if(string == null)
			return "";
		if(!string.contains(getSeparator()))
			return string.trim();
		if(string.startsWith("\"") && string.endsWith("\""))
			return string;
		if(!getFieldType().equals(CSVFieldType.OfString))
				string.replace(getSeparator(), "");
		return "\"" + string + "\"";
	}
	private String getSeparator() {return this.separator;}
}//END CSVField   =============================================================



//=============================================================================
//=============================================================================
//=============================================================================



/**
 * Implements the Type (Long, Double, String) specifics of a Field
 * @author ws
 * @version 1.0.0
 */
@SuppressWarnings({"rawtypes", "unchecked"})
abstract class CSVFieldType<E extends Serializable> {
	public final static CSVFieldType<Long>   OfLong   = new CSVFieldType<Long>  (  Long.class){
		@Override public Long convert(final String stringValue)	{return new Long(stringValue);}
		@Override public Long getValueForPrefSize() 		{return 123456L;}
		@Override public Long getEmptyValue() 				{return -999999999999999999L;}
		@Override public String toString() 					{return "Long";}
	};
	public final static CSVFieldType<Double> OfDouble = new CSVFieldType<Double>(Double.class){
		@Override public Double convert(final String stringValue)	{return new Double(stringValue);}
		@Override public Double getValueForPrefSize() 		{return 123.45;}
		@Override public Double getEmptyValue() 			{return Double.NaN;}
		@Override public String toString() 					{return "Double";}
	};
	public final static CSVFieldType<String> OfString = new CSVFieldType<String>(String.class){
		@Override public String convert(final String stringValue)	{return stringValue;}
		@Override public String getValueForPrefSize() 		{return "     ";}
		@Override public String getEmptyValue() 			{return "";}
		@Override public String toString() 					{return "String";}
	};
	//-------------------------------------------------------------------------
	public static CSVField calcCSVField(final String separator, final int iCol, final SplitTextRow header, final List<CSVRow> rows) {
		final String name = header.get(iCol);
		final boolean isWideEditor = rows.size() > 0
								  && rows.get(0).get(iCol).length() > 15;
		final CSVFieldType type = calcType(iCol, rows);
		final boolean isDiscrete = type.calcIsDiscrete(iCol, rows);
		final CSVField field = type.createCSVField(separator, iCol, name, isWideEditor, isDiscrete);
		return field;
	}

	private static CSVFieldType calcType(final int col, final List<CSVRow> rows) {
		final CSVFieldType<?>[] kandidaten = {OfLong, OfDouble};
		int lowestValidType = 0;
		for (final CSVRow row : rows) {
			if(row.isComment())
				continue;
			final String stringValue = row.get(col);
			for (int iType = lowestValidType; iType < kandidaten.length; iType++) {
				final CSVFieldType<?> kandidat = kandidaten[iType];
				try {
					kandidat.convert(stringValue);
				} catch (final Exception ex) {
					lowestValidType = iType + 1;
					if(lowestValidType >= kandidaten.length)
						return OfString;//kein spezielles format, geht immer
				}
			}
		}
		return kandidaten[lowestValidType];
	}
	//-------------------------------------------------------------------------
	private final Class<E> clazz;
	//-------------------------------------------------------------------------
	CSVFieldType(final Class<E> clazz) {this.clazz = clazz;}
	//-------------------------------------------------------------------------
	/**@return proper Conversion from String to Type E*/
	abstract public E convert(final String stringValue);
	abstract public E getValueForPrefSize() ;
	abstract public E getEmptyValue();

	@Override public abstract String toString();
	//-------------------------------------------------------------------------
	public boolean calcIsDiscrete(final int col, final List<CSVRow> rows) {
		if(this != CSVFieldType.OfString)
			return false;
		final HashMap<String, Integer> map = new HashMap<String, Integer>();
		int count = 0;
		for (final CSVRow row : rows) {
			if(row.isComment())
				continue;
			count++;
			final String key = row.get(col);
			if(map.containsKey(key))
				map.put(key, map.get(key) + 1);
			map.put(key, 1);
		}
		return map.size() < (count * 0.7);
	}
	public Class<E> getClazz() {return this.clazz;}
	private CSVField<E> createCSVField(final String separator, final int iCol, final String name, final boolean isWideEditor, final boolean isDiscrete) {
		return new CSVField<E>(this, separator, iCol, name, getValueForPrefSize(), isWideEditor, isDiscrete);
	}
}//END CSVFieldType   =========================================================


@SuppressWarnings({"rawtypes", "unchecked"})
class CSVLogSourceWriter extends ALogSourceWriter<CSVRow> {
	CSVLogSourceWriter(final CSVSource source) {super(source);}
	//-------------------------------------------------------------------------
	@Override 
	public <E extends Serializable> void updateMessage(final Object value, final int index, final AField<E, CSVRow> field) {
		if(field instanceof CSVField)
			try {((CSVField)field).set(getLogSource().getMessage(index), (Serializable)value);}
			catch (final Exception e) {/**ignore*/;}
	}
}//END CSVLogSourceWriter   ===================================================

