/**
 * 	Mar  4, 2008 - Mar  4, 2008	ws:	Creation from LogMsgAuswertung
 * 	May  7, 2008 - May  7, 2008	ws:	model non global
 * 	May 19, 2008 - May 23, 2008	ws:	ALogSource now generisch
 *	Jan 15, 2009 - Jan 15, 2009	ws:	serializable
 *	Jul  3, 2009 - Jul  3, 2009	ws:	class public -> pp
 *	Jul 23, 2009 - Jul 23, 2009	ws:	changed List<L> getMessages() -> IRList<L> ... 
 */
package de.desy.mst.libs.framework.mstapp.logging.gui;

import java.io.Serializable;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

/**
 * Berechnet kleinste und groesste Werte so wie in M.Field definiert.
 * (Typisch String-Laenge)
 * @author ws
 * @version 1.0.0
 */
class LogMsgAuswertungMinMax<L, M extends AMsgModel<L>> {
	private final Map<AField<?, L>, MinMax<?, L>> minMaxMap;
//=============================================================================
	public LogMsgAuswertungMinMax(final ALogSource<L, M> logSource) {
		super();
		final __IRList<L> msgs = logSource.getMessages();
		final M model      = logSource.getMsgModel();
		final Map<? extends AField<?, L>, ? extends MinMax<?, L>> map =
												createMinMaxMap(msgs, model);
		this.minMaxMap  = Collections.unmodifiableMap(map);
	}
//=============================================================================
	public <E extends Serializable> E getMin(final AField<E, L> field){return getMinMax(field).getMin();}
	public <E extends Serializable> E getMax(final AField<E, L> field){return getMinMax(field).getMax();}

//=============================================================================
	private Map<AField<?, L>, MinMax<?, L>> getMinMaxMap() {return this.minMaxMap;}
	@SuppressWarnings("unchecked")
	private <E extends Serializable> MinMax<E, L> getMinMax(final AField<E, L> field) {
		return (MinMax<E, L>)getMinMaxMap().get(field);
	}

	private Map<? extends AField<?, L>, ? extends MinMax<?, L>> createMinMaxMap(final __IRList<L> msgs, final M model) {
		final Map<AField<?, L>, MinMax<?, L>> minMaxMap = new HashMap<AField<?, L>, MinMax<?, L>>(model.getSize());
		initMinMaxMap(model, minMaxMap);
		fillMinMaxMap(msgs,  minMaxMap);
//		sealEntriesInMinMaxMap(minMaxMap);
		return minMaxMap;
	}
	private void initMinMaxMap(final M model
							 , final Map<AField<?, L>, MinMax<?, L>> minMaxMap) {
		final int n = model.getSize();
		for (int i = 0; i < n; i++) {
			final AField<?, L> field = model.getField(i);
			final MinMax<?, L> minMax = MinMax.create(field);
			minMaxMap.put(field, minMax);
		}
	}
	private void fillMinMaxMap(final __IRList<L> messages
							 , final Map<AField<?, L>, MinMax<?, L>> minMaxMap) {
		final Set<AField<?, L>> fields = minMaxMap.keySet();
		for (final L logMsg : messages) {
			for (final AField<?, L> field : fields) {
				final MinMax<?, L> minMax = minMaxMap.get(field);
				minMax.updateWithNewCandidate(logMsg);
			}
		}
	}

	private static class MinMax<E extends Serializable, L>{
		private final AField<E, L> field;
		private E min;
		private E max;
	//-------------------------------------------------------------------------
		private MinMax(final AField<E, L> field) {
			super();
			this.field = field;
			this.min = null;
			this.max = null;
		}
		public static <T extends Serializable, L> MinMax<T, L> create(final AField<T, L> field){
			return new MinMax<T, L>(field);
		}
	//-------------------------------------------------------------------------
		public E getMin() {return this.min;}
		public E getMax() {return this.max;}
		public void updateWithNewCandidate(final L logMsg){
			final E candidate = this.field.get(logMsg);
			this.min = this.field.min(getMin(), candidate);
			this.max = this.field.max(getMax(), candidate);
		}
	} 
}
