/**
 *  Created on  Aug 11, 2006 - Aug 21, 2006
 *  Jan 29, 2008 - Jan 29, 2008 ws: mstApp == null problem
 *	Feb 15, 2008 - Feb 15, 2008 ws: added "@Override"es
 *	Aug 20, 2010 - Aug 20, 2010	ws:	added: ensureIndexIsVisible() with index and jList protection
 */
package de.desy.mst.libs.framework.mstapp.logging.gui;

import java.awt.event.MouseListener;
import java.util.LinkedList;
import java.util.List;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JList;
import javax.swing.JMenuItem;
import javax.swing.SwingUtilities;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import de.desy.mst.libs.framework.mstapp.IMstAppKernel;
import de.desy.mst.libs.framework.mstapp.io.PrintDialogText;
import de.desy.mst.libs.framework.mstapp.logging.ILogFormatter;
import de.desy.mst.libs.framework.mstapp.logging.ILogProvider;
import de.desy.mst.libs.framework.mstapp.logging.LogLvl;
import de.desy.mst.libs.framework.mstapp.logging.LogMsg;
import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.logging.LoggerManager;
import de.desy.mst.libs.framework.mstapp.logging.appender.ILogAppenderKern;
import de.desy.mst.libs.framework.mstapp.logging.filter.HauptLogFilter;
import de.desy.mst.libs.framework.mstapp.logging.filter.ILogFilter;
import de.desy.mst.libs.framework.mstapp.logging.filter.StdLogFilter;
import de.desy.mst.libs.framework.mstapp.util.xml.AXmlBeschreibung;


/**
 * Zweck:     Anschluss des Logger an Swing<br>
 * Benutzung: im eigenen Object registrieren und am ende dergistrieren<br>
 * <li> swingLoggerManager = new SwingLoggerManager(); //fuer den Standard<br>
 *      swingLoggerManager = new SwingLoggerManager(22, Logger.AUFTRAG); //fuer Logging Messages vom Typ Auftrag 22 messages tief<br>
 * <li> optional: Konfigurieren wie: Aendern der Filterung und/oder Formatierung
 * <li> register(final JList jList)
 * <li> ... lauft, kann dabei auch umkonfiguriert werden
 * <li> deregister(final JList jList)
 * 
 * Vorsichtig: diese Klasse managed 
 * 	<li> Das Daten Modell (Manipulation via Filter Einstellungen, Size)
 * 	<li> Die Formatierung (Manipulation via LogFormatter Einstellungen)
 * 
 * @author ws
 *
 */
@SuppressWarnings({"rawtypes", "unchecked"})
public class SwingLoggerManager extends AXmlBeschreibung{
	private static final String NAME = "SwingLoggerManager";
	private JList jList = null;
    private final LogMsgListModel <LogMsg> dataModel	 	= new LogMsgListModel <LogMsg> ();
    private final JListLogCellRenderer 	 cellRenderer	= new JListLogCellRenderer();
    private ILogFilter	filter;
    private boolean		ensureLastElementIsVisible = true;
    private JCheckBoxMenuItem jCheckBoxMenuItemLastElement = null;
    private JMenuItem jMenuItemListeDrucken = null;
    private final LogAppender appender;
    private final IMstAppKernel mstApp;

    private class LogAppender extends AXmlBeschreibung implements ILogAppenderKern{
		final private ILogProvider logProvider;
		//---------------------------------------------------------------------
		public LogAppender(final ILogProvider logProvider) {
			super();
			this.logProvider = logProvider;
		}
		//---------------------------------------------------------------------
		@Override public void doLog(final LogMsg msg) {
			if(! getFilter().pass(msg)) return;//nothing to do
			/**inhalt muss in den UI Update Thread*/
			SwingUtilities.invokeLater( 
				new Runnable(){
					@Override public void run() {
						getModel().addElement(msg);
						if(SwingLoggerManager.this.ensureLastElementIsVisible) 
							ensureIndexIsVisible();
					}
					private void ensureIndexIsVisible() {
						final JList jList = SwingLoggerManager.this.jList;
						if(jList == null)
							return;
						final int index = getModel().getSize() - 1;
						if(index >= 0)
							jList.ensureIndexIsVisible(index);
					}
				});
		}
		@Override public void doLog(final List<LogMsg> msgList) {
			for (final LogMsg msg : msgList) {
				this.doLog(msg);
			}
		}
		@Override public void flush() {/*no buffering therefore no flushing*/}
		//---------------------------------------------------------------------
		public boolean connect(){
			getModel().clear(); 
			this.doLog(getLogProvider().getMessages());
			return getLogProvider().register(this);
		}
		public boolean disconnect(){return getLogProvider().deregister(this);}
		@Override public Element getBeschreibungAsElement(Document document) {
			if(document == null) document = createDocument();
			if(document == null) {return null;}	//in real trouble on a real minor feature
			
			final Element element = document.createElement(NAME + "-Appender");
			element.setAttribute("isConnected", Boolean.toString(getLogProvider().isRegistered(this)));
			return element;
		}
		private ILogProvider getLogProvider(){return this.logProvider;}
    }
    
//=============================================================================
	
	/**@throws NullPointerException*/
	public SwingLoggerManager(final IMstAppKernel mstApp){
		this(mstApp, LogMsgListModel.DEFAULT_SIZE);
	}
	/**@throws NullPointerException*/
	public SwingLoggerManager(final IMstAppKernel mstApp, final int capacity){
		this(mstApp, capacity, new HauptLogFilter(), LoggerManager.getSystemLogProvider());
	}
	/**@throws NullPointerException*/
	public SwingLoggerManager(final IMstAppKernel mstApp, final int capacity, final Logger kategorie){
		this(mstApp, capacity);
		final StdLogFilter filter = new StdLogFilter(kategorie);
		this.setFilter(filter);
	}
	/**@throws NullPointerException*/
	public SwingLoggerManager(final IMstAppKernel mstApp, final int capacity, final Logger kategorie, final LogLvl logLvl){
		this(mstApp, capacity); 
		final StdLogFilter filter = new StdLogFilter(kategorie, logLvl);
		this.setFilter(filter);
	}
	/**@throws NullPointerException*/
	public SwingLoggerManager(final IMstAppKernel mstApp, final int capacity, final ILogFilter filter){
		this(mstApp, capacity, filter, LoggerManager.getSystemLogProvider());
	}
	/**@throws NullPointerException*/
	public SwingLoggerManager(final IMstAppKernel mstApp, final int capacity, final ILogFilter filter, final ILogProvider logProvider){
		super();
		this.setCapacity(capacity);
		this.setFilter(filter);
		if(mstApp == null) {throw new NullPointerException("IMstAppKernel darf nicht null sein.");}
		this.mstApp = mstApp;
		this.appender = new LogAppender(logProvider);
	}
//=============================================================================
	
	/**muss immer deregistriert werden (sonst: memory leak), maximal eine jList != null pro Object*/
	public final void register(final JList jList){
		if(this.jList == jList) return;	//nichts zu tun
		if(jList == null) {
			this.disconnect();
		}else{
			if(this.jList != null) disconnect();	//nur maximal eine zur Zeit
			this.connect(jList);
		}
	}
	public final void deregister(final JList jList){disconnect();}
	
	/**0 <=  getSize() <= getCapacity() <br>tatsaechlich gespeicherte Eintraege*/ 
	public final int getSize() 		{return getModel().getSize();		}
	/**0 <=  getSize() <= getCapacity() <br>maximal verlustfrei speicherbare Eintraege*/ 
	public final int getCapacity() 	{return getModel().getCapacity();	}
	/**>=0, else ignored*/
	public final void setCapacity(final int capacity) {
		if(capacity < 0) return;
		getModel().setCapacity(capacity);
	}
	/**der buffer wird vollstaendig geleert, die Capacity aber erhalten*/
	public final void clear() {getModel().clear();}

	public final ILogFilter getFilter() {return this.filter;}
	public final  void 		setFilter(final ILogFilter filter) {
		if(filter == null){ throw new NullPointerException("Der filter darf nicht null sein.");}
		this.filter = filter;
	}
	public final boolean isEnsureLastElementIsVisible() {return this.ensureLastElementIsVisible;}
	public final void setEnsureLastElementIsVisible(final boolean ensureLastElementIsVisible) {
		this.ensureLastElementIsVisible = ensureLastElementIsVisible;
	}
	
	public LogMsg getMessage(final int listIndex) {
		return this.getModel().getEntry(listIndex);
	}
	public final List<LogMsg> getMessages() {
		return this.getModel().getEntries();
	}	
	public LogMsg getSelectedMessage() {
		try {
			final int jListIndex = this.jList.getSelectedIndex();
			return this.getMessage(jListIndex);
			
		} catch (final Exception e) {
			return new LogMsg(LogLvl.error, Logger.SWFW, "Kann den Eintrag nicht zuorden.", e);
		}
	}
	public final String[] getMessagesAsStringArray() {
		return getMessagesAsStringArray(this.getCellRenderer().getLogFormatter());
	}
	public final  String[] getMessagesAsStringArray(final ILogFormatter formatter) {
		final List<LogMsg> messages = this.getMessages();
		final String[] strings = new String[messages.size()];
		for (int i = 0; i < strings.length; i++) {
			strings[i] = formatter.format(messages.get(i));
		}
		return strings;
	}
	public final String getBeschreibung() {
		return this.getXMLBeschreibung(0);
	}
	@Override public Element getBeschreibungAsElement(Document document) {
		if(document == null) document = createDocument();
		if(document == null) {return null;}	//in real trouble on a real minor feature
		
		final Element element = document.createElement(NAME);
		element.setAttribute("ensureLastElementIsVisible", Boolean.toString(isEnsureLastElementIsVisible()));
		element.appendChild(getModel().BESCHREIBUNG.getBeschreibungAsElement(document));
		element.appendChild(getAppender().getBeschreibungAsElement(document));
		element.appendChild(getFilter().getBeschreibungAsElement(document));
		return element;
	}
	
//=============================================================================
		
	@Override protected void finalize() throws Throwable {
		if(this.jList != null){
			Logger.SWFW.error(this.getClass().getName() + " Fehler in der Benutzung. Vergessen jList zu deregistrieren.");
		}
		super.finalize();
	}
	
//=============================================================================
			
	/**never null*/
	private void connect(final JList jList) { 
		this.jList = jList;
		if( jList.getModel() != this.getModel()){
			jList.setModel(this.getModel());
		}
		jList.setCellRenderer(this.getCellRenderer());
		registerSpecialPopup();
		jList.addMouseListener(
				getMstApp().getPopUpMenuMgr().getStandardPopUpMouseListener());
		getAppender().connect();
	}
	private void disconnect() {
		try{
			this.jList.removeMouseListener(this.jList.getMouseListeners()[0]);
		}catch(final Exception e){	
		}
		getAppender().disconnect();
		this.jList = null;
	}
	
	/**never null*/	private final LogMsgListModel <LogMsg>	getModel()			{return this.dataModel;}	
	/**never null*/	private final JListLogCellRenderer		getCellRenderer()	{return this.cellRenderer;}
	/**never null*/	private final LogAppender     			getAppender()    	{return this.appender;}
	/**never null*/	        final IMstAppKernel 			        getMstApp() 	    {return this.mstApp;}
	public MouseListener getStandardSwingLoggerListMouseListener() {
		final MouseListener mouseListener = new java.awt.event.MouseAdapter() {
			/* (non-Javadoc)
			 * @see java.awt.event.MouseAdapter#mousePressed(java.awt.event.MouseEvent)
			 */
			@Override public void mousePressed(final java.awt.event.MouseEvent e) {
				//andere Betriebssysteme z. B. LINUX ?
				doMouseEvent(e);
			}
			/* (non-Javadoc)
			 * @see java.awt.event.MouseAdapter#mouseReleased(java.awt.event.MouseEvent)
			 */
			@Override public void mouseReleased(final java.awt.event.MouseEvent e) {
				//Windows (XP)
				doMouseEvent(e);
			}
		};
		return mouseListener; 
	}
	private void doMouseEvent(final java.awt.event.MouseEvent e){
		if(e.isPopupTrigger()){
			getMstApp().getPopUpMenuMgr().addSeparator();
			getMstApp().getPopUpMenuMgr().register(getJCheckBoxMenuItemLastElement());
			getMstApp().getPopUpMenuMgr().register(getJMenuItemListeDrucken());
			getMstApp().getPopUpMenuMgr().showMenu(e.getComponent(), e.getX(), e.getY());
			getMstApp().getPopUpMenuMgr().deRegister(getJCheckBoxMenuItemLastElement());
			getMstApp().getPopUpMenuMgr().deRegister(getJMenuItemListeDrucken());
		}
	}
	private JCheckBoxMenuItem getJCheckBoxMenuItemLastElement() {
		if (this.jCheckBoxMenuItemLastElement == null) {
			this.jCheckBoxMenuItemLastElement = new JCheckBoxMenuItem();
			this.jCheckBoxMenuItemLastElement.setText("Letztes Element der Liste anwhlen");
			this.jCheckBoxMenuItemLastElement.setSelected(true);
			this.jCheckBoxMenuItemLastElement.addActionListener(new java.awt.event.ActionListener() {
				@Override public void actionPerformed(final java.awt.event.ActionEvent e) {
					setEnsureLastElementIsVisible(SwingLoggerManager.this.jCheckBoxMenuItemLastElement.isSelected());
				}
			});
		}
		return this.jCheckBoxMenuItemLastElement;
	}
	private JMenuItem getJMenuItemListeDrucken() {
		if (this.jMenuItemListeDrucken == null) {
			this.jMenuItemListeDrucken = new JMenuItem();
			this.jMenuItemListeDrucken.setText("Inhalt der Liste drucken");
			this.jMenuItemListeDrucken.addActionListener(new java.awt.event.ActionListener() {
				@Override public void actionPerformed(final java.awt.event.ActionEvent e) {
					final PrintDialogText myPrinter = new PrintDialogText();
					myPrinter.setText(getMessagesAsStringArray());
					myPrinter.setHeadLine(SwingLoggerManager.this.jList.getName());
					myPrinter.setPageLine(getMstApp().getMaschineMgr().getMaschine().getLangenNamen() + " " + getMstApp().getStartParameter().appName);
					myPrinter.setHeaderFontSize(14);
					myPrinter.setTextFontSize(10);
					myPrinter.setFontName("Monospaced");
					myPrinter.setDefaultMaschine(getMstApp().getMaschineMgr().getMaschine()); 
					myPrinter.showdialog();
				}
			});
		}
		return this.jMenuItemListeDrucken;
	}
	private void registerSpecialPopup(){
		final List<JMenuItem> popup = new LinkedList<JMenuItem>();
		popup.add(getJCheckBoxMenuItemLastElement());
		popup.add(getJMenuItemListeDrucken());
		getMstApp().getPopUpMenuMgr().registerCopmponentForAdditionalPopup(this.jList, popup);		
	}
	public final ILogProvider getLogProvider() {return this.appender.logProvider;}
}
