/**
 * May 15, 2009 - May 28, 2009	ws:	Creation from scratch
 */
package de.desy.mst.libs.framework.mstapp.logging.map_tinelogsrv;

import java.util.Collections;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;

import de.desy.mst.libs.framework.mstapp.logging.EinfacherLogDeformatter;
import de.desy.mst.libs.framework.mstapp.logging.EinfacherLogFormatter;
import de.desy.mst.libs.framework.mstapp.logging.LogMsg;
import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.logging.map_tinelogsrv.__MapContext.Context_RawLogMsgMsg_Pair;

/**
 * Clog.text, Clog.context <-> Nr, Date, Kategorie(Logger), Eintrag(message)
 * <br> Encoding text: 		Nr|Date|Kategorie|Eintrag
 * <br> Encoding message: 	{Kontext}Originalmessage
 * <br>                     hierbei ist {Kontext} optional
 * @author ws
 * @version 1.0.0
 */
class __MapText {
	private static final ISimpleFormatter formatter = new ClogTextFormatter();
//-----------------------------------------------------------------------------
	/**LogMsg <- Clog*/
	public final static __MapText create(final String text, final String context){
		final LogMsgTextData lmtd = formatter.deformat(text, context);
		return new __MapText(lmtd, text, context);
	}
	/**LogMsg -> Clog*/
	public final static __MapText create( final long 	nr
										, final Date 	date
										, final Logger 	kategorie
										, final Context_RawLogMsgMsg_Pair 	pair){
		final LogMsgTextData lmtd = new LogMsgTextData(nr, date, kategorie, pair.rawLogMsgMessage);
		final String text    = formatter.format(lmtd);
		return new __MapText(lmtd, text, pair.cLogContext);
	}
//=============================================================================
	/**LogMsg*/private final LogMsgTextData lmtd;
	/**Clog*/  private final String text;
	/**Clog*/  private final String context;
//-----------------------------------------------------------------------------
	public __MapText( final LogMsgTextData lmtd, final String 	text, final String 	context) {
		this.lmtd	 = lmtd;
		this.text 	 = text;
		this.context = context;
	}
//-----------------------------------------------------------------------------
	/**LogMsg*/private final LogMsgTextData getLmtd() {return this.lmtd;}

	/**LogMsg*/public  final long 	getNr() 		{return getLmtd().nr;}
	/**LogMsg*/public  final Date 	getDate()		{return getLmtd().date;}
	/**LogMsg*/public  final Logger	getKategorie()	{return getLmtd().kategorie;}
	/**LogMsg*/public  final String getMessage() 	{return getLmtd().message;}

	/**Clog*/  public  final String getText() 	 {return this.text;}
	/**Clog*/  public  final String getContext() {return this.context;}
//-----------------------------------------------------------------------------
	@Override public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((this.context == null) ? 0 : this.context.hashCode());
		result = prime * result
				+ ((this.lmtd == null) ? 0 : this.lmtd.hashCode());
		result = prime * result
				+ ((this.text == null) ? 0 : this.text.hashCode());
		return result;
	}
	@Override public boolean equals(final Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final __MapText other = (__MapText) obj;
		if (this.context == null) {
			if (other.context != null)
				return false;
		} else if (!this.context.equals(other.context))
			return false;
		if (this.lmtd == null) {
			if (other.lmtd != null)
				return false;
		} else if (!this.lmtd.equals(other.lmtd))
			return false;
		if (this.text == null) {
			if (other.text != null)
				return false;
		} else if (!this.text.equals(other.text))
			return false;
		return true;
	}


//=============================================================================
	/**TineInfo.text corresponding data in LogMsg*/
	private static class LogMsgTextData{
		public final long nr;
		public final Date date;
		public final Logger kategorie;
		public final String message;
		LogMsgTextData(final long nr, final Date date, final Logger kategorie, final String message) {
			this.nr = nr;
			this.date = date;
			this.kategorie = kategorie;
			this.message = message;
		}
		@Override public int hashCode() {
			final int prime = 31;
			int result = 1;
			result = prime * result
					+ ((this.date == null) ? 0 : this.date.hashCode());
			result = prime
					* result
					+ ((this.kategorie == null) ? 0 : this.kategorie.hashCode());
			result = prime * result
					+ ((this.message == null) ? 0 : this.message.hashCode());
			result = prime * result + (int) (this.nr ^ (this.nr >>> 32));
			return result;
		}
		@Override public boolean equals(final Object obj) {
			if (this == obj)
				return true;
			if (obj == null)
				return false;
			if (getClass() != obj.getClass())
				return false;
			final LogMsgTextData other = (LogMsgTextData) obj;
			if (this.date == null) {
				if (other.date != null)
					return false;
			} else if (!this.date.equals(other.date))
				return false;
			if (this.kategorie == null) {
				if (other.kategorie != null)
					return false;
			} else if (!this.kategorie.equals(other.kategorie))
				return false;
			if (this.message == null) {
				if (other.message != null)
					return false;
			} else if (!this.message.equals(other.message))
				return false;
			if (this.nr != other.nr)
				return false;
			return true;
		}

	}
	private interface ISimpleFormatter{
		public String format(final LogMsgTextData lmtd);
		public LogMsgTextData deformat(final String text, final String context);
	}
	/**
	 * QUICK AND VERY DIRTY
	 * <br> encapsulate via ISimpleFormatter
	 * @author ws
	 * @version 1.0.0
	 */
	private static class ClogTextFormatter	extends 	EinfacherLogFormatter
											implements	ISimpleFormatter{
		/**Different LogMsg Parts are separated in Clog.text by that*/
		private static final String TEXT_SEPARATOR	= "|";	//\t wird in label schlecht angezeigt (Clog Rohdarstellung)

		private static final List<StringPair> escapedSubStrings;
		private static final List<StringPair> reverseEscapedSubStrings;
		static {
			final LinkedList<StringPair> list = new LinkedList<StringPair>();
			final LinkedList<StringPair> reverseList = new LinkedList<StringPair>();
			add(list, reverseList, "&", "&amp;");
			add(list, reverseList, "\t", "&tab;" );	//(key, value)
		  //add(list, reverseList, "\n", "&nl;"  );	// not anymore "|" due to Consolewindow direct jump to errors feature in eclipse
			add(list, reverseList, "\"", "&quot;");
			add(list, reverseList, TEXT_SEPARATOR, "&bar;");
			add(list, reverseList, NEWLINEPAIR);	//nur temporaer
			list.remove(NEWLINEPAIR);	//nur zum dekodieren
			escapedSubStrings        = Collections.unmodifiableList(list);
			reverseEscapedSubStrings = Collections.unmodifiableList(reverseList);
		}


	//=========================================================================
		private final EinfacherLogDeformatter deformatter;
	//=========================================================================
		private ClogTextFormatter() {
			this.deformatter = new EinfacherLogDeformatter(this);
			this.deformatter.deformatSpaltenTitel(this.getHeader(false));
		}
	//=========================================================================
		/**Subklassen wie ClogTextFormatter haben eigene escapedSubStrings*/
		@Override protected List<StringPair> getEscapedSubStrings() {
			return ClogTextFormatter.escapedSubStrings;
		}
		/**Subklassen wie ClogTextFormatter haben eigene escapedSubStrings*/
		@Override protected List<StringPair> getReverseEscapedSubStrings() {
			return ClogTextFormatter.reverseEscapedSubStrings;
		}
	//=========================================================================
		@Override public String getHeader(final boolean withMultiMessageSeparator) {
			return 					  HEADER_NR			//nrFormat(logMsg.nr)
			 + getSEPARATOR_InMSG() + HEADER_DATUM		//getFormatDate().format(logMsg.date)
			 + getSEPARATOR_InMSG() + HEADER_KATEGORIE	//kategorieFormat(logMsg.kategorie)
			 + getSEPARATOR_InMSG() + HEADER_MELDUNG	//msgAndThrowableFormat(logMsg.message, logMsg.throwable)
			 + (withMultiMessageSeparator ? getMultiMessageSeparator() : "")
			 ;
		}
		@Override public String format(final LogMsg logMsg) {
			final String msg =
				nrFormat(logMsg.nr)
			  + getSEPARATOR_InMSG() + getFormatDate().format(logMsg.date)
			  + getSEPARATOR_InMSG() + kategorieFormat(logMsg.kategorie)
			  + getSEPARATOR_InMSG() + msgAndThrowableFormat(logMsg.message, logMsg.throwable)
			  ;
			return getReduceThrowable() ? encodeTheNewLine(msg) : msg;
		}

		/**never pad*/
		@Override public String pad(final String string) {return string;}

		/**wird ignoriert, da immer true*/
		@Override public void setReduceThrowable(final boolean reduceThrowable) {
			super.setReduceThrowable(reduceThrowable);
		}
		@Override public boolean getReduceThrowable() {return true;}
		@Override public String  getSEPARATOR_InMSG() {return ClogTextFormatter.TEXT_SEPARATOR;}
		public final EinfacherLogDeformatter getDeformatter() {return this.deformatter;}
	//=========================================================================
		/* (non-Javadoc)
		 * @see de.desy.mst.libs.framework.mstapp.logging.map_tinelogsrv.__MapText.ISimpleFormatter#deformat(java.lang.String, java.lang.String)
		 */
		@Override public LogMsgTextData deformat(final String text, final String context) {
			final LogMsg lm = getDeformatter().deformatMessageEntry(text);
			final String logMsgMessage = __MapContext.map(context, lm.message);
			return new LogMsgTextData(lm.nr, lm.date, lm.kategorie, logMsgMessage);
		}
		/* (non-Javadoc)
		 * @see de.desy.mst.libs.framework.mstapp.logging.map_tinelogsrv.__MapText.ISimpleFormatter#format(long, java.util.Date, de.desy.mst.libs.framework.mstapp.logging.Logger, java.lang.String)
		 */
		@Override public String format(final LogMsgTextData lmtd) {
			final String msg =				nrFormat(lmtd.nr)
				  + getSEPARATOR_InMSG() + 	getFormatDate().format(lmtd.date)
				  + getSEPARATOR_InMSG() + 	kategorieFormat(lmtd.kategorie)
				  + getSEPARATOR_InMSG() + 	decorate(msgFormat(lmtd.message))
				  ;
			return encodeTheNewLine(msg);
		}
	}//END class:	ClogTextFormatter
}
