/**
 * Jan 14, 2008 - Jan 14, 2008	ws:	Creation from scratch
 * Jun 10, 2008 - Jul  8, 2008	ws: support reloadData(), dataInterval, setStart/Stop
 * Aug  1, 2008 - Aug  1, 2008	ws:	added: hasMassiveDecodingProblems()
 * Oct  6, 2008 - Oct  6, 2008	ws:	changed toString so ok for PrintOutput
 * Feb 20, 2009 - Feb 20, 2009	ws: restrict debug out to reloadAllData
 */
package de.desy.mst.libs.framework.mstapp.logging.reader;

import java.io.File;
import java.io.IOException;
import java.io.StringReader;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.atomic.AtomicLong;

import de.desy.mst.libs.framework.mstapp.logging.ILogProvider;
import de.desy.mst.libs.framework.mstapp.logging.LogMsg;
import de.desy.mst.libs.framework.mstapp.logging.LogUtil;
import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.logging.appender.ILogAppenderKern;
import de.desy.mst.libs.framework.mstapp.util.Convert;

/**
 * ILogProvider Zugang zu einer LogReader Quelle
 * @author ws
 * @version 1.0.0
 */
abstract public class LogReaderProvider implements ILogProvider {
	/**
	 * @param file  not null
	 * @return ein gueltiger Provider
	 * @throws IOException
	 */
	public static LogReaderProvider createFrom(final File file) throws IOException{
		return createFrom(file, 0, 10000-1);
	}
	/**start stop inclusive*/
	public static LogReaderProvider createFrom(final File file, final int start, final int stop) throws IOException{
		return new LRPFile(start, stop, file);
	}

	public static LogReaderProvider createFrom(final String s){
		return createFrom(s, 0, Integer.MAX_VALUE);
	}
	/**start stop inclusive Zeiten*/
	public static LogReaderProvider createFrom(final String s, final int start, final int stop){
		try {
			final LogReaderProvider provider = new LRPString(start, stop, s);
			return provider;
		} catch (final IOException e) {/*never happens on strings*/return null;}
	}
//=============================================================================

	/**index of first message to read >= 0*/private long start;
	/**index of last  message to read     */private long stop;
	private List<LogMsg> logMsgs = new LinkedList<LogMsg>();	//never null
	private final List<ILogAppenderKern> observers = new LinkedList<ILogAppenderKern>();
	private boolean hasMoreMessages = true;
	private boolean hasMassiveDecodingProblems = false;
//=============================================================================

	/**start stop inclusive*/
	LogReaderProvider(final long start, final long stop) throws IOException {
		setStart(start);
		setStop (stop);
	}

//=============================================================================
	/**index of first message to read >= 0*/
	public long getStart() 	{return this.start;}
	/**index of last  message to read     */
	public long getStop() 	{return this.stop;}
	/**best guess*/
	public boolean hasMoreMessages() {return this.hasMoreMessages;}
	public void setStart(final long start){this.start = Math.max(0, start);}//>=0
	public void setStop (final long stop ){this.stop  = stop;}
	public long countTotalAmountOfMessages() throws IOException {
		return getSize() + (hasMoreMessages()? 1:0);	//Schaetzung
	}

//=============================================================================

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.ILogProvider#getMessage(int)
	 */
	@Override public LogMsg getMessage(final int index) {return this.logMsgs.get(index);}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.ILogProvider#getMessages()
	 */
	@Override public List<LogMsg> getMessages() {
		return Collections.unmodifiableList(this.logMsgs);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.ILogProvider#getSize()
	 */
	@Override public int getSize() {return this.logMsgs.size();}



	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.ILogProvider#isRegistered(de.desy.mst.libs.framework.mstapp.logging.appender.ILogAppenderKern)
	 */
	@Override public boolean isRegistered(final ILogAppenderKern appender) {
		appender.doLog(this.getMessages());
		appender.flush();
		return this.observers.contains(appender);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.ILogProvider#register(de.desy.mst.libs.framework.mstapp.logging.appender.ILogAppenderKern)
	 */
	@Override public boolean register(final ILogAppenderKern logAppenderKern) {
		return this.observers.add(logAppenderKern);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.ILogProvider#deregister(de.desy.mst.libs.framework.mstapp.logging.appender.ILogAppenderKern)
	 */
	@Override public boolean deregister(final ILogAppenderKern logAppenderKern) {
		return this.observers.remove(logAppenderKern);
	}

//=============================================================================
	public boolean hasMassiveDecodingProblems() {
		return this.hasMassiveDecodingProblems  ;
	}

	void reloadAllData(final LogReader reader) throws IOException {
		this.logMsgs = readAllMessages(reader);
		reader.close();
	}
	private List<LogMsg> readAllMessages(final LogReader reader) throws IOException {
		final LinkedList<LogMsg> list = new LinkedList<LogMsg>();
		reader.skipMsgs(getStart());
//		final long freeMemoryAtBeginging = Runtime.getRuntime().freeMemory();
//		_memoryDebugOut(list, "before                           ", freeMemoryAtBeginging);
		while( (reader.getNumberOfMessagesRead() <= getStop()) ) {
			try {
				final LogMsg msg = reader.readMsg();
				if(msg == null)
					break;
				list.add(msg);
			} catch (final OutOfMemoryError e) {
//				_memoryDebugOut(list, "OutOfMemoryError before          ", freeMemoryAtBeginging);
				//60% da auch noch ein Rowsorter in der Tabelle erzeugt werden muss
				removeLastMessages(Math.max(1000, Convert.toInt(list.size()*.6)), list);
//				_memoryDebugOut(list, "OutOfMemoryError after removal   ", freeMemoryAtBeginging);
				System.gc();
//				_memoryDebugOut(list, "OutOfMemoryError after removal/gc", freeMemoryAtBeginging);
				final String msg = "Out of Memory: reduziere geladene Meldungen um mindestens 60% und lese nicht weiter.";
				Logger.SWFW.error(msg);
				break;
			}
		}
//		_memoryDebugOut(list, "after", freeMemoryAtBeginging);
		this.hasMoreMessages = reader.hasMoreMessages();
		this.hasMassiveDecodingProblems = reader.hasMassiveDecodingProblems();
		return list;
	}
//	private void _memoryDebugOut(final LinkedList<LogMsg> list, final String string, final long freeMemOffset) {
//		final long freeMemory = Runtime.getRuntime().freeMemory();
//		System.out.println("LogReaderProvider.readAllMessages (" + string + ")"
//						 + " nMsgs, freeMemory, totalMemory, delFree/Msg "
//						 + list.size()
//						 + ", " + freeMemory
//						 + ", " + Runtime.getRuntime().totalMemory()
//						 + (list.size() == 0 ? ""
//							 : ", " + (freeMemory - freeMemOffset)/list.size())
//						  );
//	}
	private static void removeLastMessages(int i, final LinkedList<LogMsg> list) {
		while( (i-- > 0) && (list.size() > 0))
			list.removeLast();
	}

//=============================================================================

	/**LogReaderProvider fuer Dateien*/
	private static final class LRPFile extends LogReaderProvider {
		private static AtomicLong nReload = new AtomicLong(0);
		private final File file;
		private LRPFile(final long start, final long stop, final File file) throws IOException {
			super(start, stop);
			this.file = file;
			reloadAllData();
		}
		@Override public void reloadAllData()  throws IOException {
			final long _start = System.currentTimeMillis();
			reloadAllData(LogReader.createFileReader(this.file));
			if(nReload.incrementAndGet() >= 5)
				return;	//no debug output anymore
			final String msg = "LogReaderProviderLRPFile "
							 + (this.getStart() <= 0 ? ""
									 			: this.getStart()
									 			  + " Meldungen bersprungen, ")
							 + this.getSize()
							 + " Meldungen geladen. " + this.file;
			Logger.SWFW.debug(LogUtil.timedMsg(_start, msg));
		}
		@Override public long countTotalAmountOfMessages() throws IOException {
			if(!hasMoreMessages())
				return getSize();
			final LogReader reader = LogReader.createFileReader(this.file);
			reader.skipMsgs(Long.MAX_VALUE);	//hoert bei Dateiende auf
			final long nMsgs = reader.getNumberOfMessagesRead();
			reader.close();
			return nMsgs;
		}
		@Override public String toString() {return this.file.toString();}
	}//END: LogReaderProvider LRPFile
//=============================================================================
	private static final class LRPString extends LogReaderProvider {
		private final String s;
		private LRPString(final long start, final long stop, final String s) throws IOException {
			super(start, stop);
			this.s = s;
			reloadAllData();
		}
		@Override public void reloadAllData()  throws IOException {
			reloadAllData(new LogReader(new StringReader(this.s)));
		}
	}//END: LogReaderProvider LRPString
}
