/*
 * Created on May 2, 2005 - Jun 16, 2005
 *	Oct 25, 2005 - ...			ws:	new logging system
 *	May  8, 2008 - May  8, 2008	ws: added: don't log empty or double messages
 *
 */
package de.desy.mst.libs.framework.mstapp.messages;

import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Set;

import de.desy.mst.libs.framework.mstapp.logging.Logger;

/**
 * Dispatches messages,
 * fatal ones are displayed in JOptionPane
 * @author schuette
 * @version 1.0.0
 */
//@SuppressWarnings({"rawtypes", "unchecked"})
public class MessageMgr implements ISetMessage {
    public  static final int MAX_MESSAGES    = 100;
    private static final int INITIALCAPACITY =   3;
    private final Set<ISetMessage>    observers = new HashSet<ISetMessage>(INITIALCAPACITY);
    private final LinkedList<Message> messages  = new LinkedList<Message>();
    private int nMessages = 0;

    /**
     * @param setMessage typically the MainFrame for silent Display of messages
     */
    public MessageMgr(final ISetMessage setMessage) {
        super();
        if(setMessage != null) this.observers.add(setMessage);
    }

    //all messages go to option pane
    public MessageMgr() {
        this(null);
    }

//=============================================================================

    /* (non-Javadoc)
     * @see de.desy.mst.mstapp.ISetMessage#setMessage(java.lang.String, java.lang.String, int)
     */
    @Override
	public final void setMessage(final Message message) {
        if(message == null){
            Logger.SWFW.error("New Msg is strangely enough null. Ignore it.", new NullPointerException());
            return;
        }

        if(		!message.text.isEmpty()					//keine leere message 
           && 	this.observers.size() == 0				//nur innerster logged (Hack)
           &&   !message.text.equals("KeineIniDatei")	//haeufigster Fall eh gelogged
          )	
        	Logger.SWFW.info("New Msg: '" + message.text + "'");

        this.messages.addLast(message);
        if(getNMessages() >= MAX_MESSAGES){this.messages.removeFirst();}
        else                              {this.nMessages++;}

        fireAllObservers(message);

        if(message.messageType == Message.ERROR_MESSAGE){
            //Msg wird penetrant als modaler Msg Dialog gezeigt
            message.showMessageDialog();
        }
    } 

    public final boolean wasASuccess(){
        final int lastMessageType = getLastMessage().messageType;
        return (lastMessageType == Message.INFORMATION_MESSAGE)
            || (lastMessageType == Message.PLAIN_MESSAGE);
    }

    /**
     * @return s the last Message or a default Message
     */
    public Message getLastMessage() {
        if(this.messages.isEmpty()){
            return this.getNullMessage();
        }
        return this.messages.getLast();
    }

    public final int getNMessages() {
        return this.nMessages;
    }

   /**
     * @param iSetMessage wird bei Registrierung und jeder neuen Message gesetzt
     *        wobei alle
     */
    public final void add(final ISetMessage iSetMessage ) {
        if(iSetMessage != null){
            this.observers.add(iSetMessage);
            fireAllMessagesToSingleObserver(iSetMessage);
        }
    }

    /**
     * @param iSetMessage deregistrierung (Gegenteil von add)
     */
    public final void remove(final ISetMessage iSetMessage) {
        this.observers.remove(iSetMessage);
    }
//=============================================================================

    @SuppressWarnings("rawtypes") private final void fireAllObservers(final Message message) {
        for (final Iterator iter = this.observers.iterator(); iter.hasNext();) {
            fireSingleObserver((ISetMessage) iter.next(), message);
        }
    }
    private final void fireSingleObserver(final ISetMessage iSetMessage, final Message message) {
    	iSetMessage.setMessage (message);
    }
    @SuppressWarnings("rawtypes") private void fireAllMessagesToSingleObserver(final ISetMessage iSetMessage) {
        for (final Iterator iter = this.messages.iterator(); iter.hasNext();) {
            final Message message = (Message) iter.next();
            fireSingleObserver(iSetMessage, message);
        }
    }

     private final Message getNullMessage(){
        return new Message("", "No message existing.", Message.PLAIN_MESSAGE);
    }
}
