/*
 * Created on Apr 7, 2005 - Apr 7, 2005, Jul 7 2005
 * 				Oct 25, 2005 - ...	new logging system
 *
 */
package de.desy.mst.libs.framework.mstapp.mode;

import java.awt.Component;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import de.desy.mst.libs.framework.mstapp.logging.Logger;

/**
 * setEnabled, setVisible oder aehnliches sollen je nach OpMode und OpModeListe fuer eine Komponente 
 * gesetzt werden
 * dies wird in der Abstracten Methode setValue definiert
 * 
 * package private
 * 
 * @author schuette
 * @version 1.0.0
 */
abstract class ASetComponentValueByOpModeAndListe implements IOpModeObserver{
	class Entry {
		private final Component 	component;
		private final OpModeListe 	opModeListe;
		Entry(final Component component,final OpModeListe opModeListe){
			this.component = component;
			this.opModeListe = opModeListe;
		}
	}
	private final Set<Entry> entries = new HashSet<Entry>();
	private OpMode lastOpMode;
	
//	=============================================================================
		
	/**
	 * @param opMode StartOpMode (gerade aktueller) != null
	 * @throws NullPointerException falls opMode == null
	 */
	public ASetComponentValueByOpModeAndListe(final OpMode opMode) {
		super();
		this.setOpMode(opMode);
	}
	
//=============================================================================
	
	abstract void setValue(Component component, boolean b); 
	
	public final void add(final Component component, final OpModeListe opModeListe){
		if(component == null){
			Logger.SWFW.warn("Strange usage component is empty");
			return;
		}
		if(opModeListe == null){
			Logger.SWFW.warn("Strange usage opModeListe is empty");
			return;
		}
		if(opModeListe == OpModeListe.EMPTY_LISTE ){
			Logger.SWFW.warn("Strange usage opModeListe is OpModeListe.EMPTY_LISTE");
			return;
		}
		if(this.hasComponent(component)){
			Logger.SWFW.warn("Strange usage opModeListe is Component " + component + " schon konfiguriert.");
			return;
		}
		final Entry entry = new Entry(component, opModeListe);
		setValue(entry, this.lastOpMode);
		this.entries.add(entry); 
	}

	public final void remove(final Component component){
		this.entries.remove(getEntryForComponent(component));
	}

	@SuppressWarnings("rawtypes") private Entry getEntryForComponent(final Component component) {
		for (final Iterator iter = this.entries.iterator(); iter.hasNext();) {
			final Entry  element = (Entry ) iter.next();
			if(element.component == component){ return element;}
		} 
		return null;
	}

	public final boolean hasComponent(final Component component) {
		return (null != getEntryForComponent(component));
	}

	/**
	 * @param  opMode 				StartOpMode (gerade aktueller) != null
	 * @throws NullPointerException falls opMode == null
	 */
	public final void setOpMode(final OpMode opMode) {
		if(opMode == null){throw  new NullPointerException("opMode can not be null");}
		
		if( this.lastOpMode == opMode){
			return;
		}
		this.lastOpMode = opMode;
		fireOpModeChanged(opMode);
	}

	
	@Override public void onOpModeChange(final OpMode opMode) {
		this.setOpMode(opMode);
	}
//=============================================================================
	
	@SuppressWarnings("rawtypes") private void fireOpModeChanged(final OpMode opMode) {
		for (final Iterator iter = this.entries.iterator(); iter.hasNext();) {
			setValue((Entry) iter.next(), opMode);
		}
	}

	private final void setValue(final Entry entry, final OpMode opMode){
		for (int i = 0; i < OpModeListe.VALUES.length; i++) {
			final boolean b = ( entry.opModeListe.isInList(opMode) );
			setValue(entry.component, b);
		}  
	}

}
