/*
 * Created on 2005: Feb 10 - Apr 7, Jul 7
 * Oct 25, 2005 - ...			new logging system
 * Sep 12, 2006 - Sep 12, 2006	ws: added setOpMode(..., PASSWORT);
 * Jun 19, 2007 - Jun 20, 2007	ws:	Lokalisierung auf Deutsch etwas nachgezogen
 * 									registerIOpModeObserver(), XTor:
 * 										fire new registree
 *	Feb 15, 2008 - Feb 15, 2008 ws: added "@Override"es
 * 	Oct  2, 2009 - Oct 20, 2009	ws:	support fuer Rubberband
 */

package de.desy.mst.libs.framework.mstapp.mode;

import java.awt.Component;
import java.awt.event.ItemEvent;
import java.security.AccessControlException;
import java.util.Iterator;
import java.util.Set;
import java.util.HashSet;
import java.util.concurrent.atomic.AtomicReference;

import javax.swing.AbstractButton;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JRadioButtonMenuItem;

import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.security.MstSecurity;
import de.desy.mst.libs.framework.mstapp.tabellendaten.entry.OPModeACLEntry;
import de.desy.mst.libs.framework.mstapp.util.observables.setvalue.SetTextManager;

/**
 * Operations Mode Manager
 * The one and only place where the true opmode of the current app is defined
 * (via )
 * The OpMode is never null.
 * Registered Objects will be automatically updated or notified.
 *
 * @author schuette
 *  @link OpMode
 */
//@SuppressWarnings({"rawtypes", "unchecked"})
public final class OpModeMgr implements IOpModeKonfigurationObserver, IRegisterForCaption {
	/**
	 * Usecase: Die Applikation startet im BKR ist im Betriebsmode,
	 * der Expertenmode wird angewhlt und wenig spaeter das Programm so
	 * zurckgelassen. Dies ist ein unerwartet unsicherer Zustand. Deshalb
	 * soll nach maxIdleTimeMs bei User Inaktivitaet die Applikation wieder in
	 * den Default Mode zurck gefhrt werden.
	 * <br>
	 * <br> DestinationMode is OpMode.Betrieb
	 * <br> currentMode     is OpMode.Expert
	 * <br> isPossibleToEnable(), isEnabled(), isActive() are all true
	 * <br> getMinRestTimeMS() is smaller than MAX_VALUE
	 * <br>
	 * <br> Einschalten lediglich setEnable(true) beim Starten der Applikation
	 * 		(MstFrameMain Subclass Constructor oder afterFrameCreated()).
	 * 	    Der Rest wird automatisch erledigt
	 */
	public interface IRubberband {
		/**Destination OpMode of the rubberband, is at beginning the default OpMode for the user, pc, application*/
		public abstract OpMode getDestinationMode();
		/**if destinationMode is not allowed, the actual setDestination Mode will be returned*/
		public abstract OpMode setDestinationMode(final OpMode destinationMode);

		/**Time of UI inactivity until the opMode is set to getDestinationMode()*/
		public abstract int getMaxIdleTimeMS();
		public abstract int	setMaxIdleTimeMS(final int maxIdleTime);

		/**nur moeglich zu benutzen, wenn ein SessionActivityMonitor erzeugt werden kann*/
		public abstract boolean isPossibleToEnable();

		/**( currentOpMode != getDestinationMode()) && isEnabled() ) <=> isOn() )*/
		public abstract boolean isEnabled();
		/**
		 * !!!Dieses muss true gesetzt werden zum Nutzen dieses Features,
		 * der Rest ist automatisch
		 * @return (enabled && isPossibleToTurnOn())
		 */
		public abstract boolean setEnabled(final boolean enabled);

		/**timer is running and might switch OpMode back later*/
		public abstract boolean isActive();

		/**Time left until switchback, if no user activity*/
		public abstract int getMinRestTimeMS();

		/**@return 0 bis 100%, 100% nur bei getDestinationMode() == getOpMode()*/
		public abstract int getProgress();
		}
//=============================================================================
	private OpModeKonfiguration opModeKonfiguration;

	private JMenu jMenuOpMode = null;
	private JRadioButtonMenuItem[] opModeJMenuItems = null;
	private final AtomicReference<__OpModeRubberband> refRubberband = new AtomicReference<__OpModeRubberband>(null);

	//changes all registered texts to current OpMode caption
	private final SetTextManager captionMgr;
	private final Set<IOpModeObserver> lowLevelOpModeObservers = new HashSet<IOpModeObserver>(); //of IOpModeObserver
	final ASetComponentValueByOpModeAndListe enableMgr = new ASetComponentValueByOpModeAndListe(OpMode.BETRACHTEN){
		/* (non-Javadoc)
		 * @see de.desy.mst.libs.framework.mstapp.mode.ASetComponentValueByOpModeAndListe#setValue(java.awt.Component, boolean)
		 */
		@Override void setValue(final Component component, final boolean b) {component.setEnabled(b);}
    };
    final ASetComponentValueByOpModeAndListe visibleMgr = new ASetComponentValueByOpModeAndListe(OpMode.BETRIEB){
		/* (non-Javadoc)
		 * @see de.desy.mst.libs.framework.mstapp.mode.ASetComponentValueByOpModeAndListe#setValue(java.awt.Component, boolean)
		 */
		@Override void setValue(final Component component, final boolean b) {
			component.setVisible(b);
		}
    };

//	=============================================================================

	public OpModeMgr(OpModeKonfiguration opModeKonfiguration) {
		super();
		if(opModeKonfiguration == null){
			final OPModeACLEntry aclEntry = MstSecurity.get().getOPModeACLEntry();
			Logger.SWFW.info(aclEntry.toLoggerInfo());
			opModeKonfiguration = OpModeKonfiguration.createKonfiguration(aclEntry);
		}
		this.opModeKonfiguration = opModeKonfiguration;
		this.captionMgr = new SetTextManager("OpModeMgr.XTor sollte nie zu sehen sein.");
		this.lowLevelOpModeObservers.add(this.enableMgr);
		this.lowLevelOpModeObservers.add(this.visibleMgr);
		fireOpModeObservers(this.getOpMode());
	}
	public OpModeMgr() {this(null);}

//	=============================================================================
 

	/**
	 *
	 * @param 	jMenuOptions 			Hier werden die Untermenues drangehaengt
	 * @param 	opModeObserver 			werden bei Aenderung vom OpMode benachrichtigt
	 * @throws	NullPointerException	theOptionMenu darf nicht null sein
	 * @throws	IllegalStateException	darf nur genau einmal aufgerufen werden, menuIndex must be legal
	 */
	public final void initAndRegisterOptionMenu(final JMenu jMenuOptions, final int menuIndex, final IOpModeObserver opModeObserver) {
		//proctection against bad input
		final String msgPrefix = "OpModeMgr.initAndRegisterServerAnwahlMenu(theOptionMenu): ";
		if(jMenuOptions == null){
			throw new NullPointerException(msgPrefix + "theOptionMenu may not be null");
		}
		if(this.jMenuOpMode != null){
			throw new IllegalStateException(msgPrefix + "was called more than once.");
		}

		//real bussiness
		this.jMenuOpMode  = new JMenu();
		jMenuOptions.add(this.jMenuOpMode);
		final OpMode[] opModes = OpMode.VALUES;
		this.opModeJMenuItems = new JRadioButtonMenuItem[opModes.length];
		for (int i = 0; i < opModes.length; i++) {
			this.opModeJMenuItems[i] = initMenuItem(this.jMenuOpMode, opModes[i]);
		}
		this.registerForCaption(this.jMenuOpMode);
		this.fireOpModeKonfigurationObservers(this.getOpModeKonfiguration());

		this.registerIOpModeObserver(opModeObserver);
	}

	private final JRadioButtonMenuItem initMenuItem(final JMenu jMenuOpMode, final OpMode opMode) {
		String itemName = opMode.name;
		if(this.getOpModeKonfiguration().isGeschuetzt(opMode)){
			itemName += " (Passwort)";
		}
		final JRadioButtonMenuItem  menuItem = new JRadioButtonMenuItem(itemName);
		//menuItem.setSelected(opMode == this.getOpMode());
		//menuItem.setEnabled(this.getOpModeKonfiguration().isPublic(opMode));
		menuItem.addItemListener(new OpModeChangedListener(opMode));
		jMenuOpMode.add(menuItem);
		return menuItem;
	}

	private class OpModeChangedListener implements java.awt.event.ItemListener {
		private final OpMode opMode;
		public OpModeChangedListener(final OpMode opMode) {
			super();
			this.opMode = opMode;
		}
		@Override public void itemStateChanged(final java.awt.event.ItemEvent e) {
			if(e.getStateChange() == ItemEvent.SELECTED){
				try {
					setOpMode(this.opMode);
				} catch (final AccessControlException  ex) {
					Logger.SWFW.error("OpModeChangedListener: Tried to access a mode without correct authorisation");
					fireOpModeObservers(getOpMode());	//should always succeed
				} catch (final Exception ex1) {
					Logger.SWFW.error("OpModeChangedListener: Unexpected exception. " + ex1.toString());
					fireOpModeObservers(getOpMode());	//should always succeed
				}
			}
		}
		/* (non-Javadoc)
		 * @see java.lang.Object#toString()
		 */
		@Override public String toString() {
			return super.toString() + " (" + this.opMode.toString() + ")";
		}
	}


	/**
	 * @return current OpMode, never null
	 */
	public final OpMode getOpMode() {
		return this.opModeKonfiguration.getOpMode();
	}
	/**
	 * @param  opMode 					Ziel OpMode, muss gesetzt werden duerfen
	 * 									(implementiert, erfolgreicher Passwortcheck)
	 * @throws AccessControlException 	fuer passwortgeschuetzte OpModes bei falscher Legitimation
	 * @throws IllegalArgumentException opMode muss setzbar (implementiert) sein
	 */
	public final void setOpMode(final OpMode opMode, final String passwort) {
		if(opMode != this.getOpMode()) {
			this.getOpModeKonfiguration().setOpMode(opMode, passwort);
			fireOpModeObservers(opMode);
		}
	}
	@Deprecated
	public final void setOpMode(final OpMode opMode) {
		this.setOpMode(opMode, "");
	}

	public final void registerIOpModeObserver(final IOpModeObserver opModeObserver){
		if(opModeObserver != null) {
			this.lowLevelOpModeObservers.add(opModeObserver);
			opModeObserver.onOpModeChange(getOpMode());
		}else{
			//null silently ignored
		}
	}
	public final void deregisterIOpModeObserver(final IOpModeObserver opModeObserver){
		this.lowLevelOpModeObservers.remove(opModeObserver);
	}

	//label Text will be kept in synch with current opmode
	@Override public final void registerForCaption(final JLabel label){
		this.captionMgr.add(label);
	}
	//Button, Menu, ... -text will be kept in synch with current OpMode
	@Override public final void registerForCaption(final AbstractButton menuItemOrButtonOrEtc){
		this.captionMgr.add(menuItemOrButtonOrEtc);
	}
	@Override public final void deregisterForCaption(final Object object) {
		this.captionMgr.remove(object);
	}

	public final void registerForEnablement(final Component component, final OpModeListe enabledFallsInListe){
		this.enableMgr.add(component, enabledFallsInListe);
	}
	public final void deregisterForEnablement(final Component component){
		this.enableMgr.remove(component);
	}

	public final void registerForVisibility(final Component component, final OpModeListe visibleFallsInListe){
		this.visibleMgr.add(component, visibleFallsInListe);
	}
	public final void deregisterForVisibility(final Component component){
		this.visibleMgr.remove(component);
	}


	public final OpModeKonfiguration getOpModeKonfiguration() {
		return this.opModeKonfiguration;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.mstapp.mode.IOpModeKonfigurationObserver#onKonfigurationChange(de.desy.mst.mstapp.mode.OpModeKonfiguration)
	 */
	@Override public final void onKonfigurationChange(final OpModeKonfiguration konfiguration) {
		checkNotNull(this.opModeKonfiguration, "opModeKonfiguration");
		this.opModeKonfiguration = konfiguration;
		fireOpModeKonfigurationObservers(konfiguration);
	}

	/**
	 * The Rubberband (Gummiband) lets a changed OpMode always go back to the
	 * OpMode at the time of first calling this function (typically at
	 * construction of the Main Form that means to the defaultOpMode of the
	 * Application for the current user on the current PC). The Mode will be
	 * switched back after no more User activity (button or mouse pressed
	 * anywhere within the application) and an ample time out.
	 * <br><br>
	 * Details of this mechanisms can be configured with the
	 * OpModeMgr.IRubberband object that can be get via
	 * opModeMgr.getIRubberband().
	 * <br><br>
	 * Caveat: This is a best effort mechanisms. It will not be possible to
	 * enable it if the Java security manager does not allow access to the
	 * AWTEventListener. Also some strange but probably benign behavior might
	 * result from switching between ServerModes.
	 * <br><br>
	 * @param   enabled ... true best effort, ... false always succeeds
	 * @return	isRubberbandEnabled() as a result of the call
	 */
	public final boolean setRubberbandEnabled(final boolean enabled){
		if(enabled || hasIRubberBand())
			return getIRubberband().setEnabled(enabled);
		else
			return enabled;
	}
	/**not null, Zugriff auf die automatische Betriebsmode Rueckwechsel Detail Steuerung*/
	public final OpModeMgr.IRubberband getIRubberband() {
		final __OpModeRubberband rubberband = this.refRubberband.get();
		if(rubberband != null)
			return rubberband;

		this.refRubberband.set(new __OpModeRubberband(this));
		return getIRubberband();
	}
	/**@return 0 bis 100%, 100% nur bei getDestinationMode() == getOpMode()*/
	public final int getIRubberbandProgress() {
		return hasIRubberBand() ? getIRubberband().getProgress() : 100;
	}
	/**pp for testing only routine*/final boolean hasIRubberBand(){
		return null != this.refRubberband.get();
	}
//=============================================================================

	@SuppressWarnings("rawtypes")
	private final void fireOpModeObservers(final OpMode opMode) {
		//1) optionMenuItems
		final int index = opMode.index;
		final int n = (this.opModeJMenuItems == null) ? -1 : this.opModeJMenuItems.length;
		for (int i = 0; i < n; i++) {
			this.opModeJMenuItems[i].setSelected(i == index);
		}

		//2) Captions
		this.captionMgr.setText(opMode.toString());

		//3) low level OpModeObserver
		for (final Iterator iter = this.lowLevelOpModeObservers.iterator(); iter.hasNext();) {
			final IOpModeObserver observer = (IOpModeObserver) iter.next();
			if(observer != null) {observer.onOpModeChange(opMode);}
		}
	}

	/**
	 * @param object object under test
	 * @param name   name dessen was nicht null sein darf
	 * @throws NullPointerException bei object == null
	 */
	private static final void checkNotNull(final Object object, final String name) {
		if(object == null) throw new NullPointerException(name + " darf nicht null sein.");
	}

	private final void fireOpModeKonfigurationObservers(final OpModeKonfiguration opModeKonfiguration) {
		final int n = (this.opModeJMenuItems == null) ? -1 : this.opModeJMenuItems.length;
		for (int i = 0; i < n; i++) {
			final OpMode opMode = OpMode.VALUES[i];
			this.opModeJMenuItems[i].setEnabled(opModeKonfiguration.isImplemented(opMode));
			this.opModeJMenuItems[i].setText(opModeKonfiguration.getItemName(opMode));
		}
		fireOpModeObservers(getOpMode());
	}
}

