/*
 * Created on 	Mar 24, 2005 - Mar 31, 2005
 * 				Sep 12, 2006 - Sep 12, 2006	ws: added setOpMode(..., PASSWORT);
 * 				Feb 14, 2007 - ...			ws: Error Text verbessert, Passwort
 *
 */
package de.desy.mst.libs.framework.mstapp.mode;

import java.security.AccessControlException;

import de.desy.mst.libs.framework.mstapp.logging.Logger;


/**
 * triviale Implementation von ISetOpModeProtection
 * gibt immer (je nach Konstruktorwert) immer/nie AccessControlException ohne user Kommunikation zurueck
 * @author schuette
 * @version 1.0.0
 */
public final class SecurityCheck implements ISetOpModeProtection {
	private static final String SUPERGEHEIM = "007";
	private static final String GEHEIM 		= "Mcs01";
	private final OpModeListe geschuetzteOpModes;
	private final boolean neverPass;

//	=============================================================================	
		
	public final static SecurityCheck getNoCheck(){
		return (new SecurityCheck(false, OpModeListe.EMPTY_LISTE));
	}
//	=============================================================================	
		
	/**
	 * @param  geschuetzteOpModes       != BBE und enthaelt implementierten Mode
	 * @param  neverPass                == true => check immer mit Exception, sonst nie
	 * @throws IllegalArgumentException sofort fr (BBE == geschuetzteOpModes), 
	 *         							spter falls kein ungeschtzter OpMode zugnglich ist
	 */
	public SecurityCheck(final boolean neverPass, OpModeListe geschuetzteOpModes) {
		super();
		this.neverPass = neverPass;

		if(geschuetzteOpModes == null){
			geschuetzteOpModes = OpModeListe.EMPTY_LISTE;
		}else if(geschuetzteOpModes == OpModeListe.BBE){
			throw new IllegalArgumentException("PassWortForm: mindestens ein OpMode muss zugnglich sein."); 
		}
		this.geschuetzteOpModes = geschuetzteOpModes;
}

//	=============================================================================	
		
	/* (non-Javadoc)
	 * @see de.desy.mst.mstapp.security.ISetOpModeProtection#check()
	 * use check(final OpMode opMode, final String passwort) instead
	 */
	@Deprecated
	@Override public void check() throws AccessControlException {
		this.check(OpMode.EXPERT, "");
	}
	/* (non-Javadoc)
	 * @see de.desy.mst.mstapp.security.ISetOpModeProtection#check()
	 */
	@Override public void check(final OpMode opMode, final String passwort) {
		String msg = "Successful Security Check for OpMode: ";
		if(opMode == OpMode.BETRACHTEN){
			msg += "Betrachten always passes";
		}else if(SUPERGEHEIM.equals(passwort)){
			msg += "Supergeheimes Passwort always passes";
		}else if(opMode == OpMode.BETRIEB && GEHEIM.equals(passwort)){
			msg += "Betrieb und einfach geheimes Passwort";
		}else if(this.neverPass){
			throw new AccessControlException("Das Passwort ist nicht ausreichend mchtig.");
		}else{
			msg += "Passiert nur weil der Security Check abgeschaltet ist.";
		}
		Logger.SWFW.info(msg);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.mstapp.mode.ISetOpModeProtection#getGeschuetzteOpModeList()
	 */
	@Override public OpModeListe getGeschuetzteOpModeList() {
		return this.geschuetzteOpModes;
	} 
}
