/*
 * Created on 2005: Mar 31 - Apr 18, Jul 7
 * Oct 25, 2005 - ...			ws:	new logging system
 * Feb 13, 2007 - ...			ws:	Log ServerAnwahlChange auf info
 * Jun 13, 2007 - ...			ws: (de)registerAnwahlObserver pp -> public
 * Jun 19, 2007 - Jun 19, 2007	ws:	Lokalisierung auf Deutsch etwas nachgezogen
 * 									XTor inits + fire * 2
 * Dec 10, 2008 - ...			ws:	publish for tests the observerSet
 */
package de.desy.mst.libs.framework.mstapp.mode;

import javax.swing.AbstractButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JRadioButtonMenuItem;

import java.awt.event.ItemEvent;

import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.util.observables.setvalue.SetTextManager;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

/**
 * Management of Serveranwahl und der OpModeKonfigurationen.
 *
 * Bei Wechsel der Serveranwahl wird der OpModeMgr auf die alte
 * opModeKonfiguration zurueckgesetzt.
 * @link http://mstweb/PETRA3/Maass/Operation_Modes.doc
 * Zwei Zustaende: DefaultMode und SimulationsServerMode
 * @author schuette
 * @version 1.0.0
 */
@SuppressWarnings({"rawtypes"})
public final class ServerAnwahlMgr implements IRegisterForCaption {
	private ServerAnwahl anwahl = ServerAnwahl.DEFAULT;	//Objekte werden im default mode erzeugt
	private ServerAnwahlListe implAnwahlen = ServerAnwahlListe.DS;
	private final OpModeKonfiguration[] konfigurations = new OpModeKonfiguration[ServerAnwahl.VALUES.length];

	private final IOpModeKonfigurationObserver observer;

	private JMenu jMenuSrvAnwahl = null;
	private JRadioButtonMenuItem[] srvAnwahlJMenuItems = null;

	private class SrvAnwahlChangedListener implements java.awt.event.ItemListener {
		private final ServerAnwahl anwahl;
		public SrvAnwahlChangedListener(final ServerAnwahl anwahl) {
			super();
			this.anwahl = anwahl;
		}
		@Override public void itemStateChanged(final java.awt.event.ItemEvent e) {
			if(e.getStateChange() == ItemEvent.SELECTED){
				setServerAnwahl(this.anwahl);
			}
		}
	}

	//changes all registered texts to current OpMode caption
	private final SetTextManager captionMgr;
	private final Set<IServerAnwahlObserver> lowLevelAnwahlObserver = new HashSet<IServerAnwahlObserver>();
	private final Set<JComponent>            colorObserver          = new HashSet<JComponent>();
//	=============================================================================

	/**
	 * @param 	observer				!= null
	 * @param 	defaultModeKonfiguration			!= null
	 * @throws	NullPointerException, 	falls observer oder konfiguration == null
	 */
	public ServerAnwahlMgr(final IOpModeKonfigurationObserver observer, final OpModeKonfiguration defaultModeKonfiguration) {
		super();
		if(null == observer)	 {throw new NullPointerException("Der IOpModeKonfigurationObserver kann nicht null sein.");}
		if(null == defaultModeKonfiguration){throw new NullPointerException("Die OpModeKonfiguration kann nicht null sein.");}

		this.observer= observer;
		this.konfigurations[ServerAnwahl.DEFAULT.index] = defaultModeKonfiguration;
		this.konfigurations[ServerAnwahl.SIMULATIONSSERVER.index] =
			OpModeKonfiguration.makeTestKonfiguration(defaultModeKonfiguration);
		this.captionMgr  = new SetTextManager("ServerAnwahlMgr.XTor sollte nie zu sehen sein.");

		this.fireKonfigurationChange(this.getAnwahl());
		this.fireSrvAnwahlChanged(this.getAnwahl());
	}

//	=============================================================================


	public final ServerAnwahl getAnwahl() {
		return this.anwahl;
	}
	/**
	 * @param  anwahl					gewuenschte ServerAnwahl (!= null, muss implementiert sein)
	 * @throws IllegalArgumentException	gewuenschte ServerAnwahl ist nicht implementiert
	 * @throws NullPointerException	gewuenschte ServerAnwahl == null
	 */
	public final void setServerAnwahl(final ServerAnwahl anwahl) {
		if(this.anwahl == anwahl){
			return;
		}

		if(!this.hatAnwahlImplementiert(anwahl)){
			Logger.SWFW.warn("ServerAnwahl setMode(ServerAnwahl.SIMULATIONSSERVER) strange usage ignored, obwohl kein Simulationsmode vorhanden.", new Exception());
			throw new IllegalArgumentException("Die ServerAnwahl " + anwahl.name + " ist nicht implementiert.");
		}
 
		Logger.SWFW.info("ServerAnwahlMgr: Neue ServerAnwahl: " +
						 "'" + anwahl + "', alt '" + getAnwahl() + "'.");
		this.anwahl = anwahl;
		this.fireSrvAnwahlChanged(anwahl);
	}

	public boolean hatAnwahlImplementiert(final ServerAnwahl anwahl) {
		return this.getImplAnwahlen().isInList(anwahl);
	}
	public boolean hatAnwahlImplementiert(final int index) {
		return this.getImplAnwahlen().isInList(index);
	}

	/**
	 *
	 * @param 	jMenuOptions 			Hier werden die Untermenues drangehaengt
	 * @param 	anwahlObserver          werden bei ServerAnwahlAenderung oder oefter aufgerufen
	 * @throws	NullPointerException	theOptionMenu darf nicht null sein
	 * @throws	IllegalStateException	darf nur genau einmal aufgerufen werden, menuIndex must be legal
	 */
	public final void initAndRegisterServerAnwahlMenu(final JMenu jMenuOptions, final int i, final IServerAnwahlObserver anwahlObserver) {
		//proctection against bad input
		final String msgPrefix = "ServerAnwahlMgr.initAndRegisterServerAnwahlMenu(theOptionMenu): ";
		if(jMenuOptions == null){
			final NullPointerException e = new NullPointerException(msgPrefix + "theOptionMenu may not be null");
			Logger.SWFW.error(e);
			throw e;
		}
		if(this.jMenuSrvAnwahl != null){
			final IllegalStateException e = new IllegalStateException(msgPrefix + "was called more than once.");
			Logger.SWFW.error(e);
			throw e;
		}

		//real bussiness
		this.jMenuSrvAnwahl  = new JMenu("Server Anwahl");
		jMenuOptions.add(this.jMenuSrvAnwahl);
		this.registerForCaption(this.jMenuSrvAnwahl);

		this.srvAnwahlJMenuItems  = new JRadioButtonMenuItem[this.konfigurations.length];
		initAndRegisterJMenuItem(ServerAnwahl.DEFAULT);
		initAndRegisterJMenuItem(ServerAnwahl.SIMULATIONSSERVER);
		fireImplChanged();

		this.registerAnwahlObserver(anwahlObserver);
		this.fireSrvAnwahlChanged(this.getAnwahl());
	}

	/**
	 *
	 */
	private void fireImplChanged() {
		if(this.srvAnwahlJMenuItems == null){return;}	//nichts zu tun
		for (int i = 0; i < this.srvAnwahlJMenuItems.length; i++) {
			this.srvAnwahlJMenuItems[i].setEnabled(this.hatAnwahlImplementiert(i));
		}
	}

	//Button, Menu, ... -text will be kept in synch with current OpMode
	@Override public final void registerForCaption(final AbstractButton menuItemOrButtonOrEtc){
		this.captionMgr.add(menuItemOrButtonOrEtc);
	}
	//label Text will be kept in synch with current opmode
	@Override public final void registerForCaption(final JLabel label){
		this.captionMgr.add(label);
	}

	@Override public final void deregisterForCaption(final Object object) {
		this.captionMgr.remove(object);
	}

	public final ServerAnwahlListe getImplAnwahlen() {
		return this.implAnwahlen;	//ok, da immutable
	}
	/**
	 * @param implAnwahlen 				ServerAnwahlListe mit implementierten ServerAnwahlen,
	 * 									!= null, ServerAnwahlListe.EMPTY_LISTE
	 * @throws NullPointerException		falls implAnwahlen == null
	 * @throws IllegalArgumentException		falls implAnwahlen == ServerAnwahlListe.EMPTY_LISTE
	 */
	public final void setImplAnwahlen(final ServerAnwahlListe implAnwahlen) {
		if(implAnwahlen == null){ throw new NullPointerException("implAnwahlen muss nichttrivial sein");}
		if(implAnwahlen == ServerAnwahlListe.EMPTY_LISTE ){
			throw new IllegalArgumentException("implAnwahlen muss mindestens eine gueltige Anwahl haben");
			}
		if(this.implAnwahlen != implAnwahlen){
			this.implAnwahlen = implAnwahlen;
			if(!implAnwahlen.isInList(this.getAnwahl())){
				this.setServerAnwahl(implAnwahlen.getDefaultServerAnwahl());
			}
			fireImplChanged();
		}
	}

	/**
	 * @param opModeListe 					OpModeListe mit implementierten OpModes,
	 * 										!= null, OpModeListe
	 * @throws NullPointerException			falls opModeListe == null
	 * @throws IllegalArgumentException		falls opModeListe == EMPTY_LISTE, oder nicht Passwortfrei erreichbar
	 */
	public void setImplOpModes(final OpModeListe opModeListe) {
		if(opModeListe == null){ throw new NullPointerException("Implementierte OpModeListe muss nichttrivial sein");}
		if(opModeListe == OpModeListe.EMPTY_LISTE){
			throw new IllegalArgumentException("Implementierte OpModeListe muss mindestens einen gueltigen OpMode haben");
			}
		if(opModeListe != this.getImplOpModeListe()){
			for (int i = 0; i < this.konfigurations.length; i++) {
				final OpModeKonfiguration opModeKonfiguration = this.konfigurations[i];
				this.konfigurations[i] = opModeKonfiguration.createKonfigurationForImplOpModes(opModeListe);
			}
			this.fireKonfigurationChange(this.getAnwahl());
		}
	}


	public void registerForServerAnwahlColor(final JComponent component) {
		this.colorObserver.add(component);
	}

	public void deregisterForServerAnwahlColor(final JComponent component) {
		this.colorObserver.remove(component);
	}

//=============================================================================

	public boolean registerAnwahlObserver(final IServerAnwahlObserver anwahlObserver) {
		if(anwahlObserver != null) {	//null silently ignored
			final boolean didExist = this.lowLevelAnwahlObserver.add(anwahlObserver);
			anwahlObserver.onServerAnwahlChange(this.getAnwahl());
			return didExist;
		}
		else{
			return false;
		}
	}
	public boolean deregisterAnwahlObserver(final IServerAnwahlObserver anwahlObserver) {
		return this.lowLevelAnwahlObserver.remove(anwahlObserver);
	}

//=============================================================================

	private void initAndRegisterJMenuItem(final ServerAnwahl anwahl) {
		this.srvAnwahlJMenuItems[anwahl.index] = new JRadioButtonMenuItem(anwahl.name);
		this.srvAnwahlJMenuItems[anwahl.index].addItemListener(new SrvAnwahlChangedListener(anwahl));
		this.jMenuSrvAnwahl.add(this.srvAnwahlJMenuItems[anwahl.index] );
	}

//	@SuppressWarnings("unchecked")
	private void fireSrvAnwahlChanged(final ServerAnwahl anwahl) {
		//1) Auswahlmenue
		if(this.srvAnwahlJMenuItems != null){
			for (int i = 0, index = anwahl.index; i < this.srvAnwahlJMenuItems.length; i++) {
				this.srvAnwahlJMenuItems[i].setSelected( i == index);
			}
		}
		//2) low level AnwahlObserver
		for (final Iterator iter = this.lowLevelAnwahlObserver.iterator(); iter.hasNext();) {
			final IServerAnwahlObserver observer = (IServerAnwahlObserver) iter.next();
			if(observer != null) {observer.onServerAnwahlChange(anwahl);}
		}

		//3) Captions
		this.captionMgr.setText(anwahl.toString());

		//4) Fore/Backcolor
		for (final Iterator iter = this.colorObserver.iterator(); iter.hasNext();) {
			final JComponent component = (JComponent) iter.next();
			if(component != null) {
				component.setBackground(anwahl.backColor);
				component.setForeground(anwahl.foreColor);
			}
		}

		//5) Konfigurationsobserver, da verschiedene Serveranwahlen verschiedene konfigs haben
		fireKonfigurationChange(anwahl);
	}

	private void fireKonfigurationChange(final ServerAnwahl anwahl) {
		this.observer.onKonfigurationChange(this.konfigurations[anwahl.index]);
	}

	/**
	 * @return Liste der Implementierten OpModes, ist fuer alle Konfigurationen identisch
	 */
	private OpModeListe getImplOpModeListe() {
		return this.konfigurations[0].getImplementierteModes();
	}

	/**pp for test only*/IOpModeKonfigurationObserver getAnwahlObserver() {
		return this.observer;
	}
}
