/**
 * 	Oct  2, 2009 - Oct 20, 2009	ws:	Creation from scratch
 * 	Nov 10, 2010 - Nov 10, 2010	ws:	fixed Pjotr Problem: in onOpModeChange(): compare on OpMode.index <= statt ==
 */

package de.desy.mst.libs.framework.mstapp.mode;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.atomic.AtomicReference;

import javax.swing.SwingUtilities;

import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.util.gui.SessionActivityMonitor;

/**
 * package private Hilfsklasse, um den OpMode bei UI Unaktivitaet nach einer
 * gewissen Zeit automatisch auf den destinationMode (zurueck) zusetzen
 * @author ws
 * @version 1.0.0
 */
class __OpModeRubberband implements OpModeMgr.IRubberband{
	private static final int DEFAULT_MAX_IDLE_TIME_MS = 5*60*1000;	//5 min
//=============================================================================
	private final SessionActivityMonitor sam;	//might be null
	private final OpModeMgr opModeMgr;
	private final AtomicReference<OpMode> refDestinationMode = new AtomicReference<OpMode>(OpMode.BETRACHTEN);
	private final AtomicInteger refMaxIdleTimeMS = new AtomicInteger(DEFAULT_MAX_IDLE_TIME_MS);
	private final AtomicBoolean refIsEnabled = new AtomicBoolean(false);
//=============================================================================
	/**@throws NullPointerException if opModeMgr is null*/
	__OpModeRubberband(final OpModeMgr opModeMgr) {
		this.sam = createSAM();
		this.opModeMgr = opModeMgr;
		setDestinationMode(getOpModeMgr().getOpMode());	//default Destination is current opMode, by conctruction valid
		checkNotNull(opModeMgr);

		//(1) never deregistered, since fixed pair for app Lifetime if rubberband is instantiated
		//(2) use inner class and not __OpModeRubberband itself to keep onOpModeChange(..) private
		getOpModeMgr().registerIOpModeObserver(new IOpModeObserver(){
			@Override public void onOpModeChange(final OpMode opMode) {
				if(SwingUtilities.isEventDispatchThread())
					__OpModeRubberband.this.onOpModeChange();
				else
					SwingUtilities.invokeLater(new Runnable(){
						@Override public void run() {
							__OpModeRubberband.this.onOpModeChange();
						}
					});
			}
		});
	}
//=============================================================================


	@Override public final OpMode getDestinationMode() {return this.refDestinationMode.get();}
	@Override public final OpMode setDestinationMode(final OpMode destinationMode) {
		if(isPublic(destinationMode))
			this.refDestinationMode.set(destinationMode);
		else if(getDestinationMode() == null)
			this.refDestinationMode.set(getOpModeMgr().getOpMode());
		return getDestinationMode();
	}

	@Override public final int getMaxIdleTimeMS(){return this.refMaxIdleTimeMS.get();}
	@Override public final int setMaxIdleTimeMS(final int maxIdleTime){
		if(isActive()){
			deactivate();
			setMaxIdleTimeMS(maxIdleTime);
			activate();
		} else {
			this.refMaxIdleTimeMS.set(maxIdleTime);
		}
		return getMaxIdleTimeMS();
	}

	@Override public final boolean isPossibleToEnable(){return getSam() != null;}
	@Override public final boolean isEnabled()		 {return this.refIsEnabled.get();}
	@Override public final boolean setEnabled(final boolean enabled){
		if(isEnabled() != enabled){
			this.refIsEnabled.set(enabled);
			if(!enabled)
				deactivate();
			else
				onOpModeChange();	//check for opModeChange and possibly activate
		}
		return isEnabled();
	}
	@Override public final boolean isActive(){return isPossibleToEnable() && getSam().isRunning();}

	@Override public int getMinRestTimeMS(){
		if(isActive())
			return getSam().getMinRestTimeMS();
		else
			return Integer.MAX_VALUE;
	}
	@Override public int getProgress() {
		final double maxIdleTimeMS = getMaxIdleTimeMS();
		return (isActive() && maxIdleTimeMS > 0) ? getProgress(maxIdleTimeMS) : 100;
	}

//=============================================================================
	/**privat, da nur durch opMode change Events gesteuert*/
	private final boolean deactivate(){
		//Logger.SWFW.info("Deactivate the Rubberband. (Old state isActive() == " + isActive() + ")");
		if(isActive())
			getSam().stop();
		return isActive();
	}
	/**privat, da nur durch opMode change Events gesteuert*/
	private final boolean activate(){
		//Logger.SWFW.info("Activate the Rubberband. (Old state isActive() == " + isActive() + ")");
		if(!isPossibleToEnable())
			return isActive();

		getSam().setMaxIdleTimeMs(getMaxIdleTimeMS());
		return isActive();
	}
	/**pp for testing only*/final OpModeMgr getOpModeMgr() {return this.opModeMgr;}

	/**@throws IllegalArgumentException if destinationMode not implemented or public (d.h. passwortfrei)*/
	private final void setOpModeToDestination() {getOpModeMgr().setOpMode(getDestinationMode(), "");}
	/**might be null (i.e. !isPossibleToEnable())*/
	private final SessionActivityMonitor getSam() {return this.sam;}
	/**nur via start setzen wegen Konsistenz des start*/
	private final OpModeKonfiguration getOpModeKonfiguration() {
		return getOpModeMgr().getOpModeKonfiguration();
	}
	private void checkNotNull(final Object o) {
		if(o == null)
			throw new NullPointerException();
	}
	private boolean isPublic(final OpMode mode) {
		return getOpModeKonfiguration().getPublicModes()
									   .isInList(mode);
	}
	private final SessionActivityMonitor createSAM(){
		try {
			final SessionActivityMonitor sam = new SessionActivityMonitor();
			sam.addActionListener(new ActionListener(){
				@Override public void actionPerformed(final ActionEvent evt) {
					setOpModeToDestination();
				}
			});
			return sam;
		}
		catch (final Exception ex) {
			Logger.SWFW.warn("_OpModeRubberband kann den SessionActivityMonitor nicht erzeugen. Kein automatisches Ruecksetzen der OpModes moeglich.", ex);
			return null;
		}
	}
	private void onOpModeChange() {
		if(!isEnabled())
			return;

		if(needRubberBand())
			activate();	//falls active wird die Restzeit auf maxDelay aufgefrischt
		else
			deactivate();	
	}


	private boolean needRubberBand(){
		final OpMode currentOpMode = getOpModeMgr().getOpMode();
		final OpMode destOpMode    = getDestinationMode();
		return needRubberBand(currentOpMode, destOpMode);
	}
	/**pp*/ static boolean needRubberBand(final OpMode currentOpMode, final OpMode destOpMode) {
		return destOpMode.index < currentOpMode.index;	//current OpMode ist gefaehrlicher 
	}
	
	private int getProgress(final double maxIdleTimeMS) {
		final int progress = boundItFrom0To100(100. - (100. * getMinRestTimeMS()) / maxIdleTimeMS);
		if(progress == 100 && isActive())
			return 99;	//nie 100% solange noch aktiv
		return progress;
	}
//	private int dumpGetProgress(final double progressRaw
//							  , final int progress
//							  , final boolean active
//							  , final int maxIdleTimeMS
//							  , final int minRestTimeMS) {
//		System.out.println();
//		System.out.println("dumpGetProgress START ++++++++++++");
//		System.out.println("       active = " + active);
//		System.out.println("  progressRaw = " + progressRaw);
//		System.out.println("     progress = " + progress);
//		System.out.println("maxIdleTimeMS = " + maxIdleTimeMS);
//		System.out.println("minRestTimeMS = " + minRestTimeMS);
//		System.out.println("dumpGetProgress END   ++++++++++++\n");
//		return progress;
//	}
	/**Prozente gehen nur von 0 bis 100*/
	private int boundItFrom0To100(final double progress) {
		if(progress < 0)
			return 0;
		if(progress > 100 || Double.isNaN(progress))
			return 100;
		return (int) progress;
	}
}
