/**
 * @author wst
 * Nov  9, 2006 - Nov  9, 2006	ws:	added PrefStorage
 * Sep 24, 2008 - Oct  7, 2008	ws: guard register, deregister somewhat
 * Dec  1, 2008 - Dec  2, 2008	ws:	center PrefFrame to guessed MstFrameMain
 * Jun 24, 2009 - Jun 24, 2009	ws: handle onNewSaveString(...) Exception
 */
package de.desy.mst.libs.framework.mstapp.preferences.implementation;

import java.util.ArrayList;
import java.util.List;

import de.desy.mst.libs.framework.mstapp.Main;
import de.desy.mst.libs.framework.mstapp.MstApp;
import de.desy.mst.libs.framework.mstapp.ini.PrefStorage;
import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.preferences.IPrefManager;
import de.desy.mst.libs.framework.mstapp.preferences.IPreferences;

public class PrefManager implements IPrefManager{
	private final MstApp mstApp;
	private final PrefStorage storage;
	private final List<IPreferences> iPreferences;
	private final PrefGeneral root;

	private PrefFrame preferencesFrame = null;
//=============================================================================
	public PrefManager(final MstApp mstApp, final PrefStorage prefStorage) {
		super();
		this.mstApp = mstApp;
		this.storage = prefStorage;

		this.iPreferences = new ArrayList<IPreferences>();
		this.root = new PrefGeneral(this);

		this.iPreferences.add(this.root);
	}
	//=============================================================================
	@Override public void register(final IPreferences uI){
//		System.out.println(this.getClass().getSimpleName() 
//		        + ".register(\"" + uI.getTitle() + "\").");
		if(uI == null || this.iPreferences.contains(uI))
			return;
		this.iPreferences.add(uI);
		if(this.storage.hasPrefString(uI)){
			final String prefString = this.storage.getPrefString(uI);
			try {uI.onNewSaveString(prefString);
			} catch (final Exception ex) {
				final String msg = "Problem mit Laden der gespeicherten"
								 + " Einstellungen von " + uI
								 + " prefString = '" + prefString + "'.";
				Logger.SWApp.error(msg, ex);
			}
		}
		if(this.preferencesFrame != null)	//avoid frame creation in getter
			getPreferencesFrame().add(uI);
	}
	@Override public void deRegister(final IPreferences uI){
		if(uI == null || !this.iPreferences.contains(uI))
			return;
		this.iPreferences.remove(uI);
		if(this.preferencesFrame != null)	//avoid frame creation in getter
			getPreferencesFrame().remove(uI);
	}
 
	@Override public void showPreferencesFrame(){showPreferencesFrame(null);}
	@Override public void showPreferencesFrame(final IPreferences preferences) {
		final PrefFrame frame = getPreferencesFrame();
		frame.setPrefSelection(preferences == null ? getRoot() : preferences);
		if(!frame.isVisible())
			Main.centerToMstFrameMain(frame);//Sichtbaren PrefFrame nicht verschieben
		frame.setVisible(true);
	}

	public PrefFrame getPreferencesFrame() {
		if(this.preferencesFrame == null)
			this.preferencesFrame = new PrefFrame(this);
		return this.preferencesFrame;
	}
	public  final List<IPreferences> getIPreferences()	{return this.iPreferences;}
	public  final PrefStorage 		 getStorage() 		{return this.storage;}
	//=============================================================================
	/**pp*/ final MstApp 			 getMstApp() 		{return this.mstApp;}
	/**pp*/ final PrefGeneral 		 getRoot() 			{return this.root;}
	//=============================================================================
}
