/**
 *  Created on  Aug 28, 2006
 * 	Jul 14, 2009 - Jul 14, 2009	ws:	fixed some Eclipse 3.5 warnings
 */
package de.desy.mst.libs.framework.mstapp.tabellendaten;

import java.util.Arrays;

/**
 * Multi Column Schluesselspalten werden auf einen Schluessel eineindeutig abgebildet<br>
 * Die Objekte sind immutable und koennen somit gefahrlos als Schlussel in Collections verwendet werden.<br>
 * Eigentlich ist die Klasse ein Composite Comparable.<br>
 * Immutable Class, lexikalisch sortiert mit Value orientierter Semantik
 *  
 * @author ws
 *
 */
@SuppressWarnings("rawtypes") public class CompositeKey implements Comparable {
	private final Comparable [] keys;	//nie den Array rausgeben, da er mutable ist
	
//=============================================================================
	
	/**keys muessen immatable sein (also String, Integer, ...<br>
	 * @throws NullPointerException, 		falls keine keys
	 * @throws IllegalArgumentException,	falls keys.length < 1 (O dimensionaler Array)
	 */
	public CompositeKey(final Comparable ... keys) {
		super();
		if(keys.length < 1) {throw new IllegalArgumentException("Es braucht mindestens einen key.");} 
		this.keys = keys.clone();	//nicht original array, keys sollten immutable sein
	}
	
//=============================================================================
		
	/**
	 * @throws ClassCastException,   			if other is not of type CompositeKey
	 * @throws ArrayIndexOutOfBoundsException, 	if other is not of type CompositeKey of same nSchluessel
	 */
	@Override
	public int compareTo(final Object other) {
		return this.compareTo((CompositeKey)other);
	}
	public int compareTo(final CompositeKey other) {
		final int n = Math.min(this.getNSchluessel(), other.getNSchluessel());
		for (int i = 0; i < n; i++) {
			final int partialResult = componentCompareTo(other, i);
			if(partialResult == 0) continue;
			return partialResult;
		}
		return (Integer.valueOf(this.getNSchluessel()))
					.compareTo(Integer.valueOf(other.getNSchluessel()));
	}

	@SuppressWarnings("unchecked")
	private int componentCompareTo(final CompositeKey other, final int i) {
		return this.keys[i].compareTo(other.keys[i]);
	}
 
	/**Anzahl der Schluessel*/
	public final int getNSchluessel() 					{return this.keys.length;}
	/**@throws ArrayIndexOutOfBoundsException*/
	public final Comparable getKey(final int index)	{return getKeys()[index];}
	/**Kopie der Schluessel*/
	public final Comparable[] getKeys() 			{return this.keys.clone();}

	
	
	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override public boolean equals(final Object obj) {
		if(obj instanceof CompositeKey){
			return Arrays.equals(this.keys, ((CompositeKey) obj).keys);
		}
		return false;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override public int hashCode()		{return Arrays.hashCode(this.keys);}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override public String toString()	{return "CompositeKey: " + Arrays.asList(this.keys).toString();}
}
