/**
 *  Created on 	Aug 8, 2005 - Sep 07, 2005
 * 				Oct 25, 2005 - ...	new logging system
 * 	Jul 14, 2015 - Jul 14, 2015	ws:	added equals(...), hashCode() for JUnit tests
 */
package de.desy.mst.libs.framework.mstapp.tabellendaten.entry;

import java.util.Arrays;

import de.desy.mst.libs.framework.mstapp.logging.Logger;

/**
 * Ein Tabelleneintrag mit sortierbarer Schluesselspalte Key K.<br>
 * Voraussetzung: 
 * <li>K ist eindeutig fuer die Anwendung.
 * <li>K ist sortierbar (automatisch erfuellt fuer String, Integer, ...)<br>
 * Option:<li>
 * eigene Sortierung via Ueberschreiben von 'public int compareTo(AEntry<K> entry)'
 * 
 * @author ws
 * @version 1.0.0
 */
public abstract class AEntry<K extends Comparable<K>> implements Comparable<AEntry<K>> {
	/**ein applikationsweiter eindeutiger Schluessel wie nr oder devname ...*/
	abstract public K getKey();
	abstract public boolean inSeinenWertenGleich(AEntry<K> otherEntry);
	
	private Object[] unsupportedExtraParams = null;

//=============================================================================	

	@Override public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + Arrays.hashCode(this.unsupportedExtraParams);
		return result;
	}
	@Override public boolean equals(final Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final AEntry<?> other = (AEntry<?>) obj;
		if (!Arrays.equals(this.unsupportedExtraParams, other.unsupportedExtraParams))
			return false;
		return true;
	}
	//=============================================================================	
	
	/**
	 * Default Implementation des Sortierkriteriums der Entries ist 
	 * das Sortierkriterium des Entry Schluessels
	 * @throws NullPointerException, if entry == null
	 */
	@Override public int compareTo(final AEntry<K> entry) {
		return getKey().compareTo(entry.getKey());
	}
	
	public void handleUnsupportedExtraParams(final AEntry<K> source, final Object[] unsupportedExtraParams){
		this.unsupportedExtraParams = unsupportedExtraParams;
		Logger.SWFW.warn(source.getClass().getName()
				                 + " got unsupported Parameters:\n" + Arrays.toString(unsupportedExtraParams) 
								  );
	}
	
	/**Do not change, unchecked*/
	public final Object[] getUnsupportedExtraParams()	{return this.unsupportedExtraParams;}
	public final boolean hasUnsupportedExtraParams()	{return getUnsupportedExtraParams() != null;}
}
