/**
 *  Created on  Sep  7, 2005 - Sep 19, 2005
 *  			Dec 13, 2005 - Dec 15, 2005, dgNr = 0 is explicitly a bad entry
 *  			Feb 08, 2006 - Feb 20, 2006, added Trim Support 
 *  			Apr 10, 2006 - Apr 10, 2006, changed devTrimXML0 -> devTrimXML1
 *  			May 26, 2006 - May 29, 2006, added D2Lage
 *  			Jun 21, 2006 - Jun 22, 2006	ws: added getMaschine (provisorische Implementation)
 *  			Jul 14, 2006 - Jul 14, 2006 ws: added Temperatur Devices
 *  			Jul 20, 2006 - Jul 25, 2006	ws: changed devTrimXML1 -> devTrimXML2 usw. (new PACKAGES)
 *  			Aug 25, 2006 - Aug 25, 2006 ws: added D2Seki
 */
package de.desy.mst.libs.framework.mstapp.tabellendaten.entry;

import java.util.ArrayList;
import java.util.List;

import de.desy.mst.libs.framework.mstapp.mode.Maschine;

/**
 * Hilfsklasse fuer echte AEntry Device und Unterklassen.
 * Devicegruppen sind hart kodiert.
 * Bei neuen Devicegruppen muss die Source angepasst werden 
 * (einfach und so praktischer, da als EntwicklungsZeitKonzept gebraucht).
 * Bitte danach DevicegruppenTest erfolgreich laufen lassen.
 * 
 * @author ws
 * @version 1.0.0
 */
public enum DeviceGruppe {
	  ZZ_BAD_ENTRY  ("BAD ENTRY check for dgNr = 0 init error", false)	//ILLEGAL BAD ENTRY
												//dgNr	anlage	bez		subBez		werNr	basisTabelle			basisXMLAbfrage			deviceTyp	aufgNr	iniFile	bem			added
	, PeKlimaFuehler("devPeKlimaFuehlerXML2")	//1		PE		Klima	Fuehler		29		devPeKlimaFuehlerVerz	devPeKlimaFuehlerXML0	SEDAC		9		-		-			2005.09.07
	, PeMag         ("devPeMagXML2")			//2		PE		Mag		-			42		devPeMag				devPeMagXML0			SEDAC		2		-		-			2005.09.19
	, PeKlimaAnlage ("devPeKlimaAnlageXML2")	//3		PE		Klima	Anlage		29		devPeKlimaAnlageVerz	devPeKlimaAnlageXML0	SEDAC		9		-		-			2005.09.06  
	, D2Trim 		("devTrimXML3+4")			//4		D2		Trim	-			29		devTrimVerz				devTrimXML2				SEDAC		0		-		-			2006-02-07
	, L2Trim 		("devTrimXML3+5")			//5		L2		Trim	-			29		devTrimVerz				devTrimXML2				SEDAC		0		-		-			2006-02-07
	, PeTrim 		("devTrimXML3+6")			//6		Pe		Trim	-			29		devTrimVerz				devTrimXML2				SEDAC		0		-		-			2006-02-07
	, DoTrim 		("devTrimXML3+7")			//7		Do		Trim	-			29		devTrimVerz				devTrimXML2				SEDAC		0		-		-			2006-02-07
	/**D3Trim: NICHT UNTERSTUETZT ABER VORGESEHEN*/
	, D3Trim 		("devTrimXML3+8", false)	//8		D3		Trim	-			29		devTrimVerz				devTrimXML2				SEDAC		0		-		Reserve, 3 Paare mit doppelten Sedac Adressen zusammengefasst		2006-02-07
	, D2Lage 		("devLageXML2+9")			//9		D2		Lage	-			42		devLageVerz				devLageXML0				SEDAC		0		-		DESY2 Lage/Orbit Original \\ie1\acc:  \MCA\DB\D2ORBIT\INITLADE.DBF	2006.05.26
	, L2TempAllg	("devTempXML2+10")			//10	L2		Temp	allgemein	37		devTemp					devTempXML0				SEDAC		0		-		-			2006-07-14
	, L2TempSLED	("devTempXML2+11")			//11	L2		Temp	SLED		37		devTemp					devTempXML0				SEDAC		0		-		-			2006-07-14
	, L2TempAbschnitt("devTempXML2+12")			//12	L2		Temp	Abschnitt	37		devTemp					devTempXML0				SEDAC		0		-		-			2006-07-14
	, D2Seki		("devSekiXML1+13")			//13	D2		Seki	-			45		devSeki					devSekiXML0				SEDAC		0		-		braucht sekiDetailsBitVerz, sekiDetailsMpxVerz						2006-08-24	
	;

//=============================================================================
	public final String  tabId;
	public final boolean isLegalDeviceGruppe;
//=============================================================================
	private DeviceGruppe(final String tabId, final boolean isLegalDeviceGruppe) {
		this.tabId				 = tabId;
		this.isLegalDeviceGruppe = isLegalDeviceGruppe;
	}
	private DeviceGruppe(final String tabId){this(tabId, true);}
//=============================================================================
	
	/**@throws ArrayIndexOutOfBoundsException, bei entsprechender dgNr*/
	public final static DeviceGruppe getByDgNr(final int dgNr){return values()[dgNr];}
	
	public final int     getDevGrNr()         {return ordinal();}
	
	/**
	 * @return 	die GERATENE (per Konvention Name faengt mit Maschine.name() an) Maschine 
	 * 			spaeter vielleicht aus Tabelle
	 */
	public Maschine getMaschine() {
		//IDEE use DeviceGruppenVerz.Anlage aus DB um die Maschine zu bestimmen
		final String name = this.name();
		for (final Maschine maschine : Maschine.values()) {
			final boolean nameFaengtMitMaschinenNamenAn 
					= name.toUpperCase().startsWith(maschine.name().toUpperCase());
			if(nameFaengtMitMaschinenNamenAn){
				return maschine;
			}
		}
		return Maschine.NA;
	}
	/**
	 * FRAMEWORK USE ONLY<br>
	 * Idee: besserer Default fuer enable'te Maschinen im MaschinenMgr
	 * @return 	alle Devises mit gleicher BasisTabId tun ihre geratene Maschine in die liste
	 */
	public List<Maschine> getMaschinenGleicherBasisXML() {
		final String basisTabId = this.getBasisTabId();
		final List<Maschine> maschinen = new ArrayList<Maschine>();
		for (final DeviceGruppe deviceGruppe: DeviceGruppe.values()) {
			if(!deviceGruppe.isLegalDeviceGruppe && deviceGruppe != this) continue; //nur legale fremde DeviceGruppen
			final String refBasisTabId = deviceGruppe.getBasisTabId();
			if(basisTabId.equals(refBasisTabId)){
				final Maschine neueMaschine = deviceGruppe.getMaschine();
				maschinen.add(neueMaschine);
			}
		}
		return maschinen;
	}
	
//=============================================================================
	
	/**
	 * @return TabId Links von "+"
	 */ 
	private String getBasisTabId() { 
		final String[] teile = this.tabId.split("\\+");
		final String basisTabId = teile[0];
		return basisTabId;
	}
	//IDEE mit Tabellenabfrage hinterfuettern fuer Alle ZusatzInfos	+ tabId
}
