/**
 * Jun 23, 2008 - Jun 24, 2008	ws:	Creation from scratch
 * May  5, 2009 - May  5, 2009	ws:	MSIntervallToText
 */
package de.desy.mst.libs.framework.mstapp.util;

import java.util.Calendar;
import java.util.Date;

/**
 * Date Utilities
 * @author ws
 * @version 1.0.0
 */
public class MstDate {
	public static final long SECOND_IN_MS = 1000L;
	public static final long MINUTE_IN_MS = 60L * SECOND_IN_MS;
	public static final long   HOUR_IN_MS = 60L * MINUTE_IN_MS;
	public static final long    DAY_IN_MS = 24L * HOUR_IN_MS;

	/**Morgens 0 Uhr nach lokaler Zeit, tage == 0 ... heute, 7 ... eine Woche*/
	public static Date abrundenAufTage(final int tage, final Date in) {
		final Calendar inCal = Calendar.getInstance();
		inCal.setTime(in);

		final Calendar outCal = Calendar.getInstance();
		outCal.clear();
		final int year  = inCal.get(Calendar.YEAR );
		final int month = inCal.get(Calendar.MONTH);
		final int date  = inCal.get(Calendar.DAY_OF_MONTH );
		outCal.set(year, month, date);

		outCal.add(Calendar.DAY_OF_YEAR, -tage);
		final Date out = outCal.getTime();

		return out;
	}
	/**a time interval in ms as Text like "0 Tage 00:00:05"*/
	public static String msIntervalToText(final long msIntervall) {
		if(msIntervall < 0)
			return "vor " + msIntervalToText(-msIntervall);

		long msRest = msIntervall;
		
		final long nDays = extractFull(msRest, DAY_IN_MS);
		msRest = updateRest(msRest, DAY_IN_MS, nDays);
		
		final long nHours = extractFull(msRest, HOUR_IN_MS);
		msRest = updateRest(msRest, HOUR_IN_MS, nHours);
		
		final long nMinutes = extractFull(msRest, MINUTE_IN_MS);
		msRest = updateRest(msRest, MINUTE_IN_MS, nMinutes);
		
		final long nSeconds = extractFull(msRest, SECOND_IN_MS);
		msRest = updateRest(msRest, SECOND_IN_MS, nSeconds);
		
		final long nMS = msRest;
		
		final StringBuilder sb = new StringBuilder();
		sb.append(nDays).append(" Tage ")
		  .append(to2DigitString(nHours  )).append(":")
		  .append(to2DigitString(nMinutes)).append(":")
		  .append(to2DigitString((nMS > 500L) ? nSeconds + 1 : nSeconds))
		  ;
		return sb.toString();
	}
	private static long extractFull(final long msIntervall, final long timeUnit) {
		return (long) (Math.floor(((double)msIntervall)/timeUnit));
	}
	private static long updateRest(long msRest, final long timeUnit, final long nUnits) {
		return msRest - nUnits * timeUnit;
	}
	private static String to2DigitString(final long l){
		if(l > 9 || l < 0)
			return "" + l;
		else
			return "0" + l;
	}
}
