/**
 * 	Aug  6, 2009 - Aug  6, 2009	ws:	Eclipse asked for removing unnessary casts, so I did
 */
package de.desy.mst.libs.framework.mstapp.util.applicationresizemanager.managers;

import java.awt.geom.*;
import javax.swing.*;
import de.desy.mst.libs.framework.mstapp.MstFrameMain;
import de.desy.mst.libs.framework.mstapp.util.applicationresizemanager.listeners.*;
import de.desy.mst.libs.framework.mstapp.util.applicationresizemanager.resizers.*;


/**
 * 
 * 
 * The resize manager allows to resize graphical applications containing Swing components.
 * 
 * @see <a href = "HowToUse.html">How-to-Use Instructions</a>
 * 
 * @author R.Bacher
 * @version 3.0.0
 * 
 */
public class ApplicationResizeManager {
	
	private static final int NOZOOM = 0;
	private static final int ZOOM = 1;
	private static final int ZOOMMAXONLY = 2;
	
	private int myZoomMode = NOZOOM;
	
	private JFrame myFrame = null;
	private double myFrameWidth = 0;
	private double myFrameHeight = 0;
	//private double myRefFrameWidth = 0;
	//private double myRefFrameHeight = 0;
	
	private ComponentResizer[] myComponentResizer = null;
	private JPanel myContentPane = null;
	private JMenuBar myMenuBar = null;
	private JPopupMenu myContentPanePopupMenu = null;
	private JPopupMenu myMenuBarPopupMenu = null;
	private JPopupMenu myMstAppPopupMenu = null;
	
	/**
	 * 
	 * Default constructor; only expanding allowed
	 * 
	 * @param frame Jframe to be resized 
	 */
	public ApplicationResizeManager(final JFrame frame) {
		
		this.myFrame = frame;
		init();
		addListeners();
		analyzeContainer();
		setResizeModeToZoomMaxOnly();

	}
	
	
	/**
	 * 
	 * Constructor
	 * 
	 * @param frame JFrame to be resized
	 * @param zoomMode zoommode (ZOOM, NOZOOM, ZOOMMAXONLY)
	 * @param xScale initial horizontal scaling
	 * @param yScale initial vertical scaling
	 */
	public ApplicationResizeManager(final JFrame frame, final String zoomMode, final double xScale, final double yScale) {
		
		this.myFrame = frame;
		init();
		addListeners();
		analyzeContainer();
		if (zoomMode == "ZOOM") {this.setResizeModeToZoom();}
		if (zoomMode == "NOZOOM") {this.setResizeModeToNoZoom();}
		if (zoomMode == "ZOOMMAXONLY") {this.setResizeModeToZoomMaxOnly();}
		this.setReferenceSize(xScale, yScale);
	}
	
	/**
	 * 
	 * Constructor
	 * 
	 * @param frame JFrame to be resized
	 * @param zoomMode zoommode (ZOOM, NOZOOM, ZOOMMAXONLY)
	 * @param xScale initial horizontal scaling
	 * @param yScale initial vertical scaling
	 */
	public ApplicationResizeManager(final JFrame frame, final EZoomModes zoomMode, final double xScale, final double yScale) {
		
		this.myFrame = frame;
		init();
		addListeners();
		analyzeContainer();
		switch (zoomMode) {
			case ZOOM: this.setResizeModeToZoom(); break;
			case NOZOOM: this.setResizeModeToNoZoom(); break;
			case ZOOMMAXONLY: this.setResizeModeToZoomMaxOnly(); break;
		}
		this.setReferenceSize(xScale, yScale);
	}
	
	/**
	 * Determines the graphical start-up parameters 
	 * and instantiates the ComponentResizer objects 
	 * for the JContentPane and the JMenuBar, respectively.
	 * 	
	 */
	private void init() {
		
		this.myFrameWidth = this.myFrame.getWidth();
		this.myFrameHeight = this.myFrame.getHeight();
		//myRefFrameWidth = myFrameWidth;
		//myRefFrameHeight = myFrameHeight;

		this.myComponentResizer = new ComponentResizer[5];
		 
		try {this.myContentPane = ((JPanel)((JComponent)(this.myFrame.getContentPane())));} catch (final Exception e) {}
		if (this.myContentPane != null) {this.myComponentResizer[0] = new ComponentResizer(this.myFrame, (this.myContentPane), this);}
		
		try {this.myMenuBar = (this.myFrame.getJMenuBar());} catch (final Exception e) {}
		if (this.myMenuBar!= null) {this.myComponentResizer[1] = new ComponentResizer(this.myFrame, (this.myMenuBar), this);}
		
		try {this.myContentPanePopupMenu = ((JComponent)(this.myFrame.getContentPane())).getComponentPopupMenu();} catch (final Exception e) {}
		if (this.myContentPanePopupMenu != null) {this.myComponentResizer[2] = new ComponentResizer(this.myFrame, (this.myContentPanePopupMenu), this);}
		
		try {this.myMenuBarPopupMenu = ((JComponent)(this.myFrame.getJMenuBar())).getComponentPopupMenu();} catch (final Exception e) {}
		if (this.myMenuBarPopupMenu != null) {this.myComponentResizer[3] = new ComponentResizer(this.myFrame, (this.myMenuBarPopupMenu), this);}
		
		try {this.myMstAppPopupMenu = (JPopupMenu)(((MstFrameMain)(this.myFrame)).getPopUpMenuMgr());} catch (final Exception e) {}
		if (this.myMstAppPopupMenu != null) {this.myComponentResizer[4] = new ComponentResizer(this.myFrame, (this.myMstAppPopupMenu), this);}

	}
	
	/**
	 * 
	 * Adds the listener objects to the frame 
	 *
	 */
	private void addListeners() {
			
		new ApplicationResizeListener(this.myFrame, this);	
		if (this.myContentPane != null) {new ComponentResizeListener(this.myFrame, this.myComponentResizer[0]);}
		if (this.myMenuBar != null) {new ComponentResizeListener(this.myFrame, this.myComponentResizer[1]);}
		if (this.myContentPanePopupMenu != null) {new ComponentResizeListener(this.myFrame, this.myComponentResizer[2]);}
		if (this.myMenuBarPopupMenu != null) {new ComponentResizeListener(this.myFrame, this.myComponentResizer[3]);}
	}
	

	
	/**
	 * 
	 * Adds the manager objects to analyze the next graphical level
	 *
	 */
	private void analyzeContainer() {

		if (this.myContentPane != null) {new ContainerManager(this, this.myFrame, (this.myContentPane));}
		if (this.myMenuBar != null) {new ContainerManager(this, this.myFrame, (this.myMenuBar));}
		if (this.myContentPanePopupMenu != null) {new ContainerManager(this, this.myFrame, (this.myContentPanePopupMenu));}
		if (this.myMenuBarPopupMenu != null) {new ContainerManager(this, this.myFrame, (this.myMenuBarPopupMenu));}
	}  
	
	/**
	 * Sets the resize mode of the visual application frame to "ZOOM".
	 * 	
	 */
	public void setResizeModeToZoom() {
		
		this.myZoomMode = ZOOM;
		this.myFrame.setResizable(true);
	}
	
	/**
	 * Sets the resize mode of the visual application frame to "NOZOOM".
	 * 	
	 */
	public void setResizeModeToNoZoom() {
		
		this.myZoomMode = NOZOOM;
		this.myFrame.setResizable(false);
	}
	
	/**
	 * Sets the resize mode of the visual application frame to "ZOOM" but allows only magnification of the application frame.
	 * 	
	 */
	public void setResizeModeToZoomMaxOnly() {
		
		this.myZoomMode = ZOOMMAXONLY;
		this.myFrame.setResizable(true);
	}
	
	/**
	 * Sets the reference size of the visual application frame with respect to the design size of the application.
	 * 
	 * @param xScale initial horizontal scaling factor
	 * @param yScale initial vertical scaling factor
	 * 	
	 */
	public void setReferenceSize(final double xScale, final double yScale) {
	
		if (this.myZoomMode != NOZOOM) {		
			//$ANALYSIS-IGNORE
			this.myFrame.setSize((int)Math.round(this.myFrameWidth * xScale), (int)Math.round(this.myFrameHeight * yScale));
		}
	}	
	
	/**
	 * Controls the resizing of visual application frame.	
	 * 	
	 */
	public void resizeApplication() {

		if (this.myZoomMode != NOZOOM) {			
			if (this.myZoomMode == ZOOMMAXONLY) {
				//if ((myFrame.getWidth() < myRefFrameWidth) || (myFrame.getHeight() < myRefFrameHeight)) {
					//myFrame.setSize((int)Math.round(myRefFrameWidth), (int)Math.round(myRefFrameHeight)); 	
				if ((this.myFrame.getWidth() < this.myFrameWidth) || (this.myFrame.getHeight() < this.myFrameHeight)) {
					//$ANALYSIS-IGNORE
					this.myFrame.setSize((int)Math.round(this.myFrameWidth), (int)Math.round(this.myFrameHeight)); 	
				}
			}
		}
	}
	
	/**
	 * Returns the affine scale transformation which has to be used in the paint method of a graphic object 
	 * 
	 * 	@return affine scale transformation
	 * 
	 */
	public AffineTransform getScaleTransformation() {
		return(AffineTransform.getScaleInstance(getScaleX(), getScaleY()));
	}
	
	/**
	 * Returns the x scaling value.	
	 * 	
	 * @return horizontal scaling value
	 * 
	 */
	public double getScaleX() {
		double xScale = 0.0;

     	//xScale = myFrame.getWidth() / myRefFrameWidth;
    	xScale = this.myFrame.getWidth() / this.myFrameWidth;

		return xScale;
	}
	
	/**
	 * Returns the y scaling value.	
	 * 	
	 * @return vertical scaling value
	 * 
	 */
	public double getScaleY() {
		double yScale = 0.0;
		
		//yScale = myFrame.getHeight() / myRefFrameHeight;
		yScale = this.myFrame.getHeight() / this.myFrameHeight;
		
		return yScale;
	}
}
