/**
 * ws:	Oct 31, 2007 - Oct 31, 2007 creation from scratch
 * ws:	Nov  8, 2007 - Nov  9, 2007 statische Member _* fuer Default Farben
 * ws:	Jan  7, 2007 - Jan  7, 2007 update to Steve's Farben
 * ws:	Mar 25, 2007 - Mar 25, 2007 neu: set(...) May 8: set(Component)
 * ws:	Jan 15, 2008 - Jan 15, 2008 Typ now serializable
 * kh:	Jul 02, 2014 - Jul 02, 2014 _SELECTED lighter blue nearer to the color windows uses
 */
package de.desy.mst.libs.framework.mstapp.util.gui;

import java.awt.Color;
import java.awt.Component;
import java.io.Serializable;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

/**
 * Application Color (Background)<br>
 * Adds following features:
 * <li> Color from Status farbNr
 * <li> foreGroundColor
 * <li> name (der Farbe wie "Rot" oder symbolisch wie "Aus"
 *
 *
 * @author ws
 * @version 1.0.0
 */
public class AppColor extends Color {
	static public enum Typ implements Serializable{
		  BASIS_FARBE
		, SYMBOLISCH
		, BENUTZER;
	};

	private static final long serialVersionUID = 1L;
	private static final Map<String, AppColor> vordefinierteFarben;
	public final static AppColor RED;
	public final static AppColor YELLOW;
	public final static AppColor GREEN;
	public final static AppColor WHITE;
	public final static AppColor BLACK;
	public final static AppColor GRAY;
	public final static AppColor BLUE;
	public final static AppColor LIGHT_BLUE;
	public final static AppColor SELECTED_BLUE;

	//Spezial FARBEN wie in folgendem Dokument beschrieben:
	//http://adweb.desy.de/mst/petra_III_kontrollsystem/Maass/JavaApplikationen.doc
	// bzw wie von Steve Herb Ende 2007 zusammengetragen:
	//http://ttfinfo.desy.de/MCSelog/show.jsp?dir=/doc/GUI&pos=2007-11-16T11:37:36


	//STEVE FARBEN:
	//=============

	/**Default: Gerte ist eingeschaltet und in Ordnung.*/
	public final static AppColor _EIN_UND_OK;

	/**Default: Gert ist in fehlerhaftem Zustand,
	 * kann in diesem Zustand nicht betrieben werden.*/
	public final static AppColor _FEHLER;

	/**Default: Gert luft, aber gewisse Bedingungen sind nicht OK.*/
	public final static AppColor _WARNUNG;

	/**Default: Gerte ist ausgeschaltet und in Ordnung oder hier
	 * lediglich nur unwichtige Stoerung.*/
	public final static AppColor _AUS;

	/**Default: Text Feld mit Daten oder Information*/
	public final static AppColor _Info;

	/**Default: Service Request*/
	public final static AppColor _Service_Req;

	/**Default: URL Reference ohne Unterlinie*/
	public final static AppColor _URL_REF;

	//WEITERE FARBEN:
	//===============

	/**Default: Gert ist nicht in definiertem oder eindeutigem Zustand
	 * ODER Zustand kann nicht ermittelt werden.
	 * <br>
	 * Vorsicht bei Steve diese FarbKombi "_AUS", _UNDEFINIERT existiert nicht.
	 * */
	public final static AppColor _UNDEFINIERT;

	/**Ausgewhlte Controls
	 * <br>
	 * Vorsicht bei Steve diese FarbKombi "_Service_Req",
	 * _SELECTED existiert nicht.
	 */
	public final static AppColor _SELECTED;
	/**Titelzeile von Tabellen
	 * Vorsicht bei Steve diese FarbKombi "_URL_REF",
	 * _TABLEHEADER existiert nicht.
	 */
	public final static AppColor _TABLEHEADER;

//=============================================================================
	static{
		vordefinierteFarben = new HashMap<String, AppColor>();

		//Standard AppColor's zuerst
		RED 		= create("Rot",     Color.RED,   Color.WHITE);
		YELLOW 		= create("Gelb",    Color.YELLOW,Color.BLACK);
		GREEN	 	= create("Grn",    Color.GREEN, Color.BLACK);

		WHITE 		= create("Wei",    Color.WHITE, Color.BLACK);
		BLACK 		= create("Schwarz", Color.BLACK, Color.WHITE);
		GRAY 		= create("Grau",    Color.GRAY,  Color.WHITE);

		BLUE		= create("Blau",	Color.BLUE,	 Color.WHITE);
		LIGHT_BLUE	= create("Hellblau",  new Color(216, 228, 248),Color.BLACK);
		SELECTED_BLUE	= create("Selection",new Color(102, 153, 255),Color.WHITE);

		//Danach Spezial AppColor's
		//(1) Steve
		_EIN_UND_OK	= create("Ein und OK",		AppColor.GREEN);//normally not off and ok
		_FEHLER 	= create("Fehler",			AppColor.RED);
		_WARNUNG 	= create("Warnung",			AppColor.YELLOW);
		_AUS		= create("Aus und OK",		AppColor.GRAY);
		_Info		= create("Information Data",AppColor.WHITE);
		_Service_Req= create("Service Request",	AppColor.BLUE);
		_URL_REF	= create("URL Reference",	AppColor.LIGHT_BLUE);


		//(2) Weitere
		_UNDEFINIERT= create("Undefiniert",	AppColor.GRAY);

		_SELECTED   = create("Ausgewhlt",	AppColor.SELECTED_BLUE);
		_TABLEHEADER= create("TabTitel",	AppColor.LIGHT_BLUE);
}
	/**pp for debugging*/
	static Color guessForeGround(final Color backGround) {
		final int rot   = backGround.getRed();
		final int gruen = backGround.getGreen() ;
		final int blau  = backGround.getBlue();

		//ROT
		if(gruen == 0 && blau == 0 && rot > 0){
			return (rot >= 100) ?
								Color.WHITE
								:
								Color.BLACK;
		}

		//WEISS - GRAU -SCHWARZ
		if(gruen == blau && blau == rot){
			return (rot <= 128) ?
								Color.WHITE
								:
								Color.BLACK;
		}

		//helle Farben sollen schwarz haben haben
		final int helligkeit = (rot + gruen + blau);
		final boolean istDunkel = (helligkeit <= 128);
		return istDunkel ?
						 Color.WHITE
						 :
						 Color.BLACK;
	}
	/**privat da es vordefinierte Farben aendert*/
	private static AppColor create(final Typ typ, final String name, final Color backGround, final Color foreGround){
		final AppColor appColor = new AppColor(typ, name, backGround, foreGround);
		vordefinierteFarben.put(name, appColor);
		return appColor;
	}
	/**privat da es vordefinierte Farben aendert*/
	private static AppColor create(final String name, final AppColor theAppColor) {
		return create(Typ.SYMBOLISCH, name, theAppColor.getBackGround(), theAppColor.getForeGround());
	}
	/**privat da es vordefinierte Farben aendert*/
	private static AppColor create(final String name, final Color backGround, final Color foreGround){
		return create(Typ.BASIS_FARBE, name, backGround, foreGround);
	}
//=============================================================================
	/**@return unmodifiableMap der vordefinierten Farben mit name als key*/
	public static final Map<String, AppColor> getVordefinierteFarben(){
		return Collections.unmodifiableMap(AppColor.vordefinierteFarben);
	}
	/**
	 * @param color not null
	 * @return versucht eine AppColor.Typ.BASIS_FARBE Farbe zu finden und den
	 * 		   zugehoerigen Namen zu liefern, default color.toString()
	 */
	public static  String calcBaseName(final Color color) {
		final Collection<AppColor> appColors = AppColor.getVordefinierteFarben().values();
		final int    rgb  = color.getRGB();
		for (final AppColor appColor : appColors) {
			if(appColor.getRGB() == rgb && appColor.getTyp() == AppColor.Typ.BASIS_FARBE){
				return appColor.getName();
			}
		}
		return color.toString();
	}
//=============================================================================
	final private Color foreGround;
	final private String name;
	final private Typ typ;
//=============================================================================
	/**
	 * @param name	Name der Farbe oder des Zweckes, z.B. "Rot" oder "Aus", not null
	 * @param backGround not null
	 * @param foreGround kann null sein
	 * @throws NullPointerException falls irgendein Farbe null ist.
	 */
	public AppColor(final String name, final Color backGround, final Color foreGround){
		this(Typ.BENUTZER, name, backGround, foreGround);
	}
	private AppColor(final Typ typ, final String name, final Color backGround, final Color foreGround){
		super(backGround.getRGB());
		if(name == null){
			throw new NullPointerException("The name must not be null.");
		}
		this.typ = typ;
		this.name = name;
		this.foreGround = (foreGround == null)
						? guessForeGround(backGround)
						: foreGround;
	}
//public AppColor(int rgb) 							{super(rgb);}
//public AppColor(int rgba, boolean hasalpha) 		{super(rgba, hasalpha);}
//public AppColor(int r, int g, int b) 				{super(r, g, b);}
//public AppColor(float r, float g, float b) 		{super(r, g, b);}
//public AppColor(ColorSpace cspace, float[] components, float alpha) {
//	super(cspace, components, alpha);
//}
//public AppColor(int r, int g, int b, int a) 		{super(r, g, b, a);}
//public AppColor(float r, float g, float b, float a)	{super(r, g, b, a);}
//=============================================================================
	/**@return the Typ, never null*/
	public final Typ getTyp() {return this.typ;} 
	/**@return the foreGround, never null*/
	public final Color getForeGround() {return this.foreGround;}
	/**@return the backGround, never null*/
	public final Color getBackGround() {return this;}
	/**@return the name, never null*/
	public final String getName() {return this.name;}
	/**setzt fore und backGround, null Argument ignoriert*/
	public void set(final Component component) {
		if(component == null)
			return;
		component.setForeground(getForeGround());
		component.setBackground(getBackGround());
	}
}
