/**
 * ws:	Feb 14, 2007 - Feb 14, 2007 creation from scratch
 * ws:	Apr 19, 2007 - ...			added comments, createCheckBoxWithTooltip()
 * ws:	May 09, 2007 - ...			added createHeaderFont()
 *	Feb 15, 2008 - Feb 15, 2008 ws: added "@Override"es
 *	Mar 19, 2008 - Mar 19, 2008 ws: added createTitledBorder(Title)
 *	May 22, 2008 - May 22, 2008 ws: added setCursor
 * 	Jun 25, 2008 - Jun 25, 2008	ws:	added createImageButton(...) (Jul 7)
 * 	Sep  5, 2008 - Sep  5, 2008	ws:	added createFixedWidthFont
 *	Feb 26, 2009 - Feb 26, 2009 ws: added createFont(Font, Style)
 *  Jul 17, 2009 - Jul 17, 2009 ws:	added createAndShowLongActionDialog, 
 *  									  runInExtraThread_showingBusyDialog
 */
package de.desy.mst.libs.framework.mstapp.util.gui;

import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionListener;
import java.net.URL;
import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JToolTip;
import javax.swing.border.TitledBorder;

import de.desy.mst.libs.framework.mstapp.logging.Logger;

/**
 * Factory Class zur Erzeugung von (einheitlichen) Gui Widgets<br>
 * Historie: Rausfaktoriert von Wst Preferences.
 * @author ws
 * version 1.0.0
 */
public class GUIFactory {
	public static final String FONTFAMILY = "Dialog";
	public static final String FONTFAMILY_MONOSPACED = "Monospaced";
	public static final int BASE_FONT_SIZE = 12;
//=============================================================================
	public GUIFactory() {
		final String msg = "THE 'GUIFactory' IS A PURE STATIC CLASS: "
						 + "CAN NOT INSTANCIATE OBJECTS.";
		throw new IllegalStateException(msg);
	}
//=============================================================================
	
	/**
	 * @param runnable, not null (will be executed in new Thread) runnable should
	 * 					finish only after all other relevant and timeconsuming 
	 * 					Threads spawned within are finished
	 * @param refCmp	may be null (for centering the dialog Box)
	 * @param title		displayed in dialog
	 * @param modal		dialog is on Top and other app GUI is disabled for User
	 * @return the executing thread of runnable (in case you want to do a join)
	 */
	public static Thread runInExtraThread_showingBusyDialog(
													  final Runnable runnable
													, final Component refCmp
													, final String title
													, final boolean modal){
		final __LongActionIndication lai = new __LongActionIndication();
		return lai.runInExtraThread_showingBusyDialog(runnable, refCmp, title, modal);
	}
	/**
	 * Creates a visible alive dialog centered around frame (if non null) with
	 * title title<br>
	 * ALWAYS CALL dialog.dispose() AT THE END!!!!
	 * @param 	owner 	may be null (zum Zentrieren des dialogs)
	 * @param 	title	wird zur Orientierung des Nutzers angezeigt
	 * @param 	modal	if true then rest of the Application UI is blocked
	 * @return	the busy dialog, YOU MUST call dispose() on it, when your are finisched with it
	 */
	public static JDialog createAndShowLongActionDialog(final JFrame owner
													  , final String title
													  , final boolean modal){
		return new __LongActionIndication().show(owner, title, modal);
	}
	
	/**@return ButtonFont im wst Design*/
	public static Font createButtonFont(){
		return new Font(FONTFAMILY, Font.PLAIN, BASE_FONT_SIZE);
	}
	public static Font createHeaderFont() {
		return new Font(FONTFAMILY, Font.BOLD, BASE_FONT_SIZE + 2);
	}
	public static Font createFixedWidthFont() {
		return new Font(FONTFAMILY_MONOSPACED, Font.PLAIN, BASE_FONT_SIZE);
	}
	/**same Font with a different Font."style" like Bold ....*/
	public static Font createFont(final Font font, final int style) {
		return new Font(font.getName(), style, font.getSize());
	}

	/**
	 * @param fileName	relativ zu src/resources zB "filter/editFilter24.gif"
	 * @param tooltip	beliebiger Text als Tooltip
	 * @param listener	action beim Pressen des buttons
	 * @param classFromJar	damit das image auch geladen werden kann
	 * @return			fertig konfigurierter button
	 */
	public static JButton createImageButton (final String fileName
										   , final String tooltip
										   , final ActionListener listener
										   , final Class<?> classFromJar
										   ) {
		final JButton button = new JButton();
		button.setFont(createButtonFont());		//just in case
		ImageIcon imageIcon  = createImageIcon(fileName, tooltip, classFromJar);
		if(imageIcon != null){
			button.setIcon(imageIcon);
			final int iWidth  = imageIcon.getIconWidth();
			final int iHeight = imageIcon.getIconHeight();
			final Dimension size = new Dimension(iWidth, iHeight);
			button.setPreferredSize(size);
			button.setMinimumSize(size);
			button.setMaximumSize(size);
		}
		button.setToolTipText(tooltip);
		button.addActionListener(listener);
		return button;
	}
	/**
	 * @param fileName		relativ zu src/resources zB "filter/editFilter24.gif"
	 * @param toolTip		beliebiger Text als Tooltip
	 * @param classFromJar	damit das image auch geladen werden kann
	 * @return				Image Icon mit ToolTip
	 */
	public static ImageIcon createImageIcon(final String fileName, final String toolTip, final Class<?> classFromJar) {
		final String resourceLocation = "/resources/" + fileName;
		final Class<?> clazz = (classFromJar == null) ? GUIFactory.class
													  : classFromJar;
		final URL resource = clazz.getResource(resourceLocation);
		if(resource == null){
			Logger.SWFW.warn( "GUIFactory.createImageIcon kann das Image"
							+ " nicht laden von: '" + resourceLocation + "'.");
			return null;
		}else{
//			Logger.SWFW.debug("GUIFactory.createImageIcon ldt '"
//							+ resourceLocation + "'.");
			return new ImageIcon(resource, toolTip);
		}
	}
	/**
	 * @param fileName	relativ zu src/resources zB "filter/editFilter24.gif"
	 * @param classFromJar	damit das image auch geladen werden kann
	 * @return			Image Icon ohne ToolTip
	 */
	public static ImageIcon createImageIcon(final String fileName, final Class<?> classFromJar) {
		return createImageIcon(fileName, "", classFromJar);
	}

	/**@return CheckBox mit ToolTip im WST Design*/
	public static JCheckBox createCheckBoxWithTooltip() {
		return new JCheckBox(){
			private static final long serialVersionUID = 1L;
			@Override public JToolTip createToolTip(){
				JToolTip tip = super.createToolTip();
				tip.setBackground(new Color(255,255,200));
				tip.setForeground(Color.black);
				return tip;
			}
		};
	}
	public static TitledBorder createTitledBorder(final String text) {
		return BorderFactory.createTitledBorder(
				  null
				, text
				, TitledBorder.DEFAULT_JUSTIFICATION
				, TitledBorder.DEFAULT_POSITION
				, new Font(FONTFAMILY, Font.BOLD, 12)
				, new Color(51, 51, 51)
												);
	}

	/**
	 * Setzt den Cursor zum hoechsten erreichbaren Container
	 * @param newCursor new cursor
	 * @return	old Cursor
	 */
	public static Cursor setCursor(final Cursor newCursor, final Component cmp){
		final Component topParentCmp = getTopParentCmp(cmp);
		final Cursor oldCursor = topParentCmp.getCursor();
		topParentCmp.setCursor(newCursor);
		return oldCursor;
	}
	/**berechnet den obersten GUI Container von cmp (not null)*/
	public static Component getTopParentCmp(final Component cmp){
		final Container parent = cmp.getParent();
		return (parent == null) ? cmp : getTopParentCmp(parent);
	}
}
