/*
 * Created on Feb 16-17, 2005
 * 	Oct 16, 2006 - 	Oct 16, 2006	ws:	removed Guilleaumes (-> Markus Walla) 
 *
 */
package de.desy.mst.libs.framework.mstapp.util.observables.setvalue;

import java.awt.Component;

/**
 * Typesafe observable for objects that have a 'setVisible(boolean)' method
 *  
 * 'setVisible(String)' is the call back function/event
 * 
 * Support for any component like 	Button, Canvas, Checkbox, Choice, Container,
 * 									Label, List, Scrollbar, TextComponent
 *									is already implemented.
 *  
 * If you need more kinds of objects i.e. classes
 * 1) provide a SPECIFIC (not for any Object) method add(NewClass newClass),
 * 2) provide a testcase in SetVisibleManagerTest
 * AND 
 * 3) run the test successfully
 *
 * Implemented with the non typesafe setvalue class
 * 
 * @author schuette
 */

public class SetVisibleManager extends SetValueManager {

	/**
	 * @param startValue wether visible or not
	 */
	public SetVisibleManager(boolean startValue) {
		super(new Boolean(startValue), "setVisible");
		this.forceParameterTypes(new Class[] { boolean.class }); //need the class of native type not its corresponding class
	}

	/**
	 * @param startValue	all registered clients will be receive a method call 
	 * 						to setVisible(startValue)
	 */
	public void setVisible(boolean startValue) {
		this.setValue(new Boolean(startValue)); //no autoboxing in 1.4 for native types
	}

	/**
	 * @param 	component 	a component for automated setVisible(boolean) like
	 * 						Button, Canvas, Checkbox, Choice, Container, Label,
	 * 						List, Scrollbar, TextComponent
	 */
	public void add(Component component) {
		this.addObject(component);
	}

}
