/**
 *	Feb  3, 2010 - Feb 12, 2010	ws:	creation from scratch
 *	Apr 12, 2010 - Apr 12, 2010	ws:	APP_INFO_MQ_SERVER nonfinal for testpurposes
 *									getMsgSender() -> getMsgSenderFactory() 
 *	Jul  6, 2010 - Jul  6, 2010	ws:	removed pcName
 *	Aug  5, 2010 - Aug  5, 2010	ws:	support for JMSExcQueue
 *	Aug 16, 2010 - Aug 16, 2010	ws:	logging ohne stack trace
 * 	Aug 29, 2011 - Aug 29, 2011	ws:	created from scratch
 */

package de.desy.mst.libs.info.appadmin;

import java.util.UUID;
import java.util.logging.Level;

import javax.jms.MessageListener;

/**
 * <b>Applikation administration support context.</b> 
 * <br>At the moment only statistics.
 * 
 * <p>see also: ___AppAdmin___package_info.txt in this package
 * 
 * @author ws
 * @version 1.0.0
 */
public final class AppAdminContext {
	final class _MstSenderDummy implements _IMsgSender {
		public void send(final _Msg msg) {/*ignore*/}
	}
	private static final long DEFAULT_MAX_CONSOLE_LOGGINGS = 100L;
	/**pp*/ static final int  DEFAULT_QUEUE_CAPACITY = 500;
	private static /*f*/ String APP_INFO_MQ_SERVER  = "mq://ttf2svr3.desy.de:7676/jms";//final except for extreme test
	private static final String APP_INFO_MQ_SERVERK = "imqAddressList";
	private static final String APP_INFO_TEST_TOPIC_NAME = "AppInfoTestQueue";
	private static final String APP_INFO_TOPIC_NAME 	 = "AppInfoQueue";
	private static final String APP_INFO_TEST_EXCQUEUE_NAME = "AppInfoExcTestQueue";	//for exceptions
	private static final String APP_INFO_EXCQUEUE_NAME 	 = "AppInfoExcQueue";			//for exceptions
	private static final AppAdminContext instance = new AppAdminContext();
	private static volatile boolean appAdminIsInitialized = false;
	private static volatile boolean isOffline = !_HostInfo.getInstance().isDesyIPAddress();	//DESY Addresses per default online other offline
	//-------------------------------------------------------------------------
	public  static final AppAdminContext getInstance(){return AppAdminContext.instance;}
	/**some parameter changes might have no effect if is appAdminIsInitialized true*/
	public  static boolean appAdminIsInitialized()    {return AppAdminContext.appAdminIsInitialized;}
	/**pp*/ static void    setAppAdminIsInitialized() {AppAdminContext.appAdminIsInitialized = true;}
//=============================================================================
	private final UUID   runId;
	private volatile long maxConsoleLoggings = AppAdminContext.DEFAULT_MAX_CONSOLE_LOGGINGS;
	private volatile int queueCapacity = AppAdminContext.DEFAULT_QUEUE_CAPACITY;
	private volatile _IMsgSender msgSender;
	private volatile boolean logEveryMessage = false;
	private volatile String  jmsTopicName = APP_INFO_TOPIC_NAME;
	private volatile String  jmsExcQueueName = APP_INFO_EXCQUEUE_NAME;
//-----------------------------------------------------------------------------
	private AppAdminContext(final UUID runId, final int statisticQueueCapacity) {
		this.runId  = (runId  == null)                     ? UUID.randomUUID() : runId;
	}
	private AppAdminContext() {this(null, -1)/*default context*/;}
//-----------------------------------------------------------------------------
	/**
	 * default == false
	 * state must be {@link #appAdminIsInitialized()} == false for different isOffline
	 */
	public static void setOffline(final boolean isOffline) {
		if(isOffline == isOffline())
			return;	//nothing to do
		
		if(appAdminIsInitialized()){
			final String msg = "Can only setOffline before using AppAdmin class";
			_Logging.log(Level.WARNING, msg, new IllegalStateException(msg));
			return;
		}
		
		AppAdminContext.isOffline = isOffline;
		if(isOffline())
			_Logging.log(Level.INFO, "With respect to JMS: offline.", null);
	}
	public static boolean isOffline(){return isOffline ;}
	
	/**Unique Id for every run of the program*/
	public final UUID   getRunId()		{return this.runId;}
	
	/**if changed too late might not reflect the used capacity*/
	public final long getMaxConsoleLoggings() {return this.maxConsoleLoggings;}
	/**only an effect before loading the AppAdmin class and if capacity > 0*/
	public final void setMaxConsoleLoggings(final int maxConsoleLoggings) {
		if(maxConsoleLoggings > 0)
			this.maxConsoleLoggings = maxConsoleLoggings;
	}
	
	/**if changed too late might not reflect the used capacity*/
	public final int getQueueCapacity() {return this.queueCapacity;}
	/**only an effect before loading the AppAdmin class and if capacity > 0*/
	public final void setQueueCapacity(final int capacity) {
		if(capacity > 0)
			this.queueCapacity = capacity;
	}
	
	public final _IMsgSenderFactory getMsgSenderFactory() {
		return new _IMsgSenderFactory() {
			@SuppressWarnings("synthetic-access") @Override 
			public _IMsgSender create() {
				synchronized (AppAdminContext.this) {//only one to be created
					if(AppAdminContext.this.msgSender == null)
						setMsgSender(createDefaultMsgSender());
					return AppAdminContext.this.msgSender;
				}
			}
		};
	}
	public final void setMsgSender(final _IMsgSender msgSender) {
		if(msgSender != null)
			this.msgSender = msgSender;
	}
	
	/**every fired message is logged when added to the _Queue queue*/
	public final boolean logEveryMessage() {return this.logEveryMessage;}
	public final void setLogEveryMessage(final boolean logEveryMessage){this.logEveryMessage = logEveryMessage;}
	
	public final String getJMSTopicName()     {return this.jmsTopicName;}
	public final String getJMSTestTopicName() {return AppAdminContext.APP_INFO_TEST_TOPIC_NAME;}
	public final void   setJMSTopicName(final String jmsTopicName) {if(jmsTopicName != null) this.jmsTopicName = jmsTopicName;}
	
	public final String getJMSExcQueueName()     {return this.jmsExcQueueName;}
	public final String getJMSExcTestQueueName() {return AppAdminContext.APP_INFO_TEST_EXCQUEUE_NAME;}
	public final void   setJMSExcQueueName(final String jmsExcQueueName) {if(jmsExcQueueName != null) this.jmsExcQueueName = jmsExcQueueName;}

	public final String getJMSMQ_URL() 		{return AppAdminContext.APP_INFO_MQ_SERVER;}
	public final String getJMSMQ_URL_KEY()	{return AppAdminContext.APP_INFO_MQ_SERVERK;}
//-----------------------------------------------------------------------------
	final _MsgReaderJMSTopic createDefaultReaderJMSTopic(final MessageListener msgListener, final String filter) {
		if(isOffline())
			return null;
		
		try{return new _MsgReaderJMSTopic(getJMSTopicName(), msgListener, filter);}
		catch(final Throwable th){//imq.jar may not be present
			final String msg = "Can not create a JMS Connection. "
					   + "Messages can not be read. Original message: "
					   + th.getMessage();
			_Logging.log(Level.SEVERE, msg, th);
			return null;}
	}
//-----------------------------------------------------------------------------
	private final _IMsgSender createDefaultMsgSender() {
		if(isOffline())
			return new _MstSenderDummy();
		
		try{return new _MsgSenderJMS(getJMSTopicName());}
		catch(final Throwable th){//imq.jar may not be present
			final String msg = "Can not create a JMS Connection. "
							 + "Messages will be ignored. Original message: "
							 + th.getMessage();
			_Logging.log(Level.SEVERE, msg, null);
			return new _MstSenderDummy();}
	}
}
