/**
 *	Mar 11, 2010 - Mar 12, 2010	ws:	moved to Top Level class from _Msg
 *	Jul  9, 2010 - Jul 14, 2010	ws:	added	equalByValue(..., ...)
 *										 	_ExternalSystemEntry
 *											getJMSFilter()
 * 	Aug 5, 2010 - Aug 5, 2010	ws:	added	_ExceptionEntry
 */

package de.desy.mst.libs.info.appadmin;

import java.util.UUID;
import java.util.concurrent.atomic.AtomicLong;

/**
 * Keyed and Typed entry for _Msg. Immutable class. Subclasses should obey
 * to this contract.
 * @author ws
 * @version 1.0.0
 * @param <V>
 */
abstract class _Entry<V>{
	public static boolean equalByValue(final _Entry<?> lhs, final _Entry<?> rhs){
		return lhs.getKey  ().equals(rhs.getKey  ())
		   &&  lhs.getValue().equals(rhs.getValue());
	}
	//=====================================================================
	private final String   key;
	private final V        value;
	private final Class<V> classType;
	//---------------------------------------------------------------------
	_Entry(final String key, final V value, final Class<V> classType) {
		this.key = key;
		this.value = value;
		this.classType = classType;
	}
	//---------------------------------------------------------------------
	@Override public String toString() {
		final StringBuilder sb = new StringBuilder();
		sb.append("_Msg.Entry<")
		  .append(getClassType() == null ? "null" : getClassType().getSimpleName())
		  .append(">(")
		  .append(getKey())
		  .append(", ")
		  .append(getValue())
		  .append(")")
		;
		return sb.toString();
	}
	@Override public int hashCode() {
		final int prime = 31; int result = 1;
		result = prime * result + ((this.key   == null) ? 0 : this.key  .hashCode());
		return   prime * result + ((this.value == null) ? 0 : this.value.hashCode());
	}
	@Override public boolean equals(final Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		
		final _Entry<?> other = (_Entry<?>) obj;
		if (this.key == null) {
			if (other.key != null)
				return false;
		} else if (!this.key.equals(other.key))
			return false;
		
		if (this.value == null) {
			if (other.value != null)
				return false;
		} else if (!this.value.equals(other.value))
			return false;
		
		return true;
	}
	final String 	getKey() 		{return this.key;}
	final V 		getValue() 		{return this.value;}
	final Class<V> 	getClassType()	{return this.classType;}
	/**JMS Filter String Element. Current entry is passed.*/
	final String getFilterExcludeNulls(){return "(" + getFilter() + " AND NOT " + includeNulls() + ")";}
	final String getFilterIncludeNulls(){return "(" + getFilter() + " OR "      + includeNulls() + ")";}
	private final String getFilter()    {return "(" + this.key + " = " + this.value + ")";}
	private final String includeNulls() {return "(" + this.key + " IS NULL)";}
}

//=============================================================================

/**Convenience Subclass*/
abstract class _StringEntry extends _Entry<String>{
	public  static final String KEY_HOSTADDRESS  = "hostAddress";
	public  static final String KEY_HOSTNAME     = "hostName";
	public  static final String KEY_COMPUTERUSER = "computerUser";

	_StringEntry(final String key, final String value) {super(key, value, String.class);}
	public String getJMSFilter(){return getKey() + " = '" + getValue() + "'";}
} 
/**Convenience Subclass*/
abstract class _LongEntry extends _Entry<Long>{
	_LongEntry(final String key, final Long value) {super(key, value, Long.class);}
}

//=============================================================================

final class _TestEntry extends _Entry<Boolean>{
	static final _TestEntry TE_TRUE  = new _TestEntry(true );
	static final _TestEntry TE_FALSE = new _TestEntry(false);
	public static _TestEntry get(final boolean isTestMsg) {
		return isTestMsg ? _TestEntry.TE_TRUE: _TestEntry.TE_FALSE;
	}
	
	static final String _KEY = "isTest";	
	private _TestEntry(final Boolean value) {super(_KEY, value, Boolean.class);}
}

//=============================================================================

final class _SystemEntry extends _StringEntry{
	static final _SystemEntry LIFECYCLE_STATISTIC = new _SystemEntry("LifeCycleStatistic");
	static final _SystemEntry UPDATE_NOTIFICATION = new _SystemEntry("UpdateNotification");
	
	static final String _KEY = "AppAdminSystem";	
	private _SystemEntry(final String systemName) {super(_KEY, systemName);}
} 

final class _ExternalSystemEntry extends _StringEntry{
	static final _ExternalSystemEntry FROM_JMS_MSG = new _ExternalSystemEntry("FromJmsMessage");
	
	static final String _KEY = "ExternalSystem";	
	private _ExternalSystemEntry(final String systemName) {super(_KEY, systemName);}
} 

//=============================================================================

/** Automatic generation of long numbers starting with one*/ 			
final class _NrGeneratorEntry extends _LongEntry{
	private static final AtomicLong refCount = new AtomicLong(0L);
	//-------------------------------------------------------------------------
	static _NrGeneratorEntry create() {
		return new _NrGeneratorEntry(refCount.incrementAndGet());
	}
	static final String _KEY = "Nr";	
	@Deprecated _NrGeneratorEntry(final Long nextNr){super(_KEY, nextNr);}
}

/**holds the time in ms since 1970, default is currenttime*/
final class _TimeEntry extends _LongEntry{
	static _TimeEntry create() {return new _TimeEntry();}
	
	static final String _KEY = "Time";	
	_TimeEntry(final Long timeMS){super(_KEY, timeMS);}
	_TimeEntry(){super(_KEY, System.currentTimeMillis());}
}

final class _RunIDEntry extends _StringEntry{
	static final _RunIDEntry RUNID_ENTRY = new _RunIDEntry(UUID.randomUUID().toString());

	static final String _KEY = "RunId";	
	@Deprecated _RunIDEntry(final String runID) {super(_KEY, runID);}
}

//=============================================================================

final class _LifeCycleStatisticTypeEntry extends _StringEntry{
	static final _LifeCycleStatisticTypeEntry APPLICATION = new _LifeCycleStatisticTypeEntry("Application");
	static final _LifeCycleStatisticTypeEntry PANEL       = new _LifeCycleStatisticTypeEntry("Panel");
	static final _LifeCycleStatisticTypeEntry MAINPANEL   = new _LifeCycleStatisticTypeEntry("MainPanel");
	
	static final String _KEY = "LifeCycleStatisticOf";	
	private _LifeCycleStatisticTypeEntry(final String lcsName) {super(_KEY, lcsName);}
}

/** IonGetterPanel, ...*/ 	final class _NameEntry extends _StringEntry{
	static final String _KEY = "Name";		
	_NameEntry(final String name)	{super(_KEY, name);}
}
/** Nr. 1, ...*/ 			final class _LongTagEntry extends _LongEntry{
	static final String _KEY = "LongTag";	
	_LongTagEntry(final Long value)	{super(_KEY, value);}
}
/** started, stopped, */ 	final class _EventEntry extends _StringEntry{
	static final _EventEntry STARTED = new _EventEntry("started");
	static final _EventEntry STOPPED = new _EventEntry("stopped");
	
	static final String _KEY = "Event";		
	private _EventEntry(final String name)	{super(_KEY, name);}

}
/** for Exceptions as Strings*/
final class _ExceptionEntry extends _StringEntry{
	static final String _KEY = "Exception";		
	_ExceptionEntry(final String sException)	{super(_KEY, sException);}
}




