/**
 * 	Jul  8, 2010 - Jul 15, 2010	ws:	created from scratch
 */
package de.desy.mst.libs.info.appadmin;

import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.logging.Level;

import javax.swing.SwingUtilities;

import de.desy.mst.libs.info.appadmin.AppAdmin.IUpdateCallBack;

/**
 * this is a pure multicaster into the EventDispatchThread for IUpdateCallBack
 * <br> 
 * getReceiver().onNewUpdate(name) will be sent in EDT to all registered cbs
 * @author ws
 */
class _IUpdateCallBackEDTMulticaster{
	private final List<IUpdateCallBack> cbLst;
	private final IUpdateCallBack multicastReceiver = new IUpdateCallBack() {
		@Override public void onMessage(final String name) {
			final Runnable runnable = new Runnable() {
				@Override public void run() {
					for (final IUpdateCallBack cb : getCbLst())
						try{cb.onMessage(name);}
					catch(final Exception ex){handleException(ex);}
				}
				private void handleException(final Exception ex) {
					_Logging.log(Level.SEVERE, "Error in IUpdateCallBack.", ex);
				}
			};
			SwingUtilities.invokeLater(runnable);
		}
	};
	//-------------------------------------------------------------------------
	_IUpdateCallBackEDTMulticaster() {
		this.cbLst = new CopyOnWriteArrayList<IUpdateCallBack>();
	}
	//-------------------------------------------------------------------------
	/**register for update notification, non null*/
	final boolean register(final IUpdateCallBack cb) throws NullPointerException{
		__Utils.checkForNull(cb, "IUpdateCallBack");
		return getCbLst().add(cb);
	}
	/**unregister for update notification*/
	final boolean unregister(final IUpdateCallBack cb){
		return getCbLst().remove(cb);
	}
	final boolean isRegistered(final Object cb){
		return getCbLst().contains(cb);
	}
	
	final IUpdateCallBack getMulticastReceiver(){return this.multicastReceiver;}
	//-------------------------------------------------------------------------
	@Deprecated List<IUpdateCallBack> getCbLst() {return this.cbLst;}
}
