/**
 * 
 */
package de.desy.mst.libs.pintlk.badgerviewer.config;


import de.desy.mst.libs.framework.mstapp.mode.ServerAnwahl;
import de.desy.mst.libs.pintlk.badgerviewer.network.Device;
import de.desy.mst.libs.pintlk.badgerviewer.network.IDevice;

/**
 * @author labudda
 * Liste der bekannten Standorte der Klienten.
 * An jedem Standort koennen  / werden andere Daten angezeigt.
 * Die Unterschiedlichen Daten werden von unterschiedlichen Servern 
 * und unterschiedlichen Deviceses zur Verfuegung gestellt 
 */
public enum EClientConfig {
	// HamburgBkr("Common", new String[]{"ZZonDO_1", "ZZonDO_2", "ZZonDO_3"}, new PositiveHostnameDetection("BKRXPCSCON0.", java.util.regex.Pattern.CASE_INSENSITIVE)),
	// HamburgBkr("Common", new String[]{"ZZonDO_1", "ZZonDO_2"}, new PositiveHostnameDetection("BKRXPCSCON0.|MCSW7LABUDDA", java.util.regex.Pattern.CASE_INSENSITIVE)),
	HamburgBkr("Common", new String[]{"LINAC2", "DESY2", "PETRA"}, new PositiveHostnameDetection("BKRW7CSCON.|MCSW7LABUDDA", java.util.regex.Pattern.CASE_INSENSITIVE)),
	
	Debug("Common", new String[]{"Alle"}, new PositiveHostnameDetection("MCSW7LABUDDA", java.util.regex.Pattern.CASE_INSENSITIVE)),
	Invalid(null, new String[0], new ConfigDetector());

	private final String context;
	private final String[] devNames;
	private final ConfigDetector detection;
	private EClientConfig(final String context, final String[] devNames, final ConfigDetector detection) {
		this.context = context;
		this.devNames = devNames;
		this.detection = detection;
	}
	
	/**
	 * @return Konfiguration defined by environment
	 */
	public static EClientConfig getDefinedConfig(){
		for (EClientConfig check : EClientConfig.values()) {
			if (check.detection.isAcceptable()) return check;
		}
		return EClientConfig.Invalid;
	}
	/**
	 * @param anwahl des Servertyps ( Produktiv oder Simulation)
	 * @return Liste von Geraeten dieser Konfiguration
	 */
	public java.util.Collection<IDevice>createDevices(ServerAnwahl anwahl) {
		final String context;
		if (anwahl.equals(ServerAnwahl.DEFAULT)) {
			context = this.context;
		} else {
			context = this.context + ".SIM";
		}
		final java.util.Collection<IDevice> devices = new java.util.Vector<IDevice>(this.devNames.length);
		for (String deviceName :this.devNames) {
			devices.add(new Device(context, deviceName));
		}
		return devices;
	}

}
