/**
 * 
 * Last changed: $Date$
 * By: $Author$
 * URL: $HeadURL$
 * $Revision$ 
 *
 */
package de.desy.mst.libs.pintlk.badgerviewer.network;

import de.desy.mst.libs.ser.tclientlib.TLinkAddress;
import de.desy.mst.libs.ser.tclientlib.TineLinkException;
import de.desy.mst.libs.ser.tclientlib.readers.IntReader;
import de.desy.mst.libs.ser.tclientlib.readers.NAME64DBLDBLArrayReader;
import de.desy.mst.libs.ser.tclientlib.readers.NAME64Reader;
import de.desy.mst.libs.ser.tclientlib.readers.TextReader;
import de.desy.mst.libs.ser.tclientlib.utils.TineQueryUtils;
import de.desy.tine.definitions.TErrorList;
import de.desy.tine.types.NAME64DBLDBL;

/**
 * @author labudda
 *
 */
public class Device implements IDevice {

	private final static String SERVERNAME = "BadgersSet"; 

	private final static String P_ACCCELERATOR = "DisplayedAcceler"; 
	private final static String P_NAME = "AcquisitionPosting"; 
	private final static String P_COUNT = "ItemCount";
	private final static String P_MESSAGE = "SystemMessage";

	private final NAME64Reader acceleratorName;
	private final NAME64DBLDBLArrayReader acquisitionPosting;
	private final TextReader dachsAdminMessage;
	private final IntReader count;
	private final TineLinkException initialException;

	public Device(final String context, final String deviceName) {
		final String addressFragment = "/" + context + "/" + Device.SERVERNAME + "/" + deviceName + "/";
		TineLinkException exception;
		this.count = new IntReader(addressFragment + Device.P_COUNT);  
		this.acceleratorName = new NAME64Reader(addressFragment + Device.P_ACCCELERATOR);
		NAME64DBLDBLArrayReader acquisitionPosting;
		TextReader dachsAdminMessage;
		try {
			this.acceleratorName.setAsychronous(1000);
			acquisitionPosting = new NAME64DBLDBLArrayReader(addressFragment + Device.P_NAME);
			acquisitionPosting.setAsychronous(1000);
			this.count.setAsychronous(1000);
			final TLinkAddress textAddress = new TLinkAddress(addressFragment + Device.P_MESSAGE);
			final int textSize = TineQueryUtils.getInstance().getPropertyArraySize(textAddress);
			if (textSize > 0) {
				dachsAdminMessage = new TextReader(textAddress, textSize); 
				dachsAdminMessage.setAsychronous(1000);
			} else {
				dachsAdminMessage = null;
			}
			exception = null;
		} catch (TineLinkException tle) {
			exception = new TineLinkException(new TLinkAddress(addressFragment), TErrorList.device_error," Fehler bei der Initialisierung des Links", tle);
			acquisitionPosting = null;
			dachsAdminMessage = null;
			assert false:"Unable to initialise tine link";
		}
		this.acquisitionPosting = acquisitionPosting;
		this.dachsAdminMessage = dachsAdminMessage;
		this.initialException = exception;
	}

	
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.pintlk.badgerviewer.network.IDevice#getNameOfDoor()
	 */
	public String getNameOfAccelerator() throws TineLinkException{
		if (this.initialException == null) {
			return this.acceleratorName.getValue().getName();
		} else {
			throw this.initialException;
		}
	}
	
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.pintlk.badgerviewer.network.IDevice#getEntries()
	 */
	public java.util.AbstractList<DachsEntry> getEntries() throws TineLinkException{
		if (this.initialException == null) {
			// Ansich sollten die Arrays alle eine identische Groesse besitzen. Auch im Fehlerfall?
			final java.util.ArrayList<NAME64DBLDBL> received = this.acquisitionPosting.getValueList(); 
			final int countNames;
			// Im Fall der Debug view / des simulationsservers ist ein ArayIndex OutOfBounds Fehler aufgetreten
			// TODO: Warum unterscheidet sich this.acquisitionPosting.getValueList().size() / this.count.getValue()
			if (received.size() < this.count.getValue()) {
				countNames = received.size();
			} else {
				countNames = this.count.getValue();
			}
			final java.util.AbstractList<DachsEntry> entries = new java.util.ArrayList<DachsEntry>(countNames);
			for (int index = 0; index < countNames; index++) {
				final NAME64DBLDBL receivedItem = received.get(index);
				entries.add(new DachsEntry(
					receivedItem.getName(),
					receivedItem.d1val,
					receivedItem.d2val
				));
				
			}
			return entries;
		} else {
			throw this.initialException;
		}
	}


	/* (non-Javadoc)
	 * @see de.desy.mst.libs.pintlk.badgerviewer.network.IDevice#terminate()
	 */
	@Override
	public void terminate() {
		this.acceleratorName.close();
		this.acquisitionPosting.close();
		this.dachsAdminMessage.close();
		this.count.close();
	}


	/* (non-Javadoc)
	 * @see de.desy.mst.libs.pintlk.badgerviewer.network.IDevice#getDeviceName()
	 */
	@Override
	public String getDeviceName() {
		return this.acquisitionPosting.getLinkAddress().getDevice();
	}


	/* (non-Javadoc)
	 * @see de.desy.mst.libs.pintlk.badgerviewer.network.IDevice#getMessage()
	 */
	@Override
	public String getMessage() throws TineLinkException {
		if (this.initialException == null) {
			return this.dachsAdminMessage.getValue();
		} else {
			throw this.initialException;
		}
	}


}
