package de.desy.mst.libs.ser.tclientlib.readers;
import de.desy.tine.dataUtils.TDataType;
import de.desy.mst.libs.ser.tclientlib.ATineReader;
import de.desy.mst.libs.ser.tclientlib.TineLinkException;
import de.desy.mst.libs.ser.tclientlib.TLinkAddress;
import de.desy.tine.types.DBLDBLDBL;


/**
 * A DBLDBLDBL reader.
 *
 * The DBLDBLDBLReader reads a scalar value of type DBLDBLDBL.
 *
 */
public class DBLDBLDBLReader extends ATineReader {
    
    private static final int ARRAY_SIZE = 1;
    private DBLDBLDBL[] inputBuffer = new DBLDBLDBL[ARRAY_SIZE]; // Data buffer for TINE
    private DBLDBLDBL[] outputBuffer = new DBLDBLDBL[ARRAY_SIZE]; // Data buffer for TINE

    /**
     * Constructor.
     * @param linkAddress The TINE address
     */
    public DBLDBLDBLReader(TLinkAddress linkAddress) {
        super(linkAddress);
        createBuffers();
        setup(createTDataType());
    }

    
    /**
     * Constructor.
     * @param fullDevicePath The full TINE path of
     * the property. It must be in the form 
     * <code>
     * /context/server/device/property
     * </code>.
     */
    public DBLDBLDBLReader(String fullDevicePath) {
        super(fullDevicePath);
        createBuffers();
        setup(createTDataType());
    }

    /**
     * Returns the value.
     * If the reader is in asynchronous mode (attached), 
     * the last updated values are returned. If the reader
     * in in synchronous mode, a synchronous read operation
     * (execute) is performed.
     * In both cases, when an error occurred, no valid data
     * are available, no value is returned,
     * and a TineLinkException is thrown instead.
     * @throws TineLinkException
     */
    public synchronized DBLDBLDBL getValue() throws TineLinkException {
        if (isSynchronous()) {
            readData();
            copyData();
        }
        if (hasError()) {
            throw new TineLinkException(getLinkAddress(), getLinkStatus(), getErrorMessage());
        } 
        return outputBuffer[0];
    }
    
    
    /**
     * Copies the input array to the output array.
     * This method is called when the reader is in asynchronous mode.
     */
     protected synchronized void copyData() {
           outputBuffer[0] = new DBLDBLDBL(inputBuffer[0]);
    }
 

    private void createBuffers() {
        inputBuffer[0] = new DBLDBLDBL();
        copyData();
    }
    
    private TDataType createTDataType() {
        TDataType result = null;
        result = new TDataType(inputBuffer);
        return result;
    }
    
}
