/*
 * (c) by DESY 2016 - Group MCS 1, Hamburg, Germany
 * 
 * For more information about the license you should have a look at the
 * following web page:
 * 
 *         http://www.desy.de/legal/license/index_eng.html
 * 
 * If no concrete license can be found at this web page or the connection is
 * broken, then the following license is relevant:
 * 
 * Copyright (C) 2016 DESY.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301  USA
 */
package de.desy.mst.libs.util.helper.ui.version;

import de.desy.mst.libs.util.helper.LogProvider;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Font;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * DESY Icon and providing by a click to open the DESY web page in the defined
 * favorite browser.
 * 
 * @author <a href="marcus.walla(at)desy.de">Marcus Walla</a>
 * @version $Id: VersionDesyIconPanel.java 73820 2016-11-15 19:29:13Z walla $
 */
public class VersionDesyIconPanel extends javax.swing.JPanel
{
    private final static Logger log = LogProvider.getLogger(VersionDesyIconPanel.class, Level.INFO, true, LogProvider.ELoggingProperty.MCS_LOGGING);
    
    private final static String HTTP_DESY = "http://www.desy.de";
    private final static Color LIGHT_BLUE = new Color(110,170,240);

    /**
     * Creates new form VersionDesyIconPanel
     */
    public VersionDesyIconPanel()
    {
        initComponents();
        initialize();
        lblDesy.setFont(new Font(null, Font.PLAIN, lblDesy.getFont().getSize()));
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents()
    {

        lblDesyIcon = new javax.swing.JLabel();
        lblDesy = new javax.swing.JLabel();

        lblDesyIcon.setIcon(new javax.swing.ImageIcon(getClass().getResource("/resources/images/DESY-Logo-cyan.jpg"))); // NOI18N
        lblDesyIcon.setBorder(javax.swing.BorderFactory.createCompoundBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)), javax.swing.BorderFactory.createEmptyBorder(1, 1, 1, 1)));

        lblDesy.setForeground(LIGHT_BLUE);
        lblDesy.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        lblDesy.setText(HTTP_DESY);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(lblDesy, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addComponent(lblDesyIcon)
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(lblDesyIcon)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(lblDesy))
        );
    }// </editor-fold>//GEN-END:initComponents

    private void initialize()
    {
        lblDesyIcon.setToolTipText("Opens the web page " + HTTP_DESY + " at your favorite browser.");
        lblDesyIcon.addMouseListener(new MouseAdapter()
        {
            @Override
            public void mouseEntered(MouseEvent event)
            {
                lblDesyIcon.setCursor(new Cursor(Cursor.HAND_CURSOR));
            }
            
            @Override
            public void mouseExited(MouseEvent event)
            {
                lblDesyIcon.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
            }
            
            @Override
            public void mouseReleased(final MouseEvent event)
            {
                openDesyWebPage();
            }
        });
        
        lblDesy.setToolTipText("Opens the web page " + HTTP_DESY + " at your favorite browser.");
        lblDesy.addMouseListener(new MouseAdapter()
        {
            @Override
            public void mouseEntered(MouseEvent event)
            {
                lblDesy.setFont(new Font(null, Font.BOLD, lblDesy.getFont().getSize()));
                lblDesy.setCursor(new Cursor(Cursor.HAND_CURSOR));
            }
            
            @Override
            public void mouseExited(MouseEvent event)
            {
                lblDesy.setFont(new Font(null, Font.PLAIN, lblDesy.getFont().getSize()));
                lblDesy.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
            }
            
            @Override
            public void mouseReleased(final MouseEvent event)
            {
                openDesyWebPage();
            }
        }) ;
    }
    
    private void openDesyWebPage()
    {
        try 
        {
            java.awt.Desktop.getDesktop().browse(new URI(HTTP_DESY));
        }
        catch (URISyntaxException ex) 
        {
            log.log(Level.SEVERE, "unable to open the web page " + HTTP_DESY, ex);
            lblDesyIcon.setToolTipText("Unable to open the web page " + HTTP_DESY);
            lblDesy.setToolTipText("Unable to open the web page " + HTTP_DESY);
        }
        catch (IOException ex) 
        {
            log.log(Level.SEVERE, "unable to open the web page", ex);
            lblDesyIcon.setToolTipText("Unable to open the web page " + HTTP_DESY);
            lblDesy.setToolTipText("Unable to open the web page " + HTTP_DESY);
        }
        catch(UnsupportedOperationException ex)
        {
            log.log(Level.SEVERE, "class java.awt.Desktop is not supported", ex);
            lblDesyIcon.setToolTipText("Class java.awt.Desktop is not supported!");
            lblDesy.setToolTipText("Class java.awt.Desktop is not supported!");
        }
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel lblDesy;
    private javax.swing.JLabel lblDesyIcon;
    // End of variables declaration//GEN-END:variables
}
/* end of file */